Package: multiWGCNA
Type: Package
Title: multiWGCNA
Version: 1.10.0
Authors@R: c(
    person("Dario", "Tommasini", email="dtommasini0@gmail.com",
      role = c("aut","cre"),
      comment = c(ORCID = "0000-0002-1214-6547")),
    person("Brent", "Fogel", role = c("aut","ctb")))
Description: An R package for deeping mining gene co-expression networks in
    multi-trait expression data. Provides functions for analyzing, comparing,
    and visualizing WGCNA networks across conditions. multiWGCNA was designed
    to handle the common case where there are multiple biologically meaningful
    sample traits, such as disease vs wildtype across development or anatomical
    region.
License: GPL-3
Encoding: UTF-8
LazyData: true
Suggests: BiocStyle, doParallel, ExperimentHub, knitr, markdown,
        rmarkdown, testthat (>= 3.0.0), vegan
VignetteBuilder: knitr
RoxygenNote: 7.3.2
biocViews: Sequencing, RNASeq, GeneExpression, DifferentialExpression,
        Regression, Clustering
Imports: stringr, readr, WGCNA, magrittr, dplyr, reshape2, data.table,
        patchwork, scales, igraph, flashClust, ggplot2, dcanr, cowplot,
        ggrepel, methods, SummarizedExperiment, ggraph, tidyr
Depends: R (>= 4.3.0), ggalluvial
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/multiWGCNA
git_branch: RELEASE_3_23
git_last_commit: 3076bc8
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:34:39 UTC; biocbuild
Author: Dario Tommasini [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1214-6547>),
  Brent Fogel [aut, ctb]
Maintainer: Dario Tommasini <dtommasini0@gmail.com>

# Generated by roxygen2: do not edit by hand

export(GetDatExpr)
export(PreservationPermutationTest)
export(PreservationScoreDistribution)
export(TOMFlowPlot)
export(bidirectionalBestMatches)
export(cleanDatExpr)
export(coexpressionLineGraph)
export(computeOverlapsFromWGCNA)
export(constructNetworks)
export(diffCoexpression)
export(diffModuleExpression)
export(drawMultiWGCNAnetwork)
export(getPreservation)
export(iterate)
export(makeTraitTable)
export(moduleComparisonPlot)
export(moduleExpressionPlot)
export(moduleToModuleHeatmap)
export(name)
export(overlapComparisons)
export(performANOVA)
export(preservationComparisonPlot)
export(preservationComparisons)
export(runDME)
export(summarizeResults)
export(topNGenes)
import(SummarizedExperiment)
import(WGCNA)
import(dcanr)
import(dplyr)
import(flashClust)
import(ggalluvial)
import(ggplot2)
import(ggrepel)
import(methods)
import(patchwork)
import(readr)
import(stringr)
importFrom(cowplot,plot_grid)
importFrom(data.table,fwrite)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(grDevices,rgb)
importFrom(graphics,legend)
importFrom(igraph,"E<-")
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,degree)
importFrom(igraph,delete.edges)
importFrom(igraph,delete.vertices)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,plot.igraph)
importFrom(igraph,simplify)
importFrom(scales,rescale)
importFrom(stats,IQR)
importFrom(stats,anova)
importFrom(stats,dist)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,phyper)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,t.test)
importFrom(stats,var)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,write.csv)
importFrom(utils,write.table)

# multiWGCNA: an R package for deep mining gene co-expression networks in multi-trait expression data

The multiWGCNA R package builds on the existing weighted gene co-expression 
network analysis (WGCNA) package by extending workflows to expression data with 
two dimensions. multiWGCNA is especially useful for the study of 
disease-associated modules across time or space. For more information, please 
see the multiWGCNA paper available at https://doi.org/10.1186/s12859-023-05233-z. 

# Installation 

The multiWGCNA R package can be installed from Bioconductor like this: 
```
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("multiWGCNA")
```

The development version of multiWGCNA can be installed from GitHub like this:
```
if (!require("devtools", quietly = TRUE))
    install.packages("devtools")
    
devtools::install_github("fogellab/multiWGCNA")
```

# Vignettes

We recommend running through both of the vignettes before applying multiWGCNA to your own data:

* The autism_full_workflow.Rmd vignette provides a quick example of how to use multiWGCNA.
* The astrocyte_map2.Rmd vignette provides a more in-depth tutorial discussing the preservation analyses from the manuscript and functions for visualization. 

# Citation

To cite multiWGCNA in publications, please use:

  Tommasini, D., Fogel, B.L. (2023). multiWGCNA: an R package for deep mining gene
  co-expression networks in multi-trait expression data. BMC Bioinformatics, 24,
  1:115.

For LaTeX users, a BibTeX entry is available here: 

```
  @Article{,
    title = {multiWGCNA: an R package for deep mining gene co-expression 
    networks in multi-trait expression data},
    author = {Dario Tommasini and Brent L. Fogel},
    journal = {BMC Bioinformatics},
    year = {2023},
    volume = {24},
    number = {1},
    pages = {115},
  }
```

Please also make sure to cite the original WGCNA package: 

  Langfelder, P., Horvath, S. (2008). WGCNA: an R package for weighted correlation network 
  analysis. BMC Bioinformatics 9, 559. 

```
@ARTICLE{,
  title     = "{WGCNA}: an {R} package for weighted correlation network
               analysis",
  author    = "Langfelder, Peter and Horvath, Steve",
  journal   = "BMC Bioinformatics",
  volume    =  9,
  number    =  1,
  pages     = "559",
  month     =  dec,
  year      =  2008,
}
```

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----eval=FALSE---------------------------------------------------------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("multiWGCNA")

## ----eval=FALSE---------------------------------------------------------------
# if (!require("devtools", quietly = TRUE))
#     install.packages("devtools")
# 
# devtools::install_github("fogellab/multiWGCNA")

## -----------------------------------------------------------------------------
library(multiWGCNA)

## -----------------------------------------------------------------------------
# Download data from the ExperimentHub
library(ExperimentHub)
eh = ExperimentHub()

# Note: this requires the SummarizedExperiment package to be installed
eh_query = query(eh, c("multiWGCNAdata"))
autism_se = eh_query[["EH8219"]]

# Collect the metadata in the sampleTable
sampleTable = colData(autism_se)

# Randomly sample 1500 genes from the expression matrix
set.seed(1)
autism_se = autism_se[sample(rownames(autism_se), 1500),]

# Check the data
assays(autism_se)[[1]][1:5, 1:5]
# sampleTable$Status = paste0("test_", sampleTable$Status, "_test")
# sampleTable$Sample = paste0(sampleTable$Sample, "_test")
sampleTable

# Define our conditions for trait 1 (disease) and 2 (brain region)
conditions1 = unique(sampleTable[,2])
conditions2 = unique(sampleTable[,3])

## -----------------------------------------------------------------------------
# Construct the combined networks and all the sub-networks (autism only, controls only, FC only, and TC only)
autism_networks = constructNetworks(autism_se, sampleTable, conditions1, conditions2,
                                  networkType = "unsigned", power = 10,
                                  minModuleSize = 40, maxBlockSize = 25000,
                                  reassignThreshold = 0, minKMEtoStay = 0.7,
                                  mergeCutHeight = 0.10, numericLabels = TRUE,
                                  pamRespectsDendro = FALSE, verbose=3,
                                  saveTOMs = TRUE)

## ----fig.height = 4, fig.width = 7--------------------------------------------
# Save results to a list
results=list()
results$overlaps=iterate(autism_networks, overlapComparisons, plot=TRUE)

# Check the reciprocal best matches between the autism and control networks
head(results$overlaps$autism_vs_controls$bestMatches)

## ----fig.height=6, fig.width=4------------------------------------------------
networks = c("autism", "controls")
toms = lapply(networks, function(x) {
  load(paste0(x, '-block.1.RData'))
  get("TOM")
})

# Check module autism_005
TOMFlowPlot(autism_networks, 
            networks, 
            toms, 
            genes_to_label = topNGenes(autism_networks$autism, "autism_005"), 
            color = 'black', 
            alpha = 0.1, 
            width = 0.05)

## ----fig.height = 6, fig.width = 7--------------------------------------------
# Run differential module expression analysis (DME) on combined networks
results$diffModExp = runDME(autism_networks[["combined"]], 
                            sampleTable, 
                            p.adjust="fdr", 
                            refCondition="Tissue", 
                            testCondition="Status") 
                            # plot=TRUE, 
                            # out="combined_DME.pdf")

# to run PERMANNOVA
# library(vegan)
# results$diffModExp = runDME(autism_networks[["combined"]], p.adjust="fdr", refCondition="Tissue", 
#                          testCondition="Status", plot=TRUE, test="PERMANOVA", out="PERMANOVA_DME.pdf")

# Check adjusted p-values for the two sample traits
results$diffModExp

# You can check the expression of a specific module like this. Note that the values reported in the bottom panel title are p-values and not FDR-adjusted like in results$diffModExp
diffModuleExpression(autism_networks[["combined"]], 
                     geneList = topNGenes(autism_networks[["combined"]], "combined_004"), 
                     design = sampleTable,
                     test = "ANOVA",
                     plotTitle = "combined_004",
                     plot = TRUE)

## ----fig.height = 3, fig.width = 7--------------------------------------------
# To enable multi-threading
# library(doParallel)
# library(WGCNA)
# nCores = 8
# registerDoParallel(cores = nCores)
# enableWGCNAThreads(nThreads = nCores)

# Calculate preservation statistics
results$preservation=iterate(autism_networks[conditions1], # this does autism vs control; change to "conditions2" to perform comparison between FC and TC
                             preservationComparisons, 
                             write=FALSE, 
                             plot=TRUE, 
                             nPermutations=10)

## -----------------------------------------------------------------------------
# Print a summary of the results
summarizeResults(autism_networks, results)

## -----------------------------------------------------------------------------
sessionInfo()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(multiWGCNA)

## -----------------------------------------------------------------------------
# Download data from the ExperimentHub
library(ExperimentHub)
eh = ExperimentHub()

# Note: this requires the SummarizedExperiment package to be installed
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_se = eh_query[["EH8223"]]

# Collect the metadata in the sampleTable; the first column must be named "Sample"
sampleTable = colData(astrocyte_se)

# Check the data
assays(astrocyte_se)[[1]][1:5, 1:5]
sampleTable

# Define our conditions for trait 1 (disease) and 2 (brain region)
conditions1 = unique(sampleTable[,2])
conditions2 = unique(sampleTable[,3])

## ----eval = FALSE-------------------------------------------------------------
# # Construct the combined networks and all the sub-networks (EAE, WT, and each region)
# # Same parameters as Tommasini and Fogel. BMC Bioinformatics
# astrocyte_networks = constructNetworks(astrocyte_se, sampleTable, conditions1, conditions2,
#                                   networkType = "signed", TOMType = "unsigned",
#                                   power = 12, minModuleSize = 100, maxBlockSize = 25000,
#                                   reassignThreshold = 0, minKMEtoStay = 0, mergeCutHeight = 0,
#                                   numericLabels = TRUE, pamRespectsDendro = FALSE,
#                                   deepSplit = 4, verbose = 3)
# 

## -----------------------------------------------------------------------------
# Load pre-computed astrocyte networks
astrocyte_networks = eh_query[["EH8222"]] 

# Check one of the WGCNA objects
astrocyte_networks[["combined"]]

## ----fig.height = 5, fig.width = 8--------------------------------------------
# Save results to a list
results = list()
results$overlaps = iterate(astrocyte_networks, overlapComparisons, plot=FALSE)

# Check the overlaps, ie between the EAE and wildtype networks
head(results$overlaps$EAE_vs_WT$overlap)

## ----fig.height = 6, fig.width = 7--------------------------------------------
# Run differential module expression analysis (DME) on combined networks
results$diffModExp = runDME(astrocyte_networks[["combined"]], 
                            sampleTable,
                            p.adjust = "fdr", 
                            refCondition = "Region", 
                            testCondition = "Disease") 
                            # plot=TRUE, 
                            # out="ANOVA_DME.pdf")

# Check results sorted by disease association FDR
results$diffModExp[order(results$diffModExp$Disease),]

# You can check the expression of module M13 from Tommasini and Fogel. BMC Bioinformatics. 2023 like this. Note that the values reported in the bottom panel title are p-values and not adjusted for multiple comparisons like in results$diffModExp
diffModuleExpression(astrocyte_networks[["combined"]], 
                     geneList = topNGenes(astrocyte_networks[[1]], "combined_013"), 
                     design = sampleTable,
                     test = "ANOVA",
                     plotTitle = "combined_013",
                     plot = TRUE)


## ----fig.height = 6, fig.width = 7--------------------------------------------
drawMultiWGCNAnetwork(astrocyte_networks, 
                      results$overlaps, 
                      "combined_013", 
                      design = sampleTable, 
                      overlapCutoff = 0, 
                      padjCutoff = 1, 
                      removeOutliers = TRUE, 
                      alpha = 1e-50, 
                      layout = NULL, 
                      hjust = 0.4, 
                      vjust = 0.3, 
                      width = 0.5)

## ----fig.height = 8, fig.width = 10-------------------------------------------
bidirectionalBestMatches(results$overlaps$combined_vs_EAE)

## ----fig.height=5, fig.width=7------------------------------------------------
# Get expression data for top 20 genes in EAE_015 module
datExpr = GetDatExpr(astrocyte_networks[[1]], 
                     genes = topNGenes(astrocyte_networks$EAE, "EAE_015", 20))

# Plot
coexpressionLineGraph(datExpr, splitBy = 1.5, fontSize = 2.5) + 
  geom_vline(xintercept = 20.5, linetype='dashed')

## ----eval = FALSE, fig.height = 3, fig.width = 7------------------------------
# # To enable multi-threading
# # library(doParallel)
# # library(WGCNA)
# # nCores = 2
# # registerDoParallel(cores = nCores)
# # enableWGCNAThreads(nThreads = nCores)
# 
# # Turn off multi-threading
# # registerDoSEQ()
# # disableWGCNAThreads()
# 
# # Calculate preservation statistics
# results$preservation=iterate(astrocyte_networks[c("EAE", "WT")],
#                              preservationComparisons,
#                              write=FALSE,
#                              plot=TRUE,
#                              nPermutations=2)

## ----eval = FALSE-------------------------------------------------------------
# options(paged.print = FALSE)
# results$permutation.test = PreservationPermutationTest(astrocyte_networks$combined@datExpr[sample(17000,3000),],
#                                                        sampleTable,
#                                                        constructNetworksIn = "EAE", # Construct networks using EAE samples
#                                                        testPreservationIn = "WT", # Test preservation of disease samples in WT samples
#                                                        nPermutations = 10, # Number of permutations for permutation test
#                                                        nPresPermutations = 10, # Number of permutations for modulePreservation function
# 
#                                                        # WGCNA parameters for re-sampled networks (should be the same as used for network construction)
#                                                        networkType = "signed", TOMType = "unsigned",
#                                                        power = 12, minModuleSize = 100, maxBlockSize = 25000,
#                                                        reassignThreshold = 0, minKMEtoStay = 0, mergeCutHeight = 0,
#                                                        numericLabels = TRUE, pamRespectsDendro = FALSE,
#                                                        deepSplit = 4, verbose = 3
#                                                        )

## ----eval = TRUE--------------------------------------------------------------
# Load pre-computed results
data(permutationTestResults) 

# Remove outlier modules
permutationTestResultsFiltered = lapply(permutationTestResults, function(x) x[!x$is.outlier.module,])

# Extract the preservation score distribution
results$scores.summary = PreservationScoreDistribution(permutationTestResultsFiltered, 
                                                       moduleOfInterestSize = 303 # The size of the module of interest (dM15)
                                                       )

# Observed preservation score of dM15
observed.score = 9.16261490617938

# How many times did we observe a score lower than or equal to this observed score?
z.summary.dist = results$scores.summary$z.summary
below=length(z.summary.dist[z.summary.dist <= observed.score])
probability= below/100
message("Probability of observing a score of ", round(observed.score, 2), " is ", probability)

## ----eval = TRUE--------------------------------------------------------------
# Plot on a histogram
ggplot(results$scores.summary, aes(x=z.summary)) + 
      geom_histogram(color="black", fill="white", bins = 15)+
      xlab("Preservation score")+
      ylab("Frequency")+
      geom_vline(xintercept=observed.score, color="red3", linetype="solid")+
      scale_y_continuous(expand = c(0,0))+
      theme_classic()+
      theme(plot.title = element_text(hjust = 0.5))

## -----------------------------------------------------------------------------
sessionInfo()

#' Plots an expression profile for a module
#'
#' A plotting function that returns a heatmap and barplot for a module
#'
#' @param WGCNAobject an object of class WGCNAobject
#' @param geneList a vector of gene names to be extracted from WGCNAobject
#' @param mode use first principal component or averageZscore?
#' @param legend plot legend?
#' @param title title of the plot
#' @param clusterGenes cluster heatmap genes by hierarchical clustering?
#'
#' @return a patchworked ggplot object
#' 
#' @author Dario Tommasini
#'
#' @import ggplot2
#' @import patchwork
#' @import WGCNA
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]]
#' moduleExpressionPlot(astrocyte_networks[["combined"]], 
#'   geneList = topNGenes(astrocyte_networks$combined, "combined_013"))
#' 
moduleExpressionPlot <- function(WGCNAobject, geneList, mode=c("PC1", "averageZscore"), legend=FALSE, title=NULL, clusterGenes=FALSE){

  # Check that mode is either PC1 or averageZscore
  mode = match.arg(mode)
  
	datExpr=WGCNAobject@datExpr
	design=WGCNAobject@conditions
	heatmap <- expressionHeatmap(datExpr, 
	                             geneList, 
	                             column.labels=FALSE, 
	                             axis.titles=FALSE, 
	                             legend=legend, 
	                             clusterGenes=clusterGenes, 
	                             title=title)
	cleanDatExpr=t(cleanDatExpr(datExpr))
	subset=cleanDatExpr[toupper(rownames(cleanDatExpr)) %in% toupper(geneList),]

	if(mode=="averageZscore"){
		mean=rowMeans(subset)
		stdev=apply(subset, 1, sd)
		zscoreMatrix=(subset-mean)/stdev
		zscoreMatrix=na.omit(zscoreMatrix)
		moduleExpression=reshape2::melt(zscoreMatrix)
		colnames(moduleExpression)=c("Gene", "Sample", "Expression")
		mergedData=cbind(moduleExpression, design[match(moduleExpression$Sample, design$X),-1])
		boxplot <- ggplot(data=mergedData, aes(x=Sample, y=Expression, color=eval(parse(text=colnames(design)[[2]])))) +
    				geom_boxplot() +
    				guides(color=guide_legend(title="Condition")) +
    				{if(!legend) theme(legend.position = "none")} +
    				theme(panel.background = element_blank(), axis.line = element_line(colour = "black"),
    					panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
    					axis.text.x=element_text(angle=45, hjust=1), plot.title=element_text(hjust=0.5))

	return(heatmap / boxplot)
	}

	if(mode=="PC1"){
		PC1=moduleEigengenes(t(subset), colors = rep("Module", nrow(subset)), nPC=1)$eigengenes
		moduleExpression=data.frame(Sample=rownames(PC1), moduleExpression=PC1)
		colnames(moduleExpression)=c("Sample","moduleExpression")
		mergedData=cbind(moduleExpression, design[match(moduleExpression$Sample, design$X),-1])
		bargraph <- ggplot(data=moduleExpression, aes(x=factor(Sample, levels=Sample), y=moduleExpression))+
					ylab(mode) +
					geom_bar(aes(fill= moduleExpression), stat="identity", color="black", position=position_dodge(9)) +
	       			scale_fill_gradient2(name=mode, low = "blue", mid="white", high = "red", midpoint=0) +
	     		  	theme(axis.ticks.x=element_blank(), axis.text.x=element_text(angle=90, hjust=1),
	     		  	panel.background = element_blank(), axis.line.y = element_line(colour = "black"),
	       			plot.title=element_text(hjust=0.5), panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
	       			axis.title.x=element_blank()) +
	       			{if(!legend) theme(legend.position = "none")} +
	      			geom_hline(yintercept=0, linetype="solid", color = "black", size=0.5)
	return(heatmap / bargraph)
	}
}

# heatmap of z-scored expression for a given expression data.frame and list of genes
expressionHeatmap <- function(datExpr, 
                              geneList, 
                              lower=-2, 
                              upper=2, 
                              design=NULL, 
                              column.labels=TRUE, 
                              row.labels=FALSE, 
                              legend=FALSE, 
                              axis.titles=TRUE, 
                              plotTitle=NULL, 
                              clusterGenes=FALSE, 
                              title=NULL){
	datExpr=t(cleanDatExpr(datExpr))
	subset=datExpr[toupper(rownames(datExpr)) %in% toupper(geneList),]
	subset=subset[match(toupper(geneList), toupper(rownames(subset))),]
	subset=na.omit(subset)
	mean=rowMeans(subset)
	stdev=apply(subset, 1, sd)
	zscoreMatrix=(subset-mean)/stdev
	zscoreMatrix=na.omit(zscoreMatrix)
	zscoreMatrix[zscoreMatrix<lower]=lower
	zscoreMatrix[zscoreMatrix>upper]=upper
	collapsedMatrix=reshape2::melt(zscoreMatrix)
	colnames(collapsedMatrix)=c("Gene", "Sample", "Zscore")
	if(clusterGenes) {
		tree=hclust(dist(zscoreMatrix), method = "average")
		collapsedMatrix$Gene=factor(collapsedMatrix$Gene, levels=rev(rownames(zscoreMatrix)[tree$order]))
	}
	plt = ggplot(collapsedMatrix, aes(x = Sample, y = Gene, fill = Zscore)) +
				geom_tile() +
				ylab("Gene") +
				{if(!is.null(plotTitle)) ggtitle(plotTitle)}+
				theme_classic()+
				scale_fill_gradient2(low = "blue", mid="white", high = "red", na.value="white", limits=c(lower, upper)) +
				{if(!is.null(title)) ggtitle(title)} +
				{if(!axis.titles) theme(axis.title=element_blank())} +
				{if(!column.labels) theme(axis.text.x = element_blank()) else theme(axis.text.x = element_text(angle = 90, vjust=(0.5)))} +
				{if(!row.labels) theme(axis.text.y = element_blank())} +
				{if(!legend) theme(legend.position = "none")} +
				theme(axis.ticks=element_blank(), axis.line=element_blank(), plot.title=element_text(hjust=0.5))
	
	return(plt)
}

#' Run differential module expression
#'
#' A wrapper to run diffModuleExpression on all the modules in a network
#'
#' @param WGCNAobject object of class WGCNA with the modules to run DME on
#' @param design the sampleTable
#' @param alphaLevel level of significance
#' @param testCondition the column of the sampleTable to be resolved
#' @param refCondition the column of the sampleTable to be used as biological variation
#' @param test statistical test to perform, either "ANOVA" or "PERMANOVA"
#' @param p.adjust adjust for multiple comparisons, argument to pass to p.adjust function
#' @param plot generate a plot?
#' @param write write results to a file?
#' @param out file name for DME plots, only used if write is TRUE
#'
#' @return a data.frame summarizing the results of the analysis
#'
#' @author Dario Tommasini
#'
#' @importFrom data.table fwrite
#'
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_se = eh_query[["EH8223"]]
#' sampleTable = colData(astrocyte_se)
#' astrocyte_networks = eh_query[["EH8222"]]
#' runDME(astrocyte_networks[["combined"]], 
#'   design = sampleTable,
#'   p.adjust = "fdr", 
#'   refCondition = "Region", 
#'   testCondition = "Disease") 
#' 
runDME <- function(WGCNAobject, design, alphaLevel=0.05, testCondition=NULL, refCondition=NULL, p.adjust="fdr", plot=FALSE, test=c("ANOVA", "PERMANOVA"), write=FALSE, out=NULL){
	
  # Check argument
  test = match.arg(test)
  
  datExpr=WGCNAobject@datExpr
	if(is.null(refCondition)) refCondition=colnames(design)[[3]]
	if(is.null(testCondition)) testCondition=colnames(design)[[2]]
	modules=sort(unique(datExpr$dynamicLabels))
	modulePrefix=name(WGCNAobject)
	pval.dfs=list()
	if(plot) {
	  if(is.null(out)) pdf(paste0(modulePrefix,"_DME.pdf")) else pdf(paste0(out))
	}
	element=1
	for(module in modules){
		moduleGenes=datExpr$X[datExpr$dynamicLabels==module]
		pval.dfs[[element]]=diffModuleExpression(WGCNAobject, moduleGenes, design, plotTitle=module, plot=plot, test=test)
		colnames(pval.dfs[[element]])=c("Factors", module)
		element=element+1
	}
	if(plot) dev.off()
	mergedPval=Reduce(function(dtf1, dtf2) merge(dtf1, dtf2, by = "Factors", all.x = TRUE), pval.dfs)
	mergedPadj=as.matrix(rbind(p.adjust(mergedPval[1,-1], method=p.adjust),
			  	p.adjust(mergedPval[2,-1], method=p.adjust),
			  	p.adjust(mergedPval[3,-1], method=p.adjust)))
	rownames(mergedPadj)=mergedPval$Factors
	colnames(mergedPadj)=colnames(mergedPval[,-1])
	results.df=as.data.frame(t(mergedPadj))
	if(write) data.table::fwrite(results.df, paste0(modulePrefix, "/", modulePrefix, "_diffModExp.txt"), sep="\t", row.names=TRUE)

	return(results.df)
}

#' Differential module expression
#'
#' Runs (and plots) the differential module expression analysis
#'
#' @param WGCNAobject WGCNA object
#' @param geneList vector of genes in WGCNAobject
#' @param design the sampleTable
#' @param plotTitle title for the plot
#' @param mode either PC1 or Zscore, default is PC1
#' @param testColumn the column of the sampleTable to be resolved
#' @param refColumn the column of the sampleTable to be used as biological variation
#' @param test statistical test to perform, either "ANOVA" or "PERMANOVA"
#' @param plot generate a plot?
#' 
#' @return a data.frame with the resulting p-values
#'
#' @import patchwork
#' @import ggplot2
#' @import WGCNA
#' @importFrom magrittr %>%
#' @importFrom dplyr arrange
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_se = eh_query[["EH8223"]]
#' sampleTable = colData(astrocyte_se)
#' astrocyte_networks = eh_query[["EH8222"]]
#' diffModuleExpression(astrocyte_networks[["combined"]], 
#'   topNGenes(astrocyte_networks$combined, "combined_013"), 
#'   sampleTable,
#'   test = "ANOVA",
#'   plotTitle = "combined_013",
#'   plot = TRUE)
#' 
diffModuleExpression <- function(WGCNAobject, 
                                 geneList, 
                                 design, 
                                 plotTitle=NULL, 
                                 mode=c("PC1", "Zscore"), 
                                 testColumn=2, 
                                 refColumn=3, 
                                 test=c("ANOVA", "PERMANOVA"), 
                                 plot=TRUE){
  
  # Check arguments
  mode = match.arg(mode)
  test = match.arg(test)

  datExpr=WGCNAobject@datExpr
  
	refCondition=colnames(design)[[refColumn]]
	testCondition=colnames(design)[[testColumn]]

	cleanDatExpr=t(cleanDatExpr(datExpr))
	geneList=geneList[geneList %in% rownames(cleanDatExpr)]
	subset=cleanDatExpr[rownames(cleanDatExpr) %in% geneList,]
	if(mode=="Zscore"){
		mean=rowMeans(subset)
		stdev=apply(subset,1,sd)
		zscoreMatrix=(subset-mean)/stdev
		zscoreMatrix=na.omit(zscoreMatrix)
		averageExpression=apply(zscoreMatrix, 2, mean)
		moduleExpression=data.frame(Sample=names(averageExpression), 
		                            moduleExpression=averageExpression)
	}

	if(mode=="PC1"){
		PC1=moduleEigengenes(t(subset), colors = rep("Module", length(geneList)), nPC=1)$eigengenes
		moduleExpression=data.frame(Sample=rownames(PC1), moduleExpression=PC1)
		colnames(moduleExpression)=c("Sample", "moduleExpression")
	}

	mergedData=cbind(moduleExpression, design[match(moduleExpression$Sample, design$Sample),-1])
	mergedData=mergedData %>% arrange(eval(parse(text=testCondition)), eval(parse(text=refCondition)))
	bargraph <- ggplot(data=mergedData, aes(x=factor(Sample, levels=Sample), y=moduleExpression))+
				ylab(mode) +
				geom_bar(aes(fill= moduleExpression), stat="identity", color="black", position=position_dodge(9)) +
	       			scale_fill_gradient2(name=mode, low = "blue", mid="white", high = "red", midpoint=0) +
	       			scale_x_discrete(labels=tolower(paste0(substr(mergedData[,3],0,3),"_",substr(mergedData[,4],0,3)))) +
	     		  	theme(axis.ticks.x=element_blank(), axis.text.x=element_text(angle=90, hjust=1),
	     		  	panel.background = element_blank(), axis.line.y = element_line(colour = "black"),
	       			plot.title=element_text(hjust=0.5), panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
	       			axis.title.x=element_blank()) +
	      			geom_hline(yintercept=0, linetype="solid", color = "black", size=0.5)

	datExpr=datExpr[,c("X", mergedData$Sample)]
	heatmap <- expressionHeatmap(datExpr, geneList, plotTitle=plotTitle, column.labels=FALSE, axis.titles=FALSE, legend=TRUE)

	if(test=="ANOVA"){
		pval.df <- performANOVA(moduleExpression, design, testCondition, refCondition) #category1=test, category2=ref)
	}
	
	if(test=="PERMANOVA"){
	  requireNamespace("vegan", quietly = TRUE)
	  permanova=vegan::adonis(t(subset) ~ design[[testCondition]] + design[[refCondition]] + design[[testCondition]]*design[[refCondition]], method = "euclidean", permutations = 9999)
	  Factors=c(testCondition, refCondition, paste0(testCondition, "*", refCondition))
	  p.value=c(permanova$aov.tab$`Pr(>F)`[1:3])
	  pval.df=data.frame(Factors, p.value)
	}

	boxplot <- ggplot(data=mergedData, aes(x=eval(parse(text=refCondition)), y=moduleExpression, color=eval(parse(text=testCondition)))) +
    				labs(title=paste0("p: ", paste(pval.df$Factors, signif(pval.df$p.value,2), sep = '=', collapse = ', ')),
   					y="Expression", x=refCondition) +
    				geom_boxplot(width=1/length(unique(mergedData[,refCondition]))) +
    				scale_color_manual(values=c("red", "blue")) +
    				guides(color=guide_legend(title=testCondition)) +
    				theme(panel.background = element_blank(), axis.line = element_line(colour = "black"),
    					panel.grid.major = element_blank(), panel.grid.minor = element_blank(), plot.title=element_text(hjust=0.5))

	if(plot){
		print(heatmap / bargraph / boxplot)
		message("#### plotting ", plotTitle, " ####")
	}

	return(pval.df)
}

#' Perform ANOVA
#'
#' Test association between module expression to traits using ANOVA
#'
#' @param datExpr expression data.frame
#' @param testCondition test column in sampleTable
#' @param refCondition reference column in sampleTable
#' @param design the sampleTable
#' @param alphaLevel the significance level
#' 
#' @return a data.frame with p-values for each association
#'
#' @export
performANOVA <- function(datExpr, design, testCondition, refCondition, alphaLevel=0.05){ #category1, category2
	mergedData = cbind(datExpr, 
	                   test=design[match(datExpr$Sample, design$Sample), testCondition],
	                   ref=design[match(datExpr$Sample, design$Sample), refCondition])

	# Bug found 20230122: should use matched categorical variable from mergedData
	# H1=lm(as.numeric(datExpr$moduleExpression) ~ sampleTable[[testCondition]] + sampleTable[[refCondition]] +
	#       sampleTable[[testCondition]]*sampleTable[[refCondition]])
	# H0=lm(as.numeric(datExpr$moduleExpression) ~ sampleTable[[testCondition]] + sampleTable[[refCondition]])
	# H0_2=lm(as.numeric(datExpr$moduleExpression) ~ sampleTable[[testCondition]])
	
	H1=lm(as.numeric(mergedData$moduleExpression) ~ mergedData$test + mergedData$ref +
	        mergedData$test*mergedData$ref)
	H0=lm(as.numeric(mergedData$moduleExpression) ~ mergedData$test + mergedData$ref)
	H0_2=lm(as.numeric(mergedData$moduleExpression) ~ mergedData$test)
	
	Factors=c(testCondition, refCondition, paste0(testCondition, "*", refCondition))
	p.value=c(summary(H0)$coefficients[2,4], anova(H0, H0_2)[2,6], anova(H0, H1)[2,6])
	pval.df=data.frame(Factors, p.value)
  
	return(pval.df)
}

#' name: Name of WGCNAobject
#'
#' Returns the module name for a given gene
#'
#' @param WGCNAobject an object of class WGCNA
#' @param gene a gene symbol, should be present in WGCNA object
#' @param use which module label to use e.g. dynamicLabels or dynamicColors
#' 
#' @return A module name
#'
#' @export
#' 
#' @examples 
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]]
#' getModule(astrocyte_networks[[1]], 'Srgn')
getModule <- function(WGCNAobject, gene, use = 'dynamicLabels'){
  if(!gene %in% WGCNAobject@datExpr$X) stop('Gene not found!')
  WGCNAobject@datExpr[WGCNAobject@datExpr$X == gene, use]
}

printModules <- function(WGCNAobject){
	name=name(WGCNAobject)
	dir_name=paste0(name, "_Genes")
	dir.create(dir_name)
	datExpr=WGCNAobject@datExpr
	for(module in sort(unique(datExpr$dynamicLabels))){
		file_name=paste0(dir_name, "/", module, ".txt")
		message("### writing ", file_name, " ###\n")
		write.table(datExpr$X[datExpr$dynamicLabels==module], 
			file_name, 
			quote=FALSE, 
			row.names=FALSE, 
			col.names=FALSE)
	}
}

#' name: Name of WGCNAobject
#'
#' Returns the name of a WGCNAobject.
#'
#' @param WGCNAobject an object of class WGCNA
#' 
#' @return Returns the name of the WGCNA object, ie "EAE" for 
#' astrocyte_networks$EAE. 
#'
#' @import stringr
#' @export
#' 
#' @examples 
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]]
#' name(astrocyte_networks$EAE)
name <- function(WGCNAobject){
	return(str_split_fixed(WGCNAobject@datExpr$dynamicLabels[[1]], "_", 2)[[1]])
}

getModuleOrder = function(WGCNAObject){
  WGCNAObject@trait$Module
}

getLevel <- function(level, design){
	if(level==1){
		return("combined")
	}
	return(unique(design[,level]))
}

colors <- function(nColors, random=FALSE){
	colors=c("cyan", "grey", "blue", "brown", "darkgreen", "darkmagenta", "darkolivegreen", "darkorange", "darkred",
		"darkturquoise", "green", "darkgrey", "greenyellow", "lightgreen", "magenta", "midnightblue", "orange",
		"paleturquoise", "pink", "purple", "red", "black", "royalblue", "saddlebrown", "salmon", "sienna3", "skyblue",
		"tan", "turquoise", "violet", "yellowgreen", "gold", "maroon", "yellow")
	if(random){
		return(sample(colors, nColors))
	} else {
		return(colors[1:nColors])
	}
}

#' topNGenes: Top N genes of a module
#'
#' Returns the top N number of genes of a module.
#' All genes returned if no number is specified.
#' Genes are in order of intramodular connectivity.
#'
#' @param WGCNAobject an object of class WGCNA
#' @param module the name of the module in WGCNAobject
#' @param nGenes an integer from 1 to module size; returns all genes if left NULL
#'
#' @return a character vector of the genes/probes in the module
#' 
#' @importFrom magrittr %>%
#' @importFrom dplyr arrange
#' @export
#' 
#' @examples 
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]]
#' topNGenes(astrocyte_networks$EAE, "EAE_015", nGenes = 10)
#' 
topNGenes <- function(WGCNAobject, module, nGenes=NULL){
	datExpr = WGCNAobject@datExpr
	subsetDatExpr = datExpr[datExpr$dynamicLabels==module,]
	orderedDatExpr = subsetDatExpr %>% arrange(-kWithin)
	if(is.null(nGenes)){
		orderedDatExpr$X
	} else{
		head(orderedDatExpr$X, nGenes)
	}
}

subsetDatExpr <- function(WGCNAobject, geneList){
  datExpr= WGCNAobject@datExpr
  return(cleanDatExpr(datExpr[datExpr$X %in% geneList,]))
}

prepareWGCNA <- function(WGCNAobject){
	newWGCNA= findBestTrait(WGCNAobject)
	newWGCNA = findOutlierModules(newWGCNA)
	newWGCNA
}

addWGCNA <- function(FUN){
	FUN <- match.fun(FUN)
	mixedWGCNA=FUN(mixedWGCNA)
	dimensionOneWGCNAs=lapply(dimensionOneWGCNAs, FUN)
	dimensionTwoWGCNAs= lapply(dimensionTwoWGCNAs, FUN)
}

removeOutlierModules <- function(WGCNAobject, outlierModules=NULL){
	if(missing(outlierModules)){
		outlierModules=outlierModules(WGCNAobject)
	}
	WGCNAobject@datExpr=WGCNAobject@datExpr[!WGCNAobject@datExpr$dynamicLabels %in% outlierModules,]
	WGCNAobject@trait=WGCNAobject@trait[!gsub("ME","",WGCNAobject@trait$Module) %in% outlierModules,]
	WGCNAobject@moduleEigengenes=WGCNAobject@moduleEigengenes[!gsub("ME", "", rownames(WGCNAobject@moduleEigengenes)) %in% outlierModules,]
	WGCNAobject
}

#' Generate a trait table from a sample table (version 2)
#' 
#' Generates a WGCNA-compatible trait table from a sampleTable dataframe. 
#' 
#' @param inputTable the sampleTable data.frame
#' @param column the column from the sampleTable to use as traits
#' @param detectNumbers whether to consider traits with numbers as numerical rather than categorical variables
#' 
#' @return a data.frame with integer values denoting the categorical sample traits
#' 
#' @export
#' 
#' @examples 
#' sampleTable = data.frame(Sample = c(paste0("EAE", 1:10), paste0("WT", 1:10)), 
#'                          Disease = c(rep("EAE", 10), rep("WT", 10)), 
#'                          Region = c(rep(c("Cbl", "Sc"), 5))) 
#' makeTraitTable2(sampleTable, 2)
#' 
makeTraitTable2 <- function(inputTable, column, detectNumbers=TRUE) {
  
  # Check whether column is numeric or categorical
  column_data = inputTable[,column,drop = FALSE]
  if(is.numeric(column_data[,1]) & detectNumbers){
    
    # Keep as is
    mm = column_data
    
  } else {
    
    # Convert character columns to factor so model.matrix works properly
    column_data[,1] = factor(column_data[,1])
    
    # One-hot encoding
    mm <- model.matrix(~ . - 1, data = column_data)  # -1 removes intercept
    
    # Remove the prefix
    colnames(mm) = gsub(colnames(column_data)[[1]], '', colnames(mm))
  }
  
  # Make data frame with sample 
  out <- as.data.frame(cbind(Sample = inputTable$Sample, mm))
  
  return(out)
}

#' Generate a trait table from a sample table
#' 
#' Generates a WGCNA-compatible trait table from a sampleTable dataframe. This 
#' function is deprecated. Use makeTraitTable2 instead. 
#' 
#' @param inputTable the sampleTable data.frame
#' @param column the column from the sampleTable to use as traits
#' @param detectNumbers whether to consider traits with numbers as numerical rather than categorical variables
#' 
#' @return a data.frame with integer values denoting the categorical sample traits
#' 
#' @export
#' 
#' @examples 
#' sampleTable = data.frame(Sample = c(paste0("EAE", 1:10), paste0("WT", 1:10)), 
#'                          Disease = c(rep("EAE", 10), rep("WT", 10)), 
#'                          Region = c(rep(c("Cbl", "Sc"), 5))) 
#' makeTraitTable(sampleTable, 2)
#' 
makeTraitTable <- function(inputTable, column, detectNumbers=FALSE) {
	traits=unique(inputTable[,column])
	traitTable=list()
	traitTable[[1]]=inputTable[,1]
	element=2
	for(trait in traits){
		traitTable[[element]]=lapply(inputTable[, column], function(x) if(x==trait) {
			1
		} else {
			2
		}
		)
		element=element+1
	}
	if(detectNumbers){
	  if(is.na(parse_number(inputTable[1, column]))){
                	traitTable[[element]]=as.list(parse_number(inputTable[,column]))
                	traitTable=t(do.call(rbind,traitTable))
                	colnames(traitTable)=c("Sample", c(traits),"Cont")
	  }
	} else {
                traitTable=t(do.call(rbind,traitTable))
                colnames(traitTable)=c("Sample", c(traits))
	}
	
	# Unlist
	traitTable=as.data.frame(apply(traitTable, 2, unlist))
	traitTable
}

#' cleanDatExpr
#'
#' A function that converts a data.frame where row 1 is gene symbols to a 
#' numeric matrix where columns are genes and rows are samples for compatibility 
#' with most WGCNA functions. 
#'
#' @param datExpr a data.frame were columns are samples and rows are samples and the gene symbols are in the first row
#' @param checkGenesSamples call the WGCNA function checkGenesSamples?
#'
#' @return Returns a datExpr with rows as samples and columns as genes
#' 
#' @author Dario Tommasini
#'
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_se = eh_query[["EH8223"]]
#' datExpr = data.frame(X = rownames(assays(astrocyte_se)[[1]]), assays(astrocyte_se)[[1]])
#' cleanDatExpr(datExpr)
#' 
cleanDatExpr <- function(datExpr, checkGenesSamples=FALSE) {
	cleanDatExpr <- t(datExpr[ ,!colnames(datExpr) %in% c("X","kTotal","kWithin","kOut","kDiff","dynamicColors","dynamicLabels")])
	colnames(cleanDatExpr) = datExpr$X
	if(checkGenesSamples){
		gsg=goodSamplesGenes(cleanDatExpr)
		if(!gsg$allOK) {
			cleanDatExpr = cleanDatExpr[gsg$goodSamples, gsg$goodGenes]
		}
	}
	return(cleanDatExpr)
}

keyModules <- function(WGCNAobject){
	keyModules=WGCNAobject@trait$Module[WGCNAobject@trait$trait!="None"]
	keyModules=keyModules[!keyModules %in% WGCNAobject@outlierModules]
	keyModules=keyModules[!endsWith(keyModules, "0")]
	return(keyModules)
}

#' summarizeResults: Summarize results from a results list object
#'
#' Prints (or writes) a summary of the results from a results list object
#'
#' @param myNetworks a list of WGCNAobjects
#' @param results results list
#' @param alphaLevel alpha level of significance
#' @param write write to file?
#' @param outputFile name of output file, defaults to results.txt
#' 
#' @return prints a summary of results from the multiWGCNA analysis
#' 
#' @export
summarizeResults <- function(myNetworks, results, alphaLevel=0.05, write=FALSE, outputFile="results.txt") {
	if(write) sink(outputFile)

	#identify interesting (trait-associated) modules across levels
	modulesOfInterest=vector()
	for(network in myNetworks){
		modulesOfInterest=append(modulesOfInterest, keyModules(network))
	}

	for(element in results$preservation){
		message("### Non-preserved modules ###")
		mod1Pres=element[[1]][which(element[[1]]$Zsummary.pres < 10 & rownames(element[[1]]) %in% modulesOfInterest),]
		if(nrow(mod1Pres>0))	message(paste0(capture.output(mod1Pres), collapse = "\n"))
		mod2Pres=element[[2]][which(element[[2]]$Zsummary.pres < 10 & rownames(element[[2]]) %in% modulesOfInterest),]
		if(nrow(mod2Pres>0)) message(paste0(capture.output(mod2Pres), collapse = "\n"))
	}

	diffModExp=results$diffModExp
	message("### Differentially expressed modules ###")
	message(paste0(capture.output(diffModExp[apply(diffModExp, 1, function(p) any(p<0.05)),]), collapse = "\n"))
	if(write) sink()
}

#' iterate: Iterate function across networks
#'
#' A high level function that iterates functions across
#' a list of WGCNA objects
#'
#' @param WGCNAlist a vector of objects of type WGCNAobject
#' @param FUN function to iterate, either overlapComparisons or preservationComparisons
#' @param ... argmuents to be passed on to overlapComparisons or preservationComparisons
#' 
#' @return a comparison list from overlapComparisons or preservationComparisons
#'
#' @author Dario Tommasini
#'
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]]
#' results = list()
#' iterate(astrocyte_networks, overlapComparisons, plot=FALSE)
#' 
iterate <- function(WGCNAlist, FUN, ...){
  
  # Check arguments
  stopifnot(deparse(substitute(FUN)) %in% c("overlapComparisons", "preservationComparisons"))
  
	comparisonList=list()
	FUN <- match.fun(FUN)
	element=1
	for(first in 1:(length(WGCNAlist)-1)){
		for(second in (first+1):length(WGCNAlist)){
			comparisonList <- FUN(comparisonList,
							WGCNAlist,
							first,
							second,
							element,
							...)
			element=element+1
		}
	}
	return(comparisonList)
}

#' @importFrom grDevices dev.off pdf rgb
#' @importFrom stats IQR anova dist lm na.omit p.adjust phyper quantile runif sd t.test var
#' @importFrom utils capture.output head write.csv write.table
#' @importFrom graphics legend
NULL

#' getPreservation
#'
#' Performs a network preservation analysis
#'
#' @param reference reference network of class WGCNA
#' @param test test network of class WGCNA
#' @param nPermutations number of permutations to perform; at least 50 permutations
#' @param write write to file?
#' 
#' @return a data.frame summarizing results of preservation analysis
#' 
#' @author Dario Tommasini
#'
#' @import WGCNA
#' @import stringr
#' @export
getPreservation <- function(reference, test, nPermutations=100, write=FALSE) {
	
  # Check if objects are in required format
  stopifnot(inherits(test, "WGCNA") | inherits(test, "data.frame"))
  stopifnot(inherits(reference, "WGCNA"))
  
  if(inherits(test, "WGCNA")){
    test=test@datExpr
  }

  # reference must be a WGCNA object
	reference=reference@datExpr
	name1=str_split_fixed(reference$dynamicLabels, "_", 2)[1,1]
	
	# process reference
	referenceExpr = cleanDatExpr(reference, checkGenesSamples=TRUE)
	referenceModules=reference$dynamicLabels[match(colnames(referenceExpr), reference$X)]
	
	# clean up test datExpr
	testExpr = cleanDatExpr(test, checkGenesSamples=TRUE)
	
	# calculate perservation Z-scores
	multiExpr2 = list(A = list(data = referenceExpr), B = list(data = testExpr));
	modLabels2 = list(A = referenceModules) #, B = testModules)
	system.time( {
		mp2 = modulePreservation(multiExpr2, 
		                        modLabels2,
                  					referenceNetworks = 1,
                            nPermutations = nPermutations,
                            # maxGoldModuleSize = left as default
                            # maxModuleSize = left as default,
                            randomSeed = 1,
                            quickCor = 0,
                            verbose = 3,
                  					parallelCalculation=TRUE, 
                  					savePermutedStatistics=FALSE)
		} );
	ref = 1
	test = 2

	if(write) write.csv(mp2$preservation$Z[[1]][[2]], paste0(name1, "_in_", name2,".csv"))

	return(as.data.frame(mp2$preservation$Z[[1]][[2]]))
}

#' Preservation Comparison Scatterplot
#'
#' A plotting  function that draws a scatterplot of preservation scores between
#' two WGCNA objects
#'
#' @param preservationList a list resulting from a call to preservationComparisons
#' @param dataset1 an object of class WGCNAobject to compare with dataset2
#' @param dataset2 an object of class WGCNAobject to compare with dataset1
#' @param alphaLevel alpha level of significance, default is 0.05
#' @param outliers leave outlier modules? By default these are removed
#' 
#' @return a ggplot object
#'
#' @author Dario Tommasini
#'
#' @import ggplot2
#' @import stringr
#' @import ggrepel
#' 
#' @export
#'  
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]]
#' results = list()
#' results$preservation=iterate(astrocyte_networks[c("EAE", "WT")], 
#'   preservationComparisons, 
#'   write=FALSE, 
#'   plot=FALSE, 
#'   nPermutations=2)
#' preservationComparisonPlot(results$preservation$EAE_vs_WT, 
#'   astrocyte_networks$EAE, 
#'   astrocyte_networks$WT)
#'   
preservationComparisonPlot <- function(preservationList, dataset1, dataset2, alphaLevel = 0.05, outliers=FALSE){
  
  # Check input
  stopifnot(inherits(preservationList, "list"))
  stopifnot(inherits(dataset1, "WGCNA") & inherits(dataset2, "WGCNA"))
  
  comparison = preservationList
	name1=str_split_fixed(rownames(comparison$mod1Preservation[rownames(comparison$mod1Preservation) != "gold",]),"_",2)[,1][[1]]
	name2=str_split_fixed(rownames(comparison$mod2Preservation[rownames(comparison$mod2Preservation) != "gold",]),"_",2)[,1][[1]]

	# merge perservation statistic and trait correlation of each module into one df, then make scatterplot
	input1=dataset1@trait
	input1$Zsum=comparison$mod1Preservation$Zsummary.pres[match(input1$Module, rownames(comparison$mod1Preservation))]
	if(!outliers) input1=input1[! input1$Module %in% dataset1@outlierModules,]
	input2=dataset2@trait
        input2$Zsum=comparison$mod2Preservation$Zsummary.pres[match(input2$Module, rownames(comparison$mod2Preservation))]
        if(!outliers) input2=input2[! input2$Module %in% dataset2@outlierModules,]
	myColors=colors(length(sort(unique(c(input1$trait, input2$trait)))))
	names(myColors)=sort(unique(c(input1$trait, input2$trait)))
	myColors1=myColors[match(sort(unique(input1$trait)), names(myColors))]
	myColors2=myColors[match(sort(unique(input2$trait)), names(myColors))]

	dataset1Plot <- ggplot(input1, aes(log10Pvalue, Zsum, fill=trait)) +
			geom_point(shape=21, size=2)+
			theme_classic()+
			scale_fill_manual(values=myColors1)+
			labs(title=paste0(name1, " in ", name2), y = "Preservation (z-summary)", x="Trait association (-log10 p-value)")+
		        theme(plot.title=element_text(hjust=0.5))+
			geom_text_repel(data=input1,
				aes(label=paste0(gsub("^0+","",substr(Module, nchar(Module[[1]])-1, nchar(Module[[1]]))))))+
			geom_hline(yintercept=(2), linetype="dashed", color = "red", size=1)+
			geom_hline(yintercept=(10), linetype="dashed", color = "black", size=1)+
			geom_vline(xintercept=(-log10(alphaLevel)), linetype="dashed", color = "red", size=1)

	dataset2Plot <-ggplot(input2, aes(log10Pvalue, Zsum, fill=trait)) +
		  geom_point(shape=21, size=2)+
			theme_classic()+
			scale_fill_manual(values=myColors2)+
			labs(title=paste0(name2, " in ", name1), y = "Preservation (z-summary)", x="Trait association (-log10 p-value)")+
	        	theme(plot.title=element_text(hjust=0.5))+
			geom_text_repel(data=input2,
				aes(label=paste0(gsub("^0+","", substr(Module, nchar(Module[[1]])-1, nchar(Module[[1]]))))))+
			geom_hline(yintercept=(2), linetype="dashed", color = "red", size=1)+
			geom_hline(yintercept=(10), linetype="dashed", color = "black", size=1)+
			geom_vline(xintercept=(-log10(alphaLevel)), linetype="dashed", color = "red", size=1)

	plot = dataset1Plot | dataset2Plot
	
	return(plot)
}

#' Preservation comparisons
#'
#' A high level function that performs a perservation comparison between two
#' WGCNAobjects in a WGCNAlist, usually supplied by iterate function
#'
#' @param comparisonList a list passed by the iterate function
#' @param WGCNAlist list of objects of type WGCNAobject
#' @param first index of first WGCNAobject
#' @param second index of second WGCNAobject
#' @param element element position in the comparison list (passed by iterate function)
#' @param plot generate plots?
#' @param write write results to file? 
#' @param alphaLevel alpha level of significance for module-trait correlation
#' @param nPermutations number of permutations, defaults to 100
#'
#' @return a list of preservation comparisons results across levels 1, 2, 3
#' 
#' @author Dario Tommasini
#'
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]]
#' results = list()
#' iterate(astrocyte_networks[c("EAE", "WT")], 
#'   preservationComparisons, 
#'   write=FALSE, 
#'   plot=FALSE, 
#'   nPermutations=2)
#' 
preservationComparisons <- function(comparisonList, WGCNAlist, first, second, element, plot=FALSE, write=FALSE, alphaLevel=0.05, nPermutations=100){
	
  # Check input
  stopifnot(inherits(comparisonList, "list"))
  stopifnot(inherits(WGCNAlist[[1]], "WGCNA"))
  
  comparisonList[[element]]=list()
	comparisonList[[element]]=append(comparisonList[[element]],
								list(getPreservation(WGCNAlist[[first]],
									WGCNAlist[[second]], write=write, nPermutations=nPermutations)))
	comparisonList[[element]]=append(comparisonList[[element]],
								list(getPreservation(WGCNAlist[[second]],
									WGCNAlist[[first]], write=write, nPermutations=nPermutations)))
	names(comparisonList)[[element]]=paste0(names(WGCNAlist)[[first]], "_vs_", names(WGCNAlist)[[second]])
	names(comparisonList[[element]])[[1]]="mod1Preservation"
	names(comparisonList[[element]])[[2]]="mod2Preservation"
	if(plot){
		print(
		  preservationComparisonPlot(comparisonList[[element]],
								WGCNAlist[[first]],
								WGCNAlist[[second]],
								alphaLevel)
		  )
	}
	
	return(comparisonList)
}

#' Coexpression Line Graph
#'
#' Plots a line graph showing the co-expression of selected genes across samples
#'
#' @param datExpr a data.frame with genes as rows and samples as columns
#' @param splitBy how much to split genes by on line graph
#' @param fontSize the font size of the gene labels
#' @param colors a vector of colors; default is random colors generated by colors function
#'
#' @return a ggplot object
#' 
#' @author Dario Tommasini
#'
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]]
#' datExpr = GetDatExpr(astrocyte_networks[[1]], 
#'   genes = topNGenes(astrocyte_networks$EAE, "EAE_015", 20))
#' coexpressionLineGraph(datExpr)	+ 
#'   geom_vline(xintercept = 20.5, linetype='dashed')
#' 
coexpressionLineGraph <- function(datExpr, splitBy=1, fontSize=2.15, colors=NULL){

  datExpr = t(datExpr)
	nGenes=ncol(datExpr)
	scaled=scale(datExpr)

	if(splitBy>0){
		for(column in 1:ncol(scaled)){
			scaled[, column]= scaled[,column]+splitBy*column
		}
	}

	plot = ggplot(reshape2::melt(as.matrix(scaled)), aes(x = Var1, y = value, group= Var2, color=Var2, label=Var2)) +
				geom_line()+
				{if(is.null(colors)) scale_colour_manual(values=colors(nGenes))}+
				{if(!is.null(colors)) scale_colour_manual(values=colors)}+
				labs(y="Scaled expression", x="Samples") +
				theme_classic() +
				coord_cartesian(clip="off")+
				annotate("text", x=nrow(datExpr)+0.5, y=-0.5, hjust=0, vjust=0,
					label=paste0(rev(colnames(datExpr)), collapse='', sep='\n'), size=fontSize)+
				theme(legend.position="none", axis.text.x=element_text(angle=90, vjust=0.25, hjust=1),
					axis.text.y=element_blank(), axis.ticks=element_blank(),
					legend.key.width= unit(0.001, 'mm'), plot.margin = unit(c(1,1,1,1), "cm"))
	
	return(plot)
}

correlationComparisonBoxplot <- function(diseaseDatExpr, healthyDatExpr, geneList, label=FALSE, method="pearson"){
	name1=str_split_fixed(colnames(diseaseDatExpr)[[2]], "_", 2)[,1]
	name2=str_split_fixed(colnames(healthyDatExpr)[[2]], "_", 2)[,1]

	geneList=geneList[geneList %in% diseaseDatExpr$X & geneList %in% healthyDatExpr$X ]
	diseaseMatrix= cleanDatExpr(diseaseDatExpr[match(geneList, diseaseDatExpr$X),])
	healthyMatrix= cleanDatExpr(healthyDatExpr[match(geneList, healthyDatExpr$X),])

	diseaseCorMatrix=cor(diseaseMatrix, method=method)
	diseaseCor=diseaseCorMatrix[lower.tri(diseaseCorMatrix)]

	healthyCorMatrix=cor(healthyMatrix, method=method)
	healthyCor=healthyCorMatrix[lower.tri(healthyCorMatrix)]

	indices <- which(lower.tri(diseaseCorMatrix) == TRUE, arr.ind=TRUE)
	names1 <- geneList[indices[,1]]
	names2 <- geneList[indices[,2]]
	pair=paste0(names1,"/", names2)

	melted=as.data.frame(rbind(cbind(pair, diseaseCor, rep(name1, length(diseaseCor))),
						cbind(pair, healthyCor, rep(name2, length(healthyCor)))))
	colnames(melted)=c("Pair", "Correlation", "Status")
	melted$Correlation=as.numeric(melted$Correlation)

	message(paste0(capture.output(t.test(diseaseCor)), collapse = "\n"))
	message(paste0(capture.output(t.test(healthyCor)), collapse = "\n"))
	message(paste0(capture.output(t.test(diseaseCor, healthyCor)), collapse = "\n"))

	ggplot(melted, aes(x=Status, y=Correlation, fill=Status)) +
		{if(label) geom_text_repel()} +
		theme_classic() +
		scale_y_continuous(position="right")+
		theme(legend.position="none") +
		scale_fill_manual(values=c("magenta", "cyan")) +
		stat_boxplot(geom = 'errorbar', lwd=1, width = 0.3, coef = 3) +
		geom_boxplot(notch = TRUE, outlier.shape=NA)
}

correlationComparisonHeatmaps <- function(diseaseDatExpr, healthyDatExpr, geneList, label=FALSE, method="pearson", alphaLevel=0.05, p.adjust=TRUE, z.score.limit=3){
	geneList=geneList[geneList %in% diseaseDatExpr$X & geneList %in% healthyDatExpr$X ]
	diseaseMatrix= cleanDatExpr(diseaseDatExpr[match(geneList, diseaseDatExpr$X),])
	healthyMatrix= cleanDatExpr(healthyDatExpr[match(geneList, healthyDatExpr$X),])

	diseaseCorMatrix=cor(diseaseMatrix, method=method)
	diseaseCor=diseaseCorMatrix[upper.tri(diseaseCorMatrix)]

	healthyCorMatrix=cor(healthyMatrix, method=method)
	healthyCor=healthyCorMatrix[upper.tri(healthyCorMatrix)]

	indices <- which(upper.tri(diseaseCorMatrix) == TRUE, arr.ind=TRUE)
	names1 <- geneList[indices[,1]]
	names2 <- geneList[indices[,2]]
	pair=paste0(names1,"/", names2)

	melted=as.data.frame(rbind(cbind(pair, diseaseCor, rep("Disease", length(diseaseCor))),
						cbind(pair, healthyCor, rep("Healthy", length(healthyCor)))))
	colnames(melted)=c("Pair", "Correlation", "Status")
	melted$Correlation=as.numeric(melted$Correlation)

	dc=diffCoexpression(cbind(t(diseaseMatrix), t(healthyMatrix)),
		c(rep(1, nrow(diseaseMatrix)),rep(2, nrow(healthyMatrix))), plot=FALSE, FDR.threshold=alphaLevel)
	z_scores=dc[[1]]
	z_scores[z_scores > z.score.limit]=z.score.limit
	z_scores[z_scores < -z.score.limit]=-z.score.limit
	z.table=reshape2::melt(z_scores)
	z.table$p.value=as.list(dc[[2]])
	z.table$p.adj=as.list(dc[[3]])
	z.table$signif=FALSE
	if(p.adjust==TRUE) {
		z.table$signif[z.table$p.adj<alphaLevel]=TRUE
	} else {
		z.table$signif[z.table$p.value<alphaLevel]=TRUE
	}

	meltedDs=reshape2::melt(diseaseCorMatrix)
	meltedDs$signif=z.table$signif
	meltedWt=reshape2::melt(healthyCorMatrix)
	meltedWt$signif=z.table$signif

	diseasePlot <- ggplot(meltedDs, aes(x=Var1, y=Var2, fill=value)) +
					geom_tile(aes(color = signif), size=0.1) +
					geom_vline(xintercept=c(13.5, 24.5))+
					geom_hline(yintercept=c(13.5, 24.5))+
					scale_fill_gradient2(name=NULL, low = "red", mid="white", high = "gold",
							limits=c(-1, 1), na.value="grey",
							guide = guide_colorbar(frame.colour = "black", frame.linewidth = 1,
							ticks.linewidth=1, ticks.colour = "black")) +
					scale_color_manual(values=c("white", "black"), guide = "none")+
					theme_classic()+
					theme(axis.text.x = element_text(angle = 90, vjust=(0.25), hjust=1),
							axis.title=element_blank(), axis.ticks=element_blank(),
							legend.key.width = unit(4, 'mm'), legend.key.height = unit(20, 'mm')) +
					{if(!label) theme(axis.text.x=element_blank(), axis.text.y=element_blank())}+
					coord_fixed()
	healthyPlot <- ggplot(meltedWt, aes(x=Var1, y=Var2, fill=value)) +
					geom_tile(aes(color = signif), size=0.1) +
					geom_vline(xintercept=c(13.5,24.5))+
					geom_hline(yintercept=c(13.5,24.5))+
					scale_fill_gradient2(name=NULL, low = "red", mid="white", high = "gold",
							limits=c(-1, 1), na.value="grey",
							guide = guide_colorbar(frame.colour = "black", frame.linewidth = 1,
							ticks.linewidth=1, ticks.colour = "black")) +
					scale_color_manual(values=c("white", "black"), guide = "none")+
					theme_classic()+
					theme(axis.text.x = element_text(angle = 90, vjust=(0.25), hjust=1),
							axis.title=element_blank(), axis.ticks=element_blank(),
							legend.key.width = unit(4, 'mm'), legend.key.height = unit(20, 'mm')) +
					{if(!label) theme(axis.text.x=element_blank(), axis.text.y=element_blank())}+
					coord_fixed(clip="off")
	dcPlot <- ggplot(z.table, aes(x=Var1, y=Var2, fill=value)) +
					geom_tile(aes(color = signif), size=0.1) +
					geom_vline(xintercept=c(13.5, 24.5), color="white")+
					geom_hline(yintercept=c(13.5, 24.5), color="white")+
					scale_fill_gradient2(name=NULL, low = "cyan", mid="black", high = "magenta",
							limits=c(-z.score.limit, z.score.limit), na.value="grey",
							guide = guide_colorbar(frame.colour = "black", frame.linewidth = 1,
							ticks.linewidth=1, ticks.colour = "black")) +
					scale_color_manual(values=c("black", "white"), guide = "none")+
					theme_classic()+
					theme(axis.text.x = element_text(angle = 90, vjust=(0.25), hjust=1),
							axis.title=element_blank(), axis.ticks=element_blank(),
							legend.key.width = unit(4, 'mm'), legend.key.height = unit(20, 'mm')) +
					{if(!label) theme(axis.text.x=element_blank(), axis.text.y=element_blank())}+
					coord_fixed(clip="off")
	boxPlot <- correlationComparisonBoxplot(diseaseDatExpr, healthyDatExpr, geneList)

	plot = plot_grid(diseasePlot, healthyPlot, dcPlot, boxPlot, ncol=4, align = "hv",
		axis = "tb", rel_widths = c(2, 2, 2, 1))
	
	return(plot)
}

#' Differential co-expresison analysis 
#'
#' Performs a differential co-expression ananlysis given an expression data.frame 
#' and a conditions vector
#'
#' @param datExpr a data.frame containing expression values
#' @param conditions a vector containing conditions for the samples
#' @param geneList vector of genes, will use all genes if NULL (default)
#' @param plot plot a network?
#' @param method either "pearson" or "spearman" 
#' @param removeFreeNodes remove free nodes from network?
#' @param labelSize label size
#' @param labelDist distance from labels to nodes
#' @param shape shape of nodes
#' @param degreeForSize should node size correspond to degree?
#' @param label label nodes?
#' @param onlyPositive only draw positive correlations?
#' @param z.threshold z-score threshold
#' @param FDR.threshold FDR threshold
#' @param nodeSize size of node
#'
#' @return A list including a matrix of z-scores, a matrix of raw p-values, a 
#' matrix of adjusted p-values, and a summary data.frame 
#' 
#' @author Dario Tommasini
#'
#' @import dcanr
#' @importFrom igraph graph_from_adjacency_matrix simplify delete.vertices degree
#' @export
#' 
#' @examples
#' \donttest{
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_se = eh_query[["EH8223"]]
#' datExpr = assays(astrocyte_se)[[1]]
#' diffCoexpression(datExpr, c(rep(1,20), rep(2,16)), 
#'   geneList = c("Gfap", "Vim", "Aspg", "Serpina3n", "Cp", "Osmr", "Cd44", 
#'                "Cxcl10", "Hspb1", "Timp1", "S1pr3", "Steap4", "Lcn2"))
#' }
#'     
#' 
diffCoexpression <- function(datExpr, conditions, geneList=NULL, plot=FALSE, 
                             method=c("pearson", "spearman"), removeFreeNodes=TRUE, 
                             labelSize=0.5, labelDist=0, shape="circle", degreeForSize=FALSE,
                             label=FALSE, onlyPositive=FALSE, z.threshold=NULL, FDR.threshold=0.05, nodeSize=3){

  # Check input
  stopifnot(inherits(datExpr, "data.frame"))
  method = match.arg(method)
  
	if(!is.null(geneList)) datExpr=datExpr[rownames(datExpr) %in% geneList,]
	cor1= cor(t(datExpr[, conditions==1]), method=method)
	cor2= cor(t(datExpr[, conditions==2]), method=method)
	z_scores <- dcScore(datExpr, conditions, dc.method = 'zscore', cor.method = method)
	raw_p <- dcTest(z_scores, datExpr, conditions)
	adj_p <- dcAdjust(raw_p, f = p.adjust, method = 'fdr')

	geneList=rownames(datExpr)
	indices <- which(upper.tri(z_scores) == TRUE, arr.ind=TRUE)
	names1 <- geneList[indices[,1]]
	names2 <- geneList[indices[,2]]
	summaryDf=data.frame(gene1=names1, gene2=names2, cor1= cor1[upper.tri(cor1)], cor2=cor2[upper.tri(cor2)],
					z.score=z_scores[upper.tri(z_scores)], p.value=raw_p[upper.tri(raw_p)], p.adj=adj_p[upper.tri(adj_p)])

	adj_mat= z_scores
	adj_mat[diag(adj_mat)]=0
	if(is.null(z.threshold)) z.threshold=min(abs(na.omit(adj_mat[adj_p<FDR.threshold])))
	graph=igraph::graph_from_adjacency_matrix(adj_mat, mode="undirected", weighted=TRUE)
	graph=simplify(graph)
	graph <- delete.edges(graph, which(abs(E(graph)$weight) < z.threshold))
	if(onlyPositive) graph <- delete.edges(graph, which(E(graph)$weight < 0))
	if(removeFreeNodes) graph <- delete.vertices(simplify(graph), degree(graph)==0)
	ealpha=rescale(abs(E(graph)$weight), to=c(0.1, 0.5))
	ecol=lapply(ealpha, function(x) rgb(0, 0, 1, x))
	ecol[which(E(graph)$weight < 0)]=rgb(1, 0, 1, ealpha[which(E(graph)$weight < 0)])
	if(degreeForSize) {
		V(graph)$size = log(degree(graph, mode = "out") + nodeSize) * 2
	} else {
		V(graph)$size=nodeSize
	}

	if(!label) V(graph)$name=NA

	if(plot){
		plot(graph, layout=layout_with_fr, edge.width=1.5, vertex.shape=shape,
		vertex.color="white", vertex.label.dist=labelDist, vertex.label.color="black",
		vertex.label.cex=labelSize, edge.color=unlist(ecol))
	} 
	
	return(list(z_scores, raw_p, adj_p, summaryDf %>% arrange(p.adj)))
}

#' PreservationPermutationTest
#'
#' Performs a permutation test to determine if a null distribution of expected
#' preservation scores for modules in this dataset if the labels were 
#' randomly assigned. Please look at the astrocyte vignette for more info. 
#'
#' @param referenceDatExpr the combined datExpr
#' @param design the sampleTable
#' @param constructNetworksIn the condition to use for network construction, e.g. for the astrocyte data, this is "EAE"
#' @param testPreservationIn the condition to use for testing preservation, e.g. for the astrocyte data, this was "WT"
#' @param nPermutations the number of permutations to perform for permutation test
#' @param nPresPermutations the number of permutations to perform in modulePreservation function
#' @param ... arguments to pass to blockwiseModules function for network construction (should be the same as used for constructing the original network)
#'
#' @return A list of data.frames with preservation results for each permutation
#' 
#' @author Dario Tommasini
#'
#' @import WGCNA
#' @export
#' 
#' @examples
#' \dontrun{
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]] 
#' astrocyte_se = eh_query[["EH8223"]] 
#' sampleTable = colData(astrocyte_se)
#' results = list()
#' results$permutation.test = PreservationPermutationTest(
#' astrocyte_networks$combined@datExpr[sample(17000,3000),], 
#' sampleTable, 
#' constructNetworksIn = "EAE", # Construct networks using EAE samples
#' testPreservationIn = "WT", # Test preservation of disease samples in WT 
#' nPermutations = 10, # Number of permutations for permutation test
#' nPresPermutations = 10, # Number of permutations for modulePreservation
#' networkType = "signed", TOMType = "unsigned", 
#' power = 12, minModuleSize = 100, maxBlockSize = 25000,
#' reassignThreshold = 0, minKMEtoStay = 0, mergeCutHeight = 0,
#' numericLabels = TRUE, pamRespectsDendro = FALSE, 
#' deepSplit = 4, verbose = 3
#' )
#' }
PreservationPermutationTest = function(referenceDatExpr, 
                                       design, 
                                       constructNetworksIn,
                                       testPreservationIn,
                                       nPermutations = 100, 
                                       nPresPermutations = 100, 
                                       ...){
  
  # order in same order as design
  datExpr=referenceDatExpr[,match(design$Sample, colnames(referenceDatExpr))]
  
  # for saving resulting objects
  WGCNAobjects=list()
  preservationData=list()
  
  # Determine which conditions to study and which to control for
  if(any(grepl(testPreservationIn, design[,2]))){
    refColumn = 3
    testColumn = 2
  } else if(any(grepl(testPreservationIn, design[,3]))){
    refColumn = 2
    testColumn = 3
  } else {
    stop("testPreservationIn not found in design! Please check spelling and case and try again. ")
  }
  
  # Subset to only the conditions used for network construction and preservation testing
  design = design[design[,testColumn] %in% c(testPreservationIn, constructNetworksIn),]
  
  # Conditions to control for (balance them out in each sample)
  conditions=unique(design[, refColumn])
  
  # run through permutations
  for(permutation in 1:nPermutations){
    
    # assign phenotype labels randomly, controlling for ref condition
    WT.indices=list()
    for(condition in conditions){
      conditionalDesign=design[design[, refColumn]==condition,]
      nSamples=nrow(conditionalDesign)
      nWT=nrow(conditionalDesign[conditionalDesign[, testColumn] == testPreservationIn,])
      sampleIndices=which(design[, refColumn]==condition)
      WT.indices=append(WT.indices, sampleIndices[sample(1:nSamples, nWT, replace=F)])
    }
    # Preservation set
    WT.indices=unlist(WT.indices)
    randomHealthy=datExpr[, WT.indices]	
    
    # Other samples are used for network construction
    randomHealthy=data.frame(X=referenceDatExpr$X, randomHealthy)
    randomDisease=datExpr[, !(1:ncol(datExpr) %in% WT.indices)]
    
    # Some helpful print messages
    message("Set of samples used for network construction:")
    message(paste0(capture.output(as.data.frame(design[-WT.indices,])), collapse = "\n"))
    
    message("Set of samples used for preservation:")
    message(paste0(capture.output(as.data.frame(design[WT.indices,])), collapse = "\n"))
    
    # perform WGCNA on subset of dataset
    my_net = blockwiseModules(t(randomDisease), ...) 
    dynamicLabels=paste("rand", "_", str_pad(my_net$colors, 3, pad="0"), sep="")
    summary = cbind(data.frame(X = rownames(randomDisease), randomDisease), dynamicLabels)
    WGCNAobjects[[permutation]] = findOutlierModules(findModuleEigengenes(new("WGCNA", datExpr=summary)))
    
    # compute preservation scores in held-out samples
    preservationData[[permutation]] <- getPreservation(WGCNAobjects[[permutation]], randomHealthy, nPermutations = nPresPermutations, write=F)
    preservationData[[permutation]]$is.outlier.module = FALSE
    preservationData[[permutation]]$is.outlier.module[rownames(preservationData[[permutation]]) %in% WGCNAobjects[[permutation]]@outlierModules] = TRUE
    saveRDS(preservationData, "preservationData.rds")
  }
  
  return(preservationData)
}

#' PreservationScoreDistribution
#'
#' Extracts the preservation score distribution from the results of 
#' PreservationPermutationTest. 
#'
#' @param preservationData the results from PreservationPermutationTest
#' @param moduleOfInterestSize the number of genes in your module of interest
#'
#' @return A data.frame with Z-summary preservation scores of the module from
#' each permutation and the corresponding module size
#' 
#' @author Dario Tommasini
#'
#' @export
#' 
#' @examples
#' # Remove outlier modules
#' permutationTestResultsFiltered = lapply(permutationTestResults, function(x) 
#' x[!x$is.outlier.module,])
#' 
#' # Find preservation score distribution for a given module size
#' scores.summary = PreservationScoreDistribution(
#' permutationTestResultsFiltered, 
#' moduleOfInterestSize = 303)
#'                                                         
PreservationScoreDistribution = function(preservationData, moduleOfInterestSize){
  
  # Save z.summary scores and observed module sizes
  z.summary.dist=list()
  module.size=list()
  for(permutation in seq_along(preservationData)){
    myPerm=preservationData[[permutation]]
    z.summary.dist[[permutation]]=myPerm$Zsummary[which.min(abs(myPerm$moduleSize-moduleOfInterestSize))]
    module.size[[permutation]]=myPerm$moduleSize[which.min(abs(myPerm$moduleSize-moduleOfInterestSize))]
  }
  summary=data.frame(z.summary = unlist(z.summary.dist), 
                     module.size = unlist(module.size))
  
  return(summary)
}

#' Draw multiWGCNA network
#'
#' Draw a network where nodes are modules and edges represent significant gene overlap. 
#' Modules are sorted by levels 1, 2, and 3.  
#'
#' @param WGCNAlist list of WGCNA objects
#' @param comparisonList the list of overlap comparisons ie from iterate(myNetworks, overlapComparisons, ...) 
#' @param moduleOfInterest module of interest, ie "combined_001"
#' @param design the sampleTable design matrix
#' @param overlapCutoff cutoff to remove module correspondences with less than this number of genes
#' @param padjCutoff cutoff to remove module correspondences above this significance value
#' @param removeOutliers remove outlier modules? 
#' @param alpha alpha level of significance
#' @param layout layout of network to be passed to plot function of igraph object, defaults to multiWGCNA custom layout
#' @param hjust horizontal justification of labels
#' @param vjust vertical justification of labels
#' @param width width of labels
#' @param colors colors to use for modules, should be the same length as the number of WGCNA objects in the WGCNAlist. Defaults to random colors for each condition. 
#' 
#' @return an igraph plot
#'
#' @author Dario Tommasini
#'
#' @importFrom igraph graph_from_data_frame delete.edges V V<- E E<- plot.igraph
#' @import stringr 
#' @importFrom scales rescale
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_se = eh_query[["EH8223"]]
#' sampleTable = colData(astrocyte_se)
#' astrocyte_networks = eh_query[["EH8222"]]
#' results = list()
#' results$overlaps = iterate(astrocyte_networks, overlapComparisons, plot=FALSE)
#' drawMultiWGCNAnetwork(astrocyte_networks, 
#'   results$overlaps, 
#'   "combined_013", 
#'   sampleTable)
#'   
drawMultiWGCNAnetwork <- function(WGCNAlist, comparisonList, moduleOfInterest, design, 
                                  overlapCutoff = 0, padjCutoff = 1, removeOutliers = TRUE, alpha = 1e-50, 
                                  layout = NULL, hjust = 0.4, vjust = 0.3, width = 0.5, colors = NULL){

  # extract the overlaps objects into a list
	overlapList=lapply(comparisonList, function(x) x$overlap)
	overlapList=do.call(rbind, overlapList)
	filteredOverlapList=overlapList

	# remove outliers if necessary
	if(removeOutliers) {
		for(WGCNA in WGCNAlist){
			filteredOverlapList=filteredOverlapList[!filteredOverlapList$mod1 %in% WGCNA@outlierModules,]
			filteredOverlapList=filteredOverlapList[!filteredOverlapList$mod2 %in% WGCNA@outlierModules,]
		}
	}
	
	admittedModules=unique(c(filteredOverlapList$mod1, filteredOverlapList$mod2))
	
	# generate the multiWGCNA layout
	if(is.null(layout)){
		myCoords=list()
		for(level in 1:3){
			WGCNAs=getLevel(level, design)
			from=0-width*length(WGCNAs)/2
			to=0+width*length(WGCNAs)/2
			x.coordinates=seq(from, to, length.out=length(WGCNAs))
			if(level==1) x.coordinates=0
			for(nWGCNA in 1:length(WGCNAs)){
				nModules=length(admittedModules[startsWith(admittedModules, WGCNAs[[nWGCNA]]) ])
				myCoords=append(myCoords, list(cbind(runif(nModules, x.coordinates[[nWGCNA]], x.coordinates[[nWGCNA]]+hjust), 3-level+runif(nModules, -vjust, vjust))))
			}
		}
		layout=do.call(rbind, myCoords)
	}
	
	#make the igraph object
	graph=graph_from_data_frame(d=filteredOverlapList, directed = FALSE)
	
	#node and edge attributes
	vcol=str_split_fixed(V(graph)$name, "_", 2)[,1]
	conditions=unique(vcol)

	# Colors of modules by condition
	if(is.null(colors)) palette = colors(length(conditions), random = TRUE)
	if(!is.null(colors)) palette = colors

	for(condition in 1:length(conditions)){
		vcol[vcol==conditions[[condition]] ]=palette[[condition]]
	}
	V(graph)$color=vcol
	
	# edge attributes
	E(graph)$weight=-log10(E(graph)$p.adj)
	E(graph)$width=rescale(E(graph)$weight, from=c(0, 320), to=c(0,5))
	ealpha=rescale(-log10(E(graph)$p.adj), from=c(0, 320), to=c(0,1))
	ecol=lapply(ealpha, function(x) rgb(1, 0, 0, x))
	E(graph)$color=unlist(ecol)
	
	modulesOfInterest=unique(c(filteredOverlapList$mod2[filteredOverlapList$mod1==moduleOfInterest & filteredOverlapList$p.adj<alpha], 
		filteredOverlapList$mod1[filteredOverlapList$mod2==moduleOfInterest & filteredOverlapList$p.adj<alpha]))
	
	# delete edges
	graph <- delete.edges(graph, which(!(filteredOverlapList$mod1 %in% modulesOfInterest | filteredOverlapList$mod2 %in% modulesOfInterest)))

	# plot graph
	plot = plot(graph, vertex.label.color="black", vertex.size=3, vertex.label=NA, 
	            vertex.label.cex=.5, layout=layout) 
	
	# second call to plot over with edges of interest
	legend("right", legend = conditions, pch=21, col=palette, pt.bg=palette, 
	       pt.cex=1, cex=.8, bty="n", ncol=1)
	
  return(plot)
}

# draw a basic network
drawNetwork <- function(matrix, threshold=0, nodeList=NULL, edgeList=NULL, layout=layout_with_fr, removeFreeNodes=TRUE){
	graph=graph_from_adjacency_matrix(matrix, mode="undirected", weighted=TRUE)
	graph <- delete.edges(graph, which(E(graph)$weight < threshold))
	if(removeFreeNodes) graph <- delete.vertices(simplify(graph), degree(graph)==0)
	ealpha=rescale(E(graph)$weight, from=c(0,1), to=c(0,0.4))
	ecol=lapply(ealpha, function(x) rgb(0.2, 0.2, 0.2, x))
	
	if(length(nodeList)>0){
		graph <- igraph::delete.vertices(graph, which(!V(graph)$name %in% nodeList))
		layout=layout[match(V(graph)$name, nodeList),]
	}
	
	if(length(edgeList)>0){
		graph <- igraph::delete.edges(graph, which(!apply(igraph::as_edgelist(graph), 1, function(x) 
			any(unlist(lapply(1:nrow(edgeList), function(y) {
				x[[1]]==edgeList[y,1] & x[[2]]==edgeList[y,2]
			}))))))
	}
	
	plot(graph, layout=layout, vertex.size=4, edge.width=1.5, 
		vertex.color="white", vertex.label=NA, edge.color=unlist(ecol))
	list(V(graph)$name, igraph::layout.fruchterman.reingold(graph), igraph::as_edgelist(graph))
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----eval=FALSE---------------------------------------------------------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("multiWGCNA")

## ----eval=FALSE---------------------------------------------------------------
# if (!require("devtools", quietly = TRUE))
#     install.packages("devtools")
# 
# devtools::install_github("fogellab/multiWGCNA")

## -----------------------------------------------------------------------------
library(multiWGCNA)

## -----------------------------------------------------------------------------
# Download data from the ExperimentHub
library(ExperimentHub)
eh = ExperimentHub()

# Note: this requires the SummarizedExperiment package to be installed
eh_query = query(eh, c("multiWGCNAdata"))
autism_se = eh_query[["EH8219"]]

# Collect the metadata in the sampleTable
sampleTable = colData(autism_se)

# Randomly sample 1500 genes from the expression matrix
set.seed(1)
autism_se = autism_se[sample(rownames(autism_se), 1500),]

# Check the data
assays(autism_se)[[1]][1:5, 1:5]
sampleTable

# Define our conditions for trait 1 (disease) and 2 (brain region)
conditions1 = unique(sampleTable[,2])
conditions2 = unique(sampleTable[,3])

## -----------------------------------------------------------------------------
# Construct the combined networks and all the sub-networks (autism only, controls only, FC only, and TC only)
autism_networks = constructNetworks(autism_se, sampleTable, conditions1, conditions2,
                                  networkType = "unsigned", power = 10,
                                  minModuleSize = 40, maxBlockSize = 25000,
                                  reassignThreshold = 0, minKMEtoStay = 0.7,
                                  mergeCutHeight = 0.10, numericLabels = TRUE,
                                  pamRespectsDendro = FALSE, verbose=3,
                                  saveTOMs = TRUE)

## ----fig.height = 4, fig.width = 7--------------------------------------------
# Save results to a list
results=list()
results$overlaps=iterate(autism_networks, overlapComparisons, plot=TRUE)

# Check the reciprocal best matches between the autism and control networks
head(results$overlaps$autism_vs_controls$bestMatches)

## ----fig.height=6, fig.width=8------------------------------------------------
ModuleFlowPlot(autism_networks, # WGCNA object list
               comparisonList = results$overlaps, # overlap comparisons
               networks = c('combined', 'autism', 'controls', 'FC', 'TC'), 
               width = 0.3, # width of nodes
               height = 0.2, # height of nodes
               only.signif = TRUE, # only significant overlaps
               only.contiguous = TRUE, # only contiguous comparisons (controlled by order)
               scale.by.size = TRUE, # scale nodes by module size
               spacer = 10, # space between nodes
               label.y = 50, # adjust placement of the network labels
               col = c(autism = 'pink', FC = 'lightgreen', None = 'grey') # color palette
               ) 

## ----fig.height=5, fig.width=8------------------------------------------------
networks = c('combined', 'autism', 'controls', 'FC', 'TC')
toms = lapply(networks, function(x) {
  load(paste0(x, '-block.1.RData'))
  get("TOM")
})

tom.df = BuildTOMFlowDF(autism_networks, 
                        networks, 
                        toms, 
                        genes_to_label = topNGenes(autism_networks$autism, "autism_005"))

PlotMultiNodesTOMflow(tom.df, labels = networks)

## ----fig.height = 6, fig.width = 7--------------------------------------------
# Run differential module expression analysis (DME) on combined networks
results$diffModExp = runDME(autism_networks[["combined"]], 
                            sampleTable, 
                            p.adjust="fdr", 
                            refCondition="Tissue", 
                            testCondition="Status") 
                            # plot=TRUE, 
                            # out="combined_DME.pdf")

# to run PERMANOVA
# library(vegan)
# results$diffModExp = runDME(autism_networks[["combined"]], p.adjust="fdr", refCondition="Tissue", 
#                          testCondition="Status", plot=TRUE, test="PERMANOVA", out="PERMANOVA_DME.pdf")

# Check adjusted p-values for the two sample traits
results$diffModExp

# You can check the expression of a specific module like this. Note that the values reported in the bottom panel title are p-values and not FDR-adjusted like in results$diffModExp
diffModuleExpression(autism_networks[["combined"]], 
                     geneList = topNGenes(autism_networks[["combined"]], "combined_004"), 
                     design = sampleTable,
                     test = "ANOVA",
                     plotTitle = "combined_004",
                     plot = TRUE)

## ----fig.height = 3, fig.width = 7--------------------------------------------
# To enable multi-threading
# library(doParallel)
# library(WGCNA)
# nCores = 8
# registerDoParallel(cores = nCores)
# enableWGCNAThreads(nThreads = nCores)

# Calculate preservation statistics
results$preservation=iterate(autism_networks[conditions1], # this does autism vs control; change to "conditions2" to perform comparison between FC and TC
                             preservationComparisons, 
                             write=FALSE, 
                             plot=TRUE, 
                             nPermutations=10)

## -----------------------------------------------------------------------------
# Print a summary of the results
summarizeResults(autism_networks, results)

## -----------------------------------------------------------------------------
sessionInfo()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(multiWGCNA)

## -----------------------------------------------------------------------------
# Download data from the ExperimentHub
library(ExperimentHub)
eh = ExperimentHub()

# Note: this requires the SummarizedExperiment package to be installed
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_se = eh_query[["EH8223"]]

# Collect the metadata in the sampleTable; the first column must be named "Sample"
sampleTable = colData(astrocyte_se)

# Check the data
assays(astrocyte_se)[[1]][1:5, 1:5]
sampleTable

# Define our conditions for trait 1 (disease) and 2 (brain region)
conditions1 = unique(sampleTable[,2])
conditions2 = unique(sampleTable[,3])

## ----eval = FALSE-------------------------------------------------------------
# # Construct the combined networks and all the sub-networks (EAE, WT, and each region)
# # Same parameters as Tommasini and Fogel. BMC Bioinformatics
# astrocyte_networks = constructNetworks(astrocyte_se, sampleTable, conditions1, conditions2,
#                                   networkType = "signed", TOMType = "unsigned",
#                                   power = 12, minModuleSize = 100, maxBlockSize = 25000,
#                                   reassignThreshold = 0, minKMEtoStay = 0, mergeCutHeight = 0,
#                                   numericLabels = TRUE, pamRespectsDendro = FALSE,
#                                   deepSplit = 4, verbose = 3)
# 

## -----------------------------------------------------------------------------
# Load pre-computed astrocyte networks
astrocyte_networks = eh_query[["EH8222"]] 

# Check one of the WGCNA objects
astrocyte_networks[["combined"]]

## ----fig.height = 5, fig.width = 8--------------------------------------------
# Save results to a list
results = list()
results$overlaps = iterate(astrocyte_networks, overlapComparisons, plot=FALSE)

# Check the overlaps, ie between the EAE and wildtype networks
head(results$overlaps$EAE_vs_WT$overlap)

## ----fig.height=10, fig.width=7-----------------------------------------------
ModuleFlowPlot(WGCNAlist = astrocyte_networks, 
              comparisonList = results$overlaps, 
              networks = c('combined',  'EAE', 'WT'), 
              spacer = 100, # size of spacer
              label.y = 500, # vertical adjustment for the network labels
              x.scale = 10, # how much to spread out the nodes along x-axis
              color.by = 'trait', # color nodes by their best trait correlation
              col = c(Cbl = 'lightpink', Ctx = 'chartreuse', Hippo = 'skyblue', Sc = 'orange', EAE = 'red', None = 'grey'), 
              p.adj.threshold = 1e-50, 
              label.size = 4)

## ----fig.height = 6, fig.width = 7--------------------------------------------
# Run differential module expression analysis (DME) on combined networks
results$diffModExp = runDME(astrocyte_networks[["combined"]], 
                            sampleTable,
                            p.adjust = "fdr", 
                            refCondition = "Region", 
                            testCondition = "Disease") 
                            # plot=TRUE, 
                            # out="ANOVA_DME.pdf")

# Check results sorted by disease association FDR
results$diffModExp[order(results$diffModExp$Disease),]

# You can check the expression of module M13 from Tommasini and Fogel. BMC Bioinformatics. 2023 like this. Note that the values reported in the bottom panel title are p-values and not adjusted for multiple comparisons like in results$diffModExp
diffModuleExpression(astrocyte_networks[["combined"]], 
                     geneList = topNGenes(astrocyte_networks[[1]], "combined_013"), 
                     design = sampleTable,
                     test = "ANOVA",
                     plotTitle = "combined_013",
                     plot = TRUE)


## ----fig.height = 6, fig.width = 7--------------------------------------------
drawMultiWGCNAnetwork(astrocyte_networks, 
                      results$overlaps, 
                      "combined_013", 
                      design = sampleTable, 
                      overlapCutoff = 0, 
                      padjCutoff = 1, 
                      removeOutliers = TRUE, 
                      alpha = 1e-50, 
                      layout = NULL, 
                      hjust = 0.4, 
                      vjust = 0.3, 
                      width = 0.5)

## ----fig.height = 8, fig.width = 10-------------------------------------------
bidirectionalBestMatches(results$overlaps$combined_vs_EAE)

## ----fig.height=5, fig.width=7------------------------------------------------
# Get expression data for top 20 genes in EAE_015 module
datExpr = GetDatExpr(astrocyte_networks[[1]], 
                     genes = topNGenes(astrocyte_networks$EAE, "EAE_015", 20))

# Plot
coexpressionLineGraph(datExpr, splitBy = 1.5, fontSize = 2.5) + 
  geom_vline(xintercept = 20.5, linetype='dashed')

## ----eval = FALSE, fig.height = 3, fig.width = 7------------------------------
# # To enable multi-threading
# # library(doParallel)
# # library(WGCNA)
# # nCores = 2
# # registerDoParallel(cores = nCores)
# # enableWGCNAThreads(nThreads = nCores)
# 
# # Turn off multi-threading
# # registerDoSEQ()
# # disableWGCNAThreads()
# 
# # Calculate preservation statistics
# results$preservation=iterate(astrocyte_networks[c("EAE", "WT")],
#                              preservationComparisons,
#                              write=FALSE,
#                              plot=TRUE,
#                              nPermutations=2)

## ----eval = FALSE-------------------------------------------------------------
# options(paged.print = FALSE)
# results$permutation.test = PreservationPermutationTest(astrocyte_networks$combined@datExpr[sample(17000,3000),],
#                                                        sampleTable,
#                                                        constructNetworksIn = "EAE", # Construct networks using EAE samples
#                                                        testPreservationIn = "WT", # Test preservation of disease samples in WT samples
#                                                        nPermutations = 10, # Number of permutations for permutation test
#                                                        nPresPermutations = 10, # Number of permutations for modulePreservation function
# 
#                                                        # WGCNA parameters for re-sampled networks (should be the same as used for network construction)
#                                                        networkType = "signed", TOMType = "unsigned",
#                                                        power = 12, minModuleSize = 100, maxBlockSize = 25000,
#                                                        reassignThreshold = 0, minKMEtoStay = 0, mergeCutHeight = 0,
#                                                        numericLabels = TRUE, pamRespectsDendro = FALSE,
#                                                        deepSplit = 4, verbose = 3
#                                                        )

## ----eval = TRUE--------------------------------------------------------------
# Load pre-computed results
data(permutationTestResults) 

# Remove outlier modules
permutationTestResultsFiltered = lapply(permutationTestResults, function(x) x[!x$is.outlier.module,])

# Extract the preservation score distribution
results$scores.summary = PreservationScoreDistribution(permutationTestResultsFiltered, 
                                                       moduleOfInterestSize = 303 # The size of the module of interest (dM15)
                                                       )

# Observed preservation score of dM15
observed.score = 9.16261490617938

# How many times did we observe a score lower than or equal to this observed score?
z.summary.dist = results$scores.summary$z.summary
below=length(z.summary.dist[z.summary.dist <= observed.score])
probability= below/100
message("Probability of observing a score of ", round(observed.score, 2), " is ", probability)

## ----eval = TRUE--------------------------------------------------------------
# Plot on a histogram
ggplot(results$scores.summary, aes(x=z.summary)) + 
      geom_histogram(color="black", fill="white", bins = 15)+
      xlab("Preservation score")+
      ylab("Frequency")+
      geom_vline(xintercept=observed.score, color="red3", linetype="solid")+
      scale_y_continuous(expand = c(0,0))+
      theme_classic()+
      theme(plot.title = element_text(hjust = 0.5))

## -----------------------------------------------------------------------------
sessionInfo()

#' TOMFlowPlot
#'
#' Plots a sankey flow diagram showing the movement of genes from one WGCNA
#' to another WGCNA. Uses the ggalluvial framework. 
#'
#' @param WGCNAlist list of WGCNA objects
#' @param networks list of network names of length 2
#' @param toms a list of TOM distance objects of length 2
#' @param genes_to_label genes to label across two networks
#' @param alpha alpha of flows
#' @param color color of flows
#' @param width width of the strata
#' 
#' @return a ggplot object
#' 
#' @author Dario Tommasini
#'
#' @import flashClust
#' @import ggalluvial
#' @import WGCNA
#' @import stringr
#' @export
TOMFlowPlot = function(WGCNAlist, networks, toms, genes_to_label, alpha = 0.1, color = 'black', width = 0.05){
  
  stopifnot(length(networks) == 2)
  stopifnot(length(toms) == 2)
  stopifnot(length(genes_to_label) > 0)
  
  # Read in files (equivalent to object@datExpr slot of WGCNA object)
  datasets = WGCNAlist[networks]
  
  # Make common list of genes
  allCommonGenes=datasets[[1]]@datExpr$X[order(datasets[[1]]@datExpr$X)]
  
  # First extract the order from hierarchical clustering of TOM dissimilarity matrix
  orderList=list()
  positionList=list()
  for(element in seq_along(networks)){
    
    # Get TOM
    TOM=toms[[element]]
    dissTOM=1-TOM
    
    # Cluster TOM
    message("Clustering TOM tree ", element, "...")
    geneTree = flashClust(as.dist(dissTOM), method="average")
    
    # Compile data
    geneOrder=data.frame(Gene=datasets[[element]]@datExpr$X[geneTree$order], 
                         Color=datasets[[element]]@datExpr$dynamicColors[geneTree$order])
    geneOrder$order=1:length(allCommonGenes)
    sortedOrder=geneOrder[order(geneOrder$Gene),]
    orderList[[element]]=sortedOrder
  }
  
  # Combine results
  df=do.call(cbind, orderList)
  colnames(df)=c("one.gene","one.color", "one.order", "two.gene", "two.color", "two.order")
  df$Count=1
  head(df)
  df$one.gene=paste0("1_", df$one.gene)
  df$one.gene=factor(df$one.gene, levels=df$one.gene[order(df$one.order)])
  df$two.gene=paste0("2_", df$two.gene)
  df$two.gene=factor(df$two.gene, levels=df$two.gene[order(df$two.order)])
  colors=c(rev(df$one.color[order(df$one.order)]), rev(df$two.color[order(df$two.order)]))
  
  # Choose module to label
  df$module="other"
  df$module[allCommonGenes %in% genes_to_label]="labeled"
  df$module = factor(df$module, levels = c("other", "labeled"))
  
  # Set unlabeled genes to NAs to avoid labeling them, speeds up plotting as well
  temp1 = subset(df, module != "labeled")
  temp1$two.gene = NA
  temp2 = subset(df, module != "labeled")
  temp2$one.gene = NA
  
  new_df = rbind(subset(df, module == "labeled"), 
                 temp1, temp2)
  
  # Flow plot
  plt = ggplot(new_df, aes(y = Count, axis1 = one.gene, axis2 = two.gene)) +
      geom_flow(aes(fill = module), width=width, curve_type = "cubic", alpha = alpha, fill = color) +
      geom_stratum(width = width, fill = colors, size=0, alpha = 1) +
      ylab("Genes") + 
      theme(axis.ticks.x = element_blank(), panel.background = element_blank(),
            panel.grid.major = element_blank(), panel.grid.minor = element_blank())+
      geom_text(stat = "stratum", aes(label = after_stat(stratum)), size = 0) +
      scale_y_continuous(limits = c(0, nrow(df)))+
      scale_x_discrete(expand=c(0,0), limits = networks, labels = networks)
  
  return(plt)
}

#' BuildTOMFlowDF
#'
#' Preprocess for plotting a sankey flow diagram showing the movement of genes from one WGCNA
#' to another WGCNA. Uses the flashClust framework. 
#'
#' @param WGCNAlist list of WGCNA objects
#' @param networks list of network names of length 2
#' @param toms a list of TOM distance objects of length 2
#' @param genes_to_label genes to label across two networks
#' @param method linkage method to pass to flashClust for clustering, default is average linkage
#' 
#' @return a data.frame
#' 
#' @author Dario Tommasini, Xinye Li
#'
#' @import flashClust
#' @import WGCNA
#' @import stringr
#' @importFrom tidyr pivot_wider
#' @export
BuildTOMFlowDF <- function(WGCNAlist, networks, toms, genes_to_label, method = 'average') {
  
  stopifnot(length(networks) == length(toms))
  stopifnot(length(genes_to_label) > 0)
  
  datasets <- WGCNAlist[networks]
  allCommonGenes <- datasets[[1]]@datExpr$X[order(datasets[[1]]@datExpr$X)]
  
  # Calculate the ranking information for each network
  orderList <- list()
  colorList <- list()
  
  for(i in seq_along(networks)){
    TOM <- toms[[i]]
    dissTOM <- 1 - TOM
    
    message("Clustering TOM tree ", networks[i], "...")
    geneTree <- flashClust::flashClust(as.dist(dissTOM), method = method)
    
    geneOrder <- data.frame(
      Gene = datasets[[i]]@datExpr$X[geneTree$order],
      Color = datasets[[i]]@datExpr$dynamicColors[geneTree$order],
      Order = seq_along(allCommonGenes)
    )
    sortedOrder <- geneOrder[order(geneOrder$Gene), ]
    orderList[[i]] <- sortedOrder
    colorList[[i]] <- sortedOrder$Color
  }
  
  # Merge all orderList into one dataframe
  combined_df <- data.frame(Gene = orderList[[1]]$Gene)
  
  for(i in seq_along(orderList)){
    combined_df[[paste0("G", i)]] <- paste0(i, "_", orderList[[i]]$Gene)
    combined_df[[paste0("Color", i)]] <- colorList[[i]]
    combined_df[[paste0("Order", i)]] <- orderList[[i]]$Order
  }
  
  combined_df$Count <- 1
  
  # Construct the information of the module tag
  combined_df$module <- "other"
  combined_df$module[combined_df$Gene %in% genes_to_label] <- "labeled"
  combined_df$module <- factor(combined_df$module, levels = c("other", "labeled"))
  
  # Construct new_df (long format)
  long_df_list <- list()
  for(i in seq_along(networks)){
    temp_df <- combined_df[, c("Gene", paste0("G", i), paste0("Order", i), paste0("Color", i))]
    colnames(temp_df) <- c("Gene", "Label", "Order", "Color")
    temp_df$Network <- paste0("Network", i)
    temp_df$module <- combined_df$module
    long_df_list[[i]] <- temp_df
  }
  
  new_df <- do.call(rbind, long_df_list)
  
  # Change to wide format
  wide_df <- tidyr::pivot_wider(
    new_df,
    id_cols = Gene,
    names_from = Network,
    values_from = c(Label, Order, Color)
  )
  
  # Restore module information
  wide_df$module <- combined_df$module[match(wide_df$Gene, combined_df$Gene)]
  wide_df$Count <- 1
  wide_df <- as.data.frame(wide_df)
  
  return(wide_df)
}

#' PlotMultiNodesTOMflow
#'
#' Plots a sankey flow diagram showing the movement of genes from one WGCNA
#' to multi-WGCNA networks. Uses the ggalluvial framework. 
#'
#' @param TOMDF created by BuildTOMFlowDF
#' @param labels labels to use for the networks, default is Network1, Network2, etc.
#' @param alpha alpha of flows
#' @param width width of the strata
#' @param color color of flows
#' 
#' @return a ggplot object
#' 
#' @author Dario Tommasini, Xinye Li
#'
#' @import flashClust
#' @import ggalluvial
#' @import WGCNA
#' @import stringr
#' @export
PlotMultiNodesTOMflow <- function(TOMDF, labels = NULL, alpha = 0.1, width = 0.05, color = "black") {
  
  # Split data
  networks <- grep("^Label_Network", colnames(TOMDF), value = TRUE)
  orders <- grep("^Order_Network", colnames(TOMDF), value = TRUE)
  colors <- grep("^Color_Network", colnames(TOMDF), value = TRUE)
  
  # Equal the number of networks
  network_count <- length(networks)
  
  # Set labels to automatic labels if not specified
  if(is.null(labels)) labels = paste0("Network ", 1:network_count)
  
  if(network_count < 2) {
    stop("At least 2 networks are needed for drawing a Sankey plot.")
  }
  if(network_count > 5) {
    stop("Currently only 2-5 networks are supported.")
  }
  
  # Build basic df
  df_columns <- c(networks, orders, colors, "module", "Count")
  df <- TOMDF[, df_columns]
  
  # Sorting
  for (i in seq_along(networks)) {
    df[[networks[i]]] <- factor(df[[networks[i]]], 
                                levels = df[[networks[i]]][order(df[[orders[i]]])])
  }
  
  # Adjust connections
  new_dfs <- list()
  
  # Keep connections between labeled genes
  new_dfs[[1]] <- subset(df, module == "labeled")
  
  # Remove connections between others
  for (i in seq_along(networks)) {
    temp <- subset(df, module != "labeled")
    # Set all other nodes' genes as NA, only keep current node for only drawing the current node
    for (j in seq_along(networks)) {
      if (j != i) {
        temp[[networks[j]]] <- NA
      }
    }
    # print(head(temp))
    new_dfs[[length(new_dfs) + 1]] <- temp
  }
  
  # Combine the df
  new_df <- do.call(rbind, new_dfs)
  
  # Set colors
  all_node_colors <- list()
  for (i in seq_along(networks)) {
    sorted_df <- df[order(df[[orders[i]]]), ]
    all_node_colors[[i]] <- rev(sorted_df[[colors[i]]])
  }
  
  # combine colors
  node_colors <- unlist(all_node_colors)
  
  # Build plot
  plt <- ggplot(new_df)
  
  if (network_count == 2) {
    plt <- plt + aes(y = Count, axis1 = !!sym(networks[1]), axis2 = !!sym(networks[2]))
  } else if (network_count == 3) {
    plt <- plt + aes(y = Count, axis1 = !!sym(networks[1]), axis2 = !!sym(networks[2]), 
                     axis3 = !!sym(networks[3]))
  } else if (network_count == 4) {
    plt <- plt + aes(y = Count, axis1 = !!sym(networks[1]), axis2 = !!sym(networks[2]), 
                     axis3 = !!sym(networks[3]), axis4 = !!sym(networks[4]))
  } else if (network_count >= 5) {
    plt <- plt + aes(y = Count, axis1 = !!sym(networks[1]), axis2 = !!sym(networks[2]), 
                     axis3 = !!sym(networks[3]), axis4 = !!sym(networks[4]), 
                     axis5 = !!sym(networks[5]))
  }
  
  # Nodes & flows
  plt <- plt +
    geom_flow(aes(fill = module), width = width, curve_type = "cubic", alpha = alpha, fill = color) +
    geom_stratum(width = width, fill = node_colors, size = 0, alpha = 1) +
    ylab("Genes") +
    theme(
      axis.ticks.x = element_blank(),
      panel.background = element_blank(),
      panel.grid.major = element_blank(),
      panel.grid.minor = element_blank()
    ) +
    geom_text(stat = "stratum", aes(label = after_stat(stratum)), size = 0) +
    scale_y_continuous(limits = c(0, nrow(df))) +
    scale_x_discrete(
      expand = expansion(mult = c(0, 0.05)),
      limits = paste0("Network ", 1:network_count),
      labels = labels,
    )
  
  return(plt)
}

#' computeOverlapsFromWGCNA
#'
#' Computes overlap between the modules of two objects of class WGCNA
#'
#' @param dataset1 an object of class WGCNA to compare with dataset2
#' @param dataset2 an object of class WGCNA to compare with dataset1
#' 
#' @return Returns a data.frame showing the overlap results for modules from 
#' dataset1 with dataset2
#' 
#' @author Dario Tommasini
#'
#' @import stringr
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]]
#' computeOverlapsFromWGCNA(astrocyte_networks$EAE, astrocyte_networks$WT)
#' 
computeOverlapsFromWGCNA <- function(dataset1, dataset2) {
  
  # Check input
  stopifnot(inherits(dataset1, "WGCNA") & inherits(dataset2, "WGCNA"))
  
	datExpr1 <- dataset1@datExpr
	datExpr2 <- dataset2@datExpr
	treatDat <- datExpr1
	controlDat <- datExpr2
	
	# computation of overlaps
	all.genes = sort(unique(c(treatDat$X, controlDat$X)))
	sorted.modules <- data.frame(mod1 = treatDat$dynamicLabels[match(all.genes, treatDat$X)],
	                             mod2 = controlDat$dynamicLabels[match(all.genes, controlDat$X)])

	overlap.count <- table(sorted.modules$mod1, sorted.modules$mod2)
  
	pval = overlap.count
	for(column in seq_along(colnames(overlap.count))){
	  for(row in seq_along(rownames(overlap.count))){
	    pval[row,column]=phyper(overlap.count[row,column]-1,
	                            sum(overlap.count[row,]),
	                            sum(rowSums(overlap.count))-sum(overlap.count[row,]),
	                            sum(overlap.count[,column]),
	                            lower.tail=FALSE,
	                            log.p=FALSE)
	    if(pval[row,column]==0) pval[row,column]=.Machine$double.xmin
	  }
	}

	# Adjust for multiple comparisons
	pval.unlist = unlist(as.list(pval))
	pval.adj = p.adjust(pval.unlist, method='fdr')

	# Get module sizes
	mod1.size = table(sorted.modules$mod1)
	mod2.size = table(sorted.modules$mod2)

	# Summarize results
	output.df = reshape2::melt(overlap.count) 
	colnames(output.df) = c("mod1", "mod2", "overlap")
	output.df$mod1 = as.character(output.df$mod1)
	output.df$mod2 = as.character(output.df$mod2)
	output.df$p.value = pval.unlist
	output.df$p.adj = pval.adj
	output.df = output.df %>% arrange(mod1, mod2)
	output.df$mod1.size = mod1.size[match(output.df$mod1, names(mod1.size))]
	output.df$mod2.size = mod2.size[match(output.df$mod2, names(mod2.size))]

	# Return columns in proper order
	return(output.df[,c(1:2,6:7,3:5)] %>% arrange(mod1, mod2))
}

#' Module sankey diagram
#'
#' A plotting function that returns a plot
#'
#' @param WGCNAlist a data.frame resulting from a call to computeOverlapsFromWGCNA
#' @param comparisonList an object of class WGCNA to compare with dataset2
#' @param networks an object of class WGCNA to compare with dataset1
#' @param labels labels to show under each networks
#' @param alpha alpha value for the node tiles, default is 1
#' @param x.scale x spacing parameter
#' @param y.scale y spacing parameter
#' @param width width of the block
#' @param height height of the block
#' @param color.low default is cyan
#' @param color.high default is magenta
#' @param color.by color fill for nodes, either 'network', 'trait', or NULL
#' @param col color palette for node tiles, default is internal colors function
#' @param my_layout pass a custom layout for nodes? Default is null
#' @param use.padj use p.adj for edge width, default is false (use gene overlap value)
#' @param only.contiguous only show continuous overlaps? Default is TRUE
#' @param only.signif plot only significant overlap?
#' @param show.legend show the legend? 
#' @param scale.by.size scale the node height by the size of the module? Default is TRUE.
#' @param spacer space between the nodes
#' @param label.y adjust the y coordinate for the network labels, default is 1
#' @param label.size size of the node labels, default is 4
#' @param base_family font family for ggraph
#' @param p.adj.threshold adjusted p-value significance threshold for overlap
#' @param overlap.threshold threshold for number of genes overlapping
#'
#' @return Returns a ggalluvial diagram comparing two networks
#'
#' @author Dario Tommasini
#'
#' @import ggplot2
#' @import stringr
#' @import ggraph
#' @import igraph
#' 
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]]
#' results = list()
#' results$overlaps = iterate(astrocyte_networks, overlapComparisons, plot=FALSE)
#' ModuleFlowPlot(astrocyte_networks, results$overlaps, c('combined',  'EAE', 'WT'))
ModuleFlowPlot = function(WGCNAlist, 
                          comparisonList, 
                          networks, 
                          labels = NULL, 
                          alpha = 1, 
                          x.scale = 2, 
                          y.scale = 1, 
                          width = 0.3, 
                          height = 0.2,
                          color.by = c('trait', 'network', 'none'),
                          color.low = 'cyan', 
                          color.high = 'magenta',
                          col = NULL,
                          my_layout = NULL,
                          use.padj = FALSE,
                          only.contiguous = TRUE,
                          only.signif = TRUE,
                          show.legend = TRUE,
                          scale.by.size = TRUE, 
                          spacer = 10, 
                          label.y = 50,
                          label.size = 4,
                          base_family = 'Helvetica',
                          p.adj.threshold = 0.05, 
                          overlap.threshold = 10) {
  
  
  # Load ggraph 
  # requireNamespace("ggraph", quietly = TRUE)
  # ggraph::ggraph() # trigger namespace load and register guides
  
  # subset to modules
  stopifnot(all(networks %in% names(WGCNAlist)))
  WGCNAlist = WGCNAlist[networks]
  
  if(only.contiguous){
    full.overlaps = do.call(rbind, lapply(comparisonList, function(x) x$overlap))
    comparisons = lapply(seq_len(length(networks)-1), function(i) networks[c(i,i+1)])
    filteredOverlapList = do.call(rbind, lapply(seq_along(comparisons), function(i){
      df = subset(full.overlaps, startsWith(mod1, comparisons[[i]][[1]]) & startsWith(mod2, comparisons[[i]][[2]]))
      if(nrow(df) == 0) {
        df = subset(full.overlaps, startsWith(mod1, comparisons[[i]][[2]]) & startsWith(mod2, comparisons[[i]][[1]]))
        temp = df$mod1
        df$mod1 = df$mod2
        df$mod2 = temp
      }
      df$axis = i
      df
    }))
    
  } else {
    overlapList=lapply(comparisonList, function(x) x$overlap)
    overlapList=do.call(rbind, overlapList)
    filteredOverlapList=overlapList
    
    # remove networks not listed in "networks" 
    for(WGCNA in WGCNAlist[!names(WGCNAlist) %in% networks]){
      filteredOverlapList=filteredOverlapList[!filteredOverlapList$mod1 %in% WGCNA@outlierModules,]
      filteredOverlapList=filteredOverlapList[!filteredOverlapList$mod2 %in% WGCNA@outlierModules,]
    }
  }

  # Track modules
  admittedModules=unique(c(filteredOverlapList$mod1, filteredOverlapList$mod2))
  
  #make the igraph object
  graph=graph_from_data_frame(d=filteredOverlapList, directed = FALSE)
  
  # order vertices
  module_df = data.frame(module = as.vector(do.call(c, lapply(WGCNAlist, getModuleOrder))), 
                         trait = as.vector(do.call(c, lapply(WGCNAlist, function(object) object@trait$trait))))
  module_df = subset(module_df, module %in% admittedModules)
  V(graph)$name = module_df$module
  V(graph)$trait = module_df$trait
  
  #node and edge attributes
  V(graph)$network=str_split_fixed(V(graph)$name, "_", 2)[,1]
  # conditions=unique(V(graph)$network)
  
  # browser()
  
  # Module names
  color.by = match.arg(color.by)
  if(color.by == 'network') {
    V(graph)$color = V(graph)$network
    palette = colors(length(networks), random = TRUE)
  } else if(color.by == 'trait'){
    V(graph)$color = V(graph)$trait
    # print(unique(V(graph)$trait))
    palette = colors(length(unique(V(graph)$trait)), random = TRUE)
  } else if(color.by == 'none'){
    # White fill
    palette = c('white')
  } 
  
  if(!is.null(col)) {
    message('Using custom color palette passed thru `col`')
    palette = col
  }
  
  # browser()
  
  # Colors of modules by condition
  # if(is.null(col)) palette = colors(length(conditions), random = TRUE)
  
  # Module names by number
  V(graph)$name_clean = gsub('^0+', '', str_split_fixed(V(graph)$name, "_", 2)[,2])
  V(graph)$name_clean[V(graph)$name_clean == ''] = '0' # set zero module to '0'
  
  mod.size.list = lapply(WGCNAlist[networks], function(network){
    t(t(table(network@datExpr$dynamicLabels)))
  })
  module.size.df = do.call(rbind, mod.size.list)
  V(graph)$n.genes = module.size.df[V(graph)$name,] #rescale(module.size.df[V(graph)$name,], to = c(0,1))
  # print(V(graph)$n.genes) 
  
  # browser()
  
  # edge attributes
  if(use.padj) {
    E(graph)$weight = -log10(E(graph)$p.adj)
  } else {
    E(graph)$weight=E(graph)$overlap
  }
  E(graph)$width=rescale(E(graph)$weight, to=c(0,5))
  ealpha=rescale(-log10(E(graph)$p.adj), to=c(0,1))
  ecol=lapply(ealpha, function(x) rgb(1, 0, 0, x))
  E(graph)$color=unlist(ecol)
  E(graph)$alpha = ealpha
  
  # browser()
  # Significant only
  if(only.signif) {
    edges_remove = which(filteredOverlapList$p.adj > p.adj.threshold | filteredOverlapList$overlap < overlap.threshold) #GetSignificantOverlap(filteredOverlapList, ...)
    graph <- delete.edges(graph, edges_remove)
  }
  
  # browser()
  
  # Layout
  if(is.null(my_layout)){
    myCoords=list()
    for(i in seq_along(networks)){
      network = networks[[i]]
      n.modules = length(which(startsWith(admittedModules, network)))
      coords = cbind(i*x.scale, seq(1, 0, length.out = n.modules)*y.scale)
      if(scale.by.size){
        v1 = cumsum(mod.size.list[[network]])
        v2 = as.vector(mod.size.list[[network]])/2
        spaces = seq(0, length(v1)-1) * spacer
        y_positions = v1 + spaces - v2
        # print(v1)
        # print(v2)
        # print(spaces)
        coords = cbind(i*x.scale, ((-y_positions))*y.scale) #(rescale(-(y_positions+spaces)*y.scale, to = c(0,1))))
      }
      myCoords = append(myCoords, list(coords))
    }
    my_layout=do.call(rbind, myCoords)
  }
  
  labels_df <- data.frame(
    x = seq(1, length(networks))*x.scale,
    y = rep(label.y, length(networks)),
    text = networks
  )
  
  # browser()
  ggraph::ggraph(graph, layout = my_layout) + 
    # geom_edge_bend(aes(alpha = -log10(p.adj), width = overlap, color = -log10(p.adj)), strength = 0.5) +
    ggraph::geom_edge_link(aes(alpha = -log10(p.adj), width = overlap, color = -log10(p.adj))) +
    # geom_node_label(aes(label = name), 
    #                 label.padding = unit(0.2, "lines"), # controls rectangle size
    #                 label.r = unit(0, "lines"),         # removes rounded corners
    #                 fill = "white") +
    {if(scale.by.size) ggraph::geom_node_tile(aes(width = 1, height = n.genes, fill = color), alpha = alpha, color = 'black')}+
    {if(!scale.by.size) ggraph::geom_node_tile(aes(width = width, height = height, fill = color), alpha = alpha, color = 'black')}+
    # {if(!is.null(color.by)) geom_node_tile(aes(width = width, height = height, fill = color), alpha = alpha, color = 'black')}+
    scale_fill_manual(values = palette)+
    ggraph::geom_node_text(aes(label = name_clean), vjust = 0.5, size = label.size) +
    ggraph::scale_edge_width(range = c(0.5, 4)) +  # control min/max line thickness
    ggraph::scale_edge_color_gradient(low=color.low, high=color.high, guide = ggraph::guide_edge_colourbar())+
    geom_text(data = labels_df, aes(x = x, y = y, label = text),
              inherit.aes = FALSE, color = "black")+
    coord_cartesian(clip = 'off')+
    ggraph::theme_graph(base_family = base_family)
}

#' Module comparison plot
#'
#' A plotting function that returns a heatmap and barplot for a module
#'
#' @param overlapDf a data.frame resulting from a call to computeOverlapsFromWGCNA
#' @param dataset1 an object of class WGCNA to compare with dataset2
#' @param dataset2 an object of class WGCNA to compare with dataset1
#'
#' @return Returns a ggplot object (flowplot and heatmap) showing the module correspondence 
#' between two objects of class WGCNA
#'
#' @author Dario Tommasini
#'
#' @import ggplot2
#' @import ggalluvial
#' @import stringr
#' @importFrom cowplot plot_grid
#' 
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]]
#' overlapDf = computeOverlapsFromWGCNA(astrocyte_networks$EAE, astrocyte_networks$WT)
#' moduleComparisonPlot(overlapDf, astrocyte_networks$EAE, astrocyte_networks$WT)
#' 
moduleComparisonPlot <- function(overlapDf, dataset1, dataset2) {
  
  # Check input
  stopifnot(inherits(dataset1, "WGCNA") & inherits(dataset2, "WGCNA"))
  
	data=overlapDf
	name1=str_split_fixed(data$mod1,"_",2)[,1][[1]]
	name2=str_split_fixed(data$mod2,"_",2)[,1][[1]]
	data$mod1=str_split_fixed(data$mod1,"_",2)[,2]
	data$mod2=paste0(str_split_fixed(data$mod2,"_",2)[,2]," ") # add spaces to avoid ambiguous plot labels
	categories=unique(c(dataset1@trait$trait, dataset2@trait$trait))
	palette=colors(length(categories), random=FALSE)
	colors=palette[match(c(rev(dataset1@trait$trait), rev(dataset2@trait$trait)), categories)]
	totalGenes=sum(data$overlap)

 	# draw heatmap
 	heatmap <- moduleToModuleHeatmap(overlapDf)

	# draw flow plot
	flowPlot <- ggplot(data, aes(y = overlap, axis1 = mod1, axis3 = mod2)) +
		geom_flow(aes(fill = -log10(p.adj), alpha = -log10(p.adj)), width = .2, curve_type = "linear") +
		ylab("Genes") +
		scale_fill_gradient(name="Overlap", low="cyan", high="magenta")+
		theme(axis.ticks.x=element_blank(), 
		      legend.position = "none", 
		      axis.title.x = element_blank(),
		      axis.text.x=element_text(size=12),
		      axis.title.y=element_text(size=15),
		      panel.background = element_blank(), 
		      panel.grid.major = element_blank(), 
		      panel.grid.minor = element_blank())+
	  geom_stratum(width = .2, fill = colors) +
		annotate("text", 
		         x=2.2, 
		         y=(1:length(categories))*(totalGenes/3)/length(categories),
		         label=categories, 
		         vjust=0, 
		         hjust=0, 
		         color=palette)+
		geom_text(stat = "stratum", aes(label = gsub("^0+", "", after_stat(stratum))), size = 3, min.y=50) +
		coord_cartesian(xlim = c(0.9, 2.5), 
		                clip = "off") +
 		scale_x_discrete(expand=c(0,0), 
 		                 limits = c("Mod1", "Mod2"), 
 		                 labels=c(name1,name2))

	plot = plot_grid(flowPlot, NULL, heatmap, rel_widths = c(1, 0.2, 2.5), nrow = 1)
	
	return(plot)
}

#' Get significant overlap
#'
#' A function that subsets an overlap comaprisons data.frame. 
#'
#' @param df a data.frame resulting from a call to computeOverlapsFromWGCNA
#' @param p.adj.threshold an object of class WGCNA to compare with dataset2
#' @param overlap.threshold an object of class WGCNA to compare with dataset1
#'
#' @return Returns a data.frame with significant overlaps
#'
#' @author Dario Tommasini
#' 
#' @export
GetSignificantOverlap = function(df, p.adj.threshold = 0.05, overlap.threshold = 10){
  subset(df, p.adj < p.adj.threshold & overlap > overlap.threshold)
}


#' Module sankey diagram
#'
#' A plotting function that returns a sankey plot comparing two networks. 
#' This is a deprecated function, please use ModuleFlowPlot instead.  
#'
#' @param overlapDf a data.frame resulting from a call to computeOverlapsFromWGCNA
#' @param dataset1 an object of class WGCNA to compare with dataset2
#' @param dataset2 an object of class WGCNA to compare with dataset1
#' @param only.signif plot only significant overlap?
#' @param show.legend show the legend? 
#' @param col color palette, default is internal colors function
#' @param ... params to GetSignificantOverlap function for thresholds
#'
#' @return Returns a ggalluvial diagram comparing two networks
#'
#' @author Dario Tommasini
#'
#' @import ggplot2
#' @import ggalluvial
#' @import stringr
#' @importFrom cowplot plot_grid
#' 
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]]
#' overlapDf = computeOverlapsFromWGCNA(astrocyte_networks$EAE, astrocyte_networks$WT)
#' ModuleFlowPlot2Way(overlapDf, astrocyte_networks$EAE, astrocyte_networks$WT)
ModuleFlowPlot2Way <- function(overlapDf, dataset1, dataset2, only.signif = TRUE, 
                               show.legend = TRUE, col = NULL, ...) {
  
  # Check input
  stopifnot(inherits(dataset1, "WGCNA") & inherits(dataset2, "WGCNA"))
  
  data = overlapDf
  
  # Filter overlaps? 
  if(only.signif) {
    # data$overlap[data$overlap < 10] = 0
    # data$p.adj[data$p.adj > 1e-2] = 1
  }
  # print(data)
  
  name1=str_split_fixed(data$mod1,"_",2)[,1][[1]]
  name2=str_split_fixed(data$mod2,"_",2)[,1][[1]]
  data$mod1=str_split_fixed(data$mod1,"_",2)[,2]
  data$mod2=paste0(str_split_fixed(data$mod2,"_",2)[,2]," ") # add spaces to avoid ambiguous plot labels
  categories=unique(c(dataset1@trait$trait, dataset2@trait$trait))
  if(is.null(col)) palette=colors(length(categories), random=FALSE) else palette = col
  colors=palette[match(c(rev(dataset1@trait$trait), rev(dataset2@trait$trait)), categories)]
  totalGenes=sum(data$overlap)
  
  # data$mod2[data$overlap < 10] = NA
  # data$mod1[data$overlap < 10] = NA
  
  # draw flow plot
  flowPlot <- ggplot(data, aes(y = overlap, axis1 = mod1, axis2 = mod2)) +
    geom_flow(aes(fill = -log10(p.adj), alpha = -log10(p.adj)), width = .2, curve_type = "linear") +
    ylab("Genes") +
    scale_fill_gradient(name="Overlap", low="cyan", high="magenta")+
    theme(axis.ticks.x=element_blank(), 
          legend.position = "none", 
          axis.title.x = element_blank(),
          axis.text.x=element_text(size=12),
          axis.title.y=element_text(size=15),
          panel.background = element_blank(), 
          panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank())+
    geom_stratum(width = .2, fill = colors) +
    {if(show.legend) 
      annotate("text", 
                 x=2.2, 
                 y=(1:length(categories))*(totalGenes/3)/length(categories),
                 label=categories, 
                 vjust=0, 
                 hjust=0, 
                 color=palette)}+
    geom_text(stat = "stratum", aes(label = gsub("^0+", "", after_stat(stratum))), size = 3, min.y=50)+
    coord_cartesian(xlim = c(0.9, 2.5), 
                    clip = "off") +
    scale_x_discrete(expand=c(0,0), 
                     limits = c("Mod1", "Mod2"), 
                     labels=c(name1,name2))
  
  return(flowPlot)
}

# for study of module member allocation over a continuous trait ie time
continuousFlowPlot <- function(WGCNAlist){

	allGenes=unique(ulist(lapply(WGCNAlist, function(WGCNAobject) WGCNAobject@datExpr$X)))
	df=data.frame(Genes=allGenes)
	for(WGCNA in WGCNAlist){
		df=cbind(df, WGCNA@datExpr$dynamicLabels[match(allGenes, WGCNA@datExpr$X)])
	}
	names(df)=c("Genes",names(WGCNAlist))
	head(df)

	combinations=apply(df[,2:5], 1, function(x) x)
	geneOverlap=list()
	for(element in 1:ncol(combinations)){
		geneOverlap[[element]]=nrow(df[apply(df[,2:5], 1, function(x) all(x==combinations[,element])),])
	}
	data=as.data.frame(cbind(geneOverlap, t(combinations)))
	colnames(data)=c("geneOverlap", paste0("mod", 1:length(WGCNAlist)))

	uniqueData=unique(data)

	mod1=(sort(unique(WGCNAlist[[1]]@datExpr$dynamicLabels)))
	mod2=(sort(unique(WGCNAlist[[2]]@datExpr$dynamicLabels)))
	mod3=(sort(unique(WGCNAlist[[3]]@datExpr$dynamicLabels)))
	mod4=(sort(unique(WGCNAlist[[4]]@datExpr$dynamicLabels)))

	n1=length(mod1)
	n2=length(mod2)
	n3=length(mod3)
	n4=length(mod4)

	finalData=data.frame(mod1=unlist(lapply(mod1, function(x) rep(x, n2*n3*n4))),
		mod2=rep(unlist(lapply(mod2, function(x) rep(x, n3*n4))), n1),
		mod3=rep(unlist(lapply(mod3, function(x) rep(x, n4))), n1*n2),
		mod4=rep(mod4, n1*n2*n3), stringsAsFactors=FALSE)

	finalData$identifier=apply(finalData, 1, function(x) paste0(x, sep="_"))

	ggplot(uniqueSortedData, aes(y = geneOverlap, axis1 = mod1, axis2 = mod2, axis3 = mod3, axis4=mod4)) +
		geom_flow(aes(fill = geneOverlap), width = .4, curve_type = "linear") + ylab("Genes") +
		scale_fill_manual(values=c("white","red")) +
		theme(axis.ticks.x=element_blank(),legend.position = "none", axis.text.x=element_text(size=12),
			axis.title.y=element_text(size=15), panel.background = element_blank(),
			panel.grid.major = element_blank(), panel.grid.minor = element_blank())+
		geom_stratum(width = .4) +
		geom_text(stat = "stratum", aes(label = after_stat(stratum)), size = 3, min.y=250) +
		scale_x_discrete(expand=c(0,0), limits = c("Mod2", "Mod3", "Mod1", "Mod4"), labels=names(WGCNAlist))

	if(length(WGCNAlist)==4){
		mod1=list()
		mod2=list()
		mod3=list()
		mod4=list()
		genes=list()
		element=1
		for(one in sort(unique(WGCNAlist[[1]]@datExpr$dynamicLabels))){
	        for(two in sort(unique(WGCNAlist[[2]]@datExpr$dynamicLabels))){
                for(three in sort(unique(WGCNAlist[[3]]@datExpr$dynamicLabels))){
               		for(four in sort(unique(WGCNAlist[[4]]@datExpr$dynamicLabels))){
                		mod1[[element]]=one
                        mod2[[element]]=two
                        mod3[[element]]=three
                        mod4[[element]]=four
                        element=element+1
                	}
                }
			}
		}

	data=data.frame(genes=unlist(genes), mod1=unlist(mod1), mod2=unlist(mod2), mod3=unlist(mod3), mod4=unlist(mod4))
	data$ofInterest=FALSE
	data$ofInterest[as.character(data$mod3)==as.character(args[7])]=TRUE

	} else {
		error("only WGCNAlist length of four implemented at the moment")
	}
}

#' Module to module heatmap
#'
#' Returns a heatmap where color corresponds to FDR-adjusted overlap (hypergeometric test) and the label corresponds to the number of overlapping genes
#'
#' @param comparisonDf the data.frame output of computeOverlapFromWGCNA
#' @param dataset1 optional; WGCNA object for dataset 1
#' @param dataset2 optional; WGCNA object for dataset 2
#' @param trait1 optional; subset to modules correlated to this trait for dataset 1
#' @param trait2 optional; subset to modules correlated to this trait for dataset 2
#' @param list1 subset to this list of modules for dataset 1
#' @param list2 subset to this list of modules for dataset 2
#' @param filterByTrait only plot for modules that correlate with some trait?
#' @param alphaLevel the alpha level of significance for module-trait correlation, defaults to 0.05
#'
#' @return A ggplot object
#'
#' @import stringr
#' @import ggplot2
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]]
#' overlapDf = computeOverlapsFromWGCNA(astrocyte_networks$EAE, astrocyte_networks$WT)
#' moduleToModuleHeatmap(overlapDf)
#' 
moduleToModuleHeatmap <- function(comparisonDf, dataset1=NULL, dataset2=NULL, trait1=NULL, trait2=NULL, list1=NULL, list2=NULL, filterByTrait=FALSE, alphaLevel=0.05){
  
  # Check input
  stopifnot(inherits(comparisonDf, "data.frame"))
  
	# filter by trait if desired
	if(!is.null(dataset1)){
		if(!is.null(list1)){
			mod1ToKeep=list1
			comparisonDf=comparisonDf[comparisonDf$mod1 %in% mod1ToKeep,]
		} else if(!is.null(trait1)){
			association1=paste0("p.value.", trait1)
			index=which(colnames(dataset1@trait)==association1)
			mod1ToKeep=gsub("ME", "", dataset1@trait$Module[dataset1@trait[,index]<alphaLevel])
			comparisonDf=comparisonDf[comparisonDf$mod1 %in% mod1ToKeep,]
		} else if(filterByTrait){
			mod1ToKeep=gsub("ME", "", dataset1@trait$Module[dataset1@trait$p.value.interest_trait_code<alphaLevel])
			comparisonDf=comparisonDf[comparisonDf$mod1 %in% mod1ToKeep,]
		}
	}
	if(!is.null(dataset2)){
		if(!is.null(list2)){
			mod2ToKeep=list2
			comparisonDf=comparisonDf[comparisonDf$mod2 %in% mod2ToKeep,]
		} else if(!is.null(trait2)){
			association2=paste0("p.value.", trait2)
			index=which(colnames(dataset2@trait)==association2)
			mod2ToKeep=gsub("ME", "", dataset2@trait$Module[dataset2@trait[,index]<alphaLevel])
			comparisonDf=comparisonDf[comparisonDf$mod2 %in% mod2ToKeep,]
		} else if(filterByTrait){
			mod2ToKeep=gsub("ME", "", dataset2@trait$Module[dataset2@trait$p.value.interest_trait_code<alphaLevel])
			comparisonDf=comparisonDf[comparisonDf$mod2 %in% mod2ToKeep,]
		}
	}

	columns=str_split_fixed(comparisonDf$mod1,"_", 2)[,1][[1]]
	rows=str_split_fixed(comparisonDf$mod2,"_",2)[,1][[1]]
	comparisonDf$mod1=gsub("_"," ", gsub("^0+", "", str_split_fixed(comparisonDf$mod1,"_",2)[,2]))
	comparisonDf$mod2=gsub("_"," ", gsub("^0+", "", str_split_fixed(comparisonDf$mod2,"_",2)[,2]))

	plot = ggplot(comparisonDf, aes(x = factor(mod1, levels=(unique(mod1))),
	                         y = factor(mod2, levels=rev(unique(mod2))),
 				fill = (-log10(p.adj)), label = overlap)) +
				geom_tile(color = "black") +
				scale_fill_gradient(name="-log10(FDR)", low = "white",
					high = "red", na.value="red", #limits=c(0, 50),
					guide = guide_colorbar(frame.colour = "black", frame.linewidth = 0.5,
							ticks.linewidth = 0.5, ticks.colour = "black")) +
				geom_text(aes(label = overlap), color = "black", size=2) +
				labs(x=columns, y=rows) +
				theme_classic()+
	      ggtitle("All matches") +
				theme(axis.text.x = element_text(angle = 0, hjust=(0.5)), #angle = 90, hjust=1, vjust=(0.5)),
					axis.text.y = element_text(hjust=1), plot.title = element_text(hjust = 0.5),
					panel.background=element_blank(), axis.line=element_blank(),
					axis.ticks=element_blank(), legend.key.size=unit(4, "mm")) +
				coord_fixed()
	
	return(plot)
}

#' Best matching modules
#'
#' Find all the modules from dataset1 that have a best match to a module in dataset2
#' if that module in dataset2 is also a best match to the module in dataset1
#'
#' @param comparisonList a list with an elemnt "overlap", which is a data.frame resulting from a call to computeOverlapsFromWGCNA
#' @param plot whether to generate a heatmap; default is TRUE
#'
#' @return A ggplot object
#' 
#' @author Dario Tommasini
#'
#' @import ggplot2
#' @import stringr
#' @importFrom magrittr %>%
#' @importFrom dplyr arrange
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]]
#' comparisonList = list()
#' comparisonList$overlaps = computeOverlapsFromWGCNA(astrocyte_networks$EAE, astrocyte_networks$WT)
#' bidirectionalBestMatches(comparisonList)
#' 
bidirectionalBestMatches <- function(comparisonList, plot=TRUE){
  
  # Check input
  stopifnot(inherits(comparisonList, "list"))
  
  comparison=comparisonList
	name1=str_split_fixed(comparison$overlap$mod1,"_",2)[,1][[1]]
	name2=str_split_fixed(comparison$overlap$mod2,"_",2)[,1][[1]]
	comparison$overlap$mod1=str_split_fixed(comparison$overlap$mod1,"_",2)[,2]
	comparison$overlap$mod2=str_split_fixed(comparison$overlap$mod2,"_",2)[,2]
	columns=(unique(comparison$overlap$mod1))
	rows=(unique(comparison$overlap$mod2))
	padj_matrix=matrix(unlist(comparison$overlap$p.adj), ncol=length(columns),nrow=length(rows))
	colnames(padj_matrix)=columns
	rownames(padj_matrix)=rows
	bestMod1=list()
	bestMod2=list()
	p.adj=list()
	count=1
	for(column in 1:length(columns)){
		for(row in 1:length(rows)){
			if(which.min(padj_matrix[,column])==row & which.min(padj_matrix[row,])==column){
				bestMod1[[count]]=columns[[column]]
				bestMod2[[count]]=rows[[row]]
				p.adj[[count]]=min(padj_matrix[,column])
				count=count+1
			}
		}
	}
	bestMatches=data.frame(mod1=unlist(bestMod1), mod2=unlist(bestMod2), p.adj=unlist(p.adj))
	bestMatchesSorted=bestMatches %>% arrange(p.adj)
	subset_padj_matrix=padj_matrix[bestMatchesSorted$mod2,bestMatchesSorted$mod1]
	subsetComparisonDf=comparison$overlap[comparison$overlap$mod1 %in% bestMatches$mod1 &
											comparison$overlap$mod2 %in% bestMatches$mod2,]
	if(plot){
		plt = ggplot(subsetComparisonDf, 
		             aes(x = factor(mod1, levels=bestMatchesSorted$mod1),
		                y = factor(mod2, levels=rev(bestMatchesSorted$mod2)),
 				fill = (-log10(p.adj)))) +
				geom_tile(color = "black") +
			  scale_x_discrete(labels = function(x) gsub("^0+", "", x))+
				scale_y_discrete(labels = function(x) gsub("^0+", "", x))+
				scale_fill_gradient(name="-log10(FDR)", low = "white",
				                      high = "red", na.value="red", #limits=c(0, 50),
				                      guide = guide_colorbar(frame.colour = "black", frame.linewidth = 0.5,
				                                             ticks.linewidth = 0.5, ticks.colour = "black")) +
				geom_text(aes(label = overlap), color = "black") +
				labs(x=name1, y=name2) +
			  ggtitle("Best matches") +
  		  theme(axis.text.x = element_text(angle = 0, hjust=(0.5)), #angle = 90, hjust=1, vjust=(0.5)),
  		        axis.text.y = element_text(hjust=1), plot.title = element_text(hjust = 0.5),
  		        panel.background=element_blank(), axis.line=element_blank(),
  		        axis.ticks=element_blank(), legend.key.size=unit(4, "mm"))
				coord_fixed()
		print(plt)
	}
	colnames(bestMatchesSorted)=c(name1, name2, "p.adj")
	return(bestMatchesSorted)
}

#' Overlap comparisons
#'
#' Compares modules between two objects of type WGCNAobjects
#' within a WGCNAobject list given the indices. Recommended to be used in 
#' conjunction with the iterate function. 
#'
#' @param comparisonList a list passed by the iterate function
#' @param WGCNAlist list of objects of class WGCNA
#' @param first index of first WGCNA object
#' @param second index of second WGCNA object
#' @param element element position in the comparison list (passed by iterate function)
#' @param plot generate plots?
#' @param write write results to file?
#' 
#' @return A list, in which the first element is a data.frame showing the 
#' overlap results and the second element is a data.frame showing the best 
#' matching modules between the two WGCNA objects. 
#'
#' @author Dario Tommasini
#'
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]]
#' results = list()
#' results$overlaps = iterate(astrocyte_networks, overlapComparisons, plot=FALSE)
#' 
overlapComparisons <- function(comparisonList, WGCNAlist, first, second, element, plot=TRUE, write=FALSE){
  
  # Check input
  stopifnot(inherits(comparisonList, "list"))
  stopifnot(inherits(WGCNAlist[[1]], "WGCNA"))
  
	comparisonList[[element]]=list()
	comparisonList[[element]]=append(comparisonList[[element]],
					list(computeOverlapsFromWGCNA(WGCNAlist[[first]], WGCNAlist[[second]])))
	names(comparisonList[[element]])=append(names(comparisonList[[element]]), c("overlap"))
	names(comparisonList)[[element]]=paste0(names(WGCNAlist)[[first]], "_vs_", names(WGCNAlist)[[second]])
	if(write) write.csv(comparisonList[[element]]$overlap, paste0(names(WGCNAlist)[[first]], "_vs_", names(WGCNAlist)[[second]], ".csv"), row.names=FALSE)
	message("\n#### comparing ", names(WGCNAlist)[[first]], " and ", names(WGCNAlist)[[second]], " ####\n")
	if(plot){
		print(
		  moduleComparisonPlot(comparisonList[[element]]$overlap,
						WGCNAlist[[first]],
						WGCNAlist[[second]])
		  )
	}
	comparisonList[[element]]=append(comparisonList[[element]],
								list(bidirectionalBestMatches(comparisonList[[element]],
									plot=plot)))
	names(comparisonList[[element]])[[2]]=("bestMatches")
	
	return(comparisonList)
}

#' Permutation test results
#'
#' The results of running the PreservationPermutationTest in the astrocyte
#' vignette. This is provided since this function is quite slow. Please see
#' the astrocyte vignette for more details. 
#'
#' @docType data
#' @keywords datasets
#' @name permutationTestResults
#' @usage data(permutationTestResults)
#' @format A list of data.frames containing preservation results for each 
#' permutation
NULL

performWGCNA <- function(datExpr, traitData, identifier, alphaLevel=0.05, write=FALSE, plot=FALSE, ...){
  arguments=list(...)
  
  # Set network type to WGCNA's default unsigned networkType if not defined in constructNetworks function
  if(is.null(arguments$networkType)) arguments$networkType = "unsigned"
  
  datExpr = t(cleanDatExpr(datExpr, checkGenesSamples = TRUE))
  
  if('saveTOMs' %in% names(arguments)){
    if(arguments$saveTOMs == TRUE) {
      my_net = blockwiseModules(t(datExpr), saveTOMFileBase = identifier, ...)
    } else {
      my_net = blockwiseModules(t(datExpr), ...)
    }
  } else {
    my_net = blockwiseModules(t(datExpr), ...)
  }
  
  degrees1=intramodularConnectivity.fromExpr(t(datExpr), my_net$colors,
                                            networkType=arguments$networkType, 
                                            power=arguments$power)

  dynamicColors=WGCNA::labels2colors(my_net$colors)
  dynamicLabels=paste(identifier, "_", str_pad(my_net$colors, 3, pad="0"), sep="")
  summary = cbind(data.frame(X = rownames(datExpr), datExpr), degrees1, dynamicLabels, dynamicColors)
  if(write) write.csv(summary, file=paste0(identifier, "_summary.csv"), row.names=FALSE)
  myWGCNA <- new("WGCNA", datExpr=summary, conditions=traitData)
	myWGCNA=findModuleEigengenes(myWGCNA, write=write)
	myWGCNA=traitCor(myWGCNA, write=write)
	myWGCNA=findBestTrait(myWGCNA, alphaLevel = alphaLevel) # Edit 4/27/25: triggers error if there is no trait variance, so skip in that case
	myWGCNA=findOutlierModules(myWGCNA)
	if(plot) plotModules(myWGCNA, mode="PC1")
	return(myWGCNA)
}

#returns the indices of modules that are driven by single samples (outliers)
findOutlierModules <- function(WGCNAobject, byName=TRUE, method="Var", varCutoff=0.01, IQRcutoff=4, zScoreCutoff=4){
        modules=gsub("ME", "", rownames(WGCNAobject@moduleEigengenes))
        if(method=="zScore"){
                ME.zScores=zScoreMatrix(WGCNAobject@moduleEigengenes)
                if(byName){
                        outlierModules=modules[apply(ME.zScores, 1, function(x) any(abs(x) > zScoreCutoff))]
                } else {
                        outlierModules=which(apply(ME.zScores, 1, function(x) any(abs(x) > zScoreCutoff)))
                }
        }
        if(method=="IQR"){
                if(byName){
                        outlierModules=modules[apply(WGCNAobject@moduleEigengenes, 1, function(MEs){
                                any(MEs < quantile(MEs)[[2]]-IQRcutoff*IQR(MEs)) | any(MEs > quantile(MEs)[[4]]+IQRcutoff*IQR(MEs))
                        })]
                } else {

                }
        }
        if(method=="Var"){
                varWithMax=apply(WGCNAobject@moduleEigengenes, 1, range)
                varWithoutMax=apply(WGCNAobject@moduleEigengenes, 1, function(x) {
                        x=x[-which.max(abs(x))]; var(x)
                        })
                outlierModules=modules[varWithoutMax<varCutoff]
        }
        WGCNAobject@outlierModules=outlierModules
        WGCNAobject
}

findBestTrait <- function(WGCNAobject, alphaLevel=0.05, p.adjust=FALSE, write=FALSE) {
  traitTable=WGCNAobject@trait
  
  # Check for NAs (e.g. there is no sample variance)
  if(anyNA(traitTable)){
    traitTable$trait = "None"
    traitTable$log10Pvalue = NA
    traitTable$Module = gsub("ME", "", traitTable$Module)
  } else {
    pval_df = traitTable[,which(startsWith(colnames(traitTable),"p.value")),drop = FALSE]
    group=apply(pval_df, 1, which.min)
    traitTable$trait=gsub("p.value.", "", colnames(pval_df))[group]
    bestPvalues=apply(pval_df, 1, function(x) x[[which.min(x)]])
    bestPvalues=as.numeric(bestPvalues)
    traitTable$log10Pvalue= -log10(bestPvalues)
    traitTable$trait[traitTable$log10Pvalue < (-log10(as.numeric(alphaLevel)))]="None"
    # traitTable$Module=gsub("ME", "", traitTable$Module) # Edit on 11/25/2025: this line no longer seems necessary
  }
  
  WGCNAobject@trait=traitTable
  return(WGCNAobject)
}

traitCor <- function(WGCNAobject, write=FALSE){
	moduleEigengenes=t(WGCNAobject@moduleEigengenes)
	datExpr2=cleanDatExpr(WGCNAobject@datExpr)
	traitData=WGCNAobject@conditions
  identifier=name(WGCNAobject)
	nSamples=nrow(datExpr2)
	if(any(is.na(match(rownames(datExpr2), traitData$Sample)))){
	  stop(
	    "Could not match sample names between `datExpr` and `sampleTable`.\n\n",
	    "Some sample names in `datExpr`:\n",
	    paste0("  - ", head(rownames(datExpr2)), collapse = "\n"), "\n\n",
	    "Some sample names in `sampleTable`:\n",
	    paste0("  - ", head(traitData$Sample), collapse = "\n"), "\n\n",
	    "Note: In R, dashes (`-`) in column names are automatically converted to periods (`.`).\n",
	    "Please ensure the sample names match after this conversion."
	  )
	}
	Traits=traitData[match(rownames(datExpr2), traitData$Sample), -1, drop = FALSE]
  rownames(Traits)=traitData[match(rownames(datExpr2), traitData$Sample),c("Sample")]
  datTraits=as.data.frame(Traits)
  moduleTraitCorL = cor(moduleEigengenes, datTraits, use = "p");
  moduleTraitPvalueL = corPvalueStudent(moduleTraitCorL, nSamples);
  colnames(moduleTraitPvalueL) = paste0("p.value.", colnames(moduleTraitCorL));
  traitCor=cbind(Module=gsub("ME", "", rownames(moduleTraitCorL)), moduleTraitCorL, moduleTraitPvalueL)
  rownames(traitCor)=seq_len(nrow(traitCor))
  if(write) write.csv(traitData, paste0(identifier,"_conditions.csv"), row.names=FALSE)
  if(write) write.csv(traitCor, paste0(identifier,"_TraitCor.csv"), row.names=FALSE)
  WGCNAobject@trait=as.data.frame(traitCor)
  return(WGCNAobject)
}

findModuleEigengenes <- function(WGCNAobject, write=FALSE){
  datExpr2=WGCNAobject@datExpr
  identifier=name(WGCNAobject)
	moduleEigengenes=moduleEigengenes(cleanDatExpr(datExpr2), colors = datExpr2$dynamicLabels, nPC=1)$eigengenes
  moduleEigengenes=as.data.frame(t(moduleEigengenes))
	rownames(moduleEigengenes)=gsub("ME", "", rownames(moduleEigengenes))
	if(write) write.csv(moduleEigengenes, paste0(identifier,"_moduleEigengenes.csv"), row.names=TRUE)
	WGCNAobject@moduleEigengenes=moduleEigengenes
	return(WGCNAobject)
}

plotModules <- function(WGCNAobject, mode="PC1"){
	datExpr2=WGCNAobject@datExpr
	traitData=WGCNAobject@conditions
	identifier=name(WGCNAobject)
	pdf(paste0(identifier, "_moduleExpression.pdf"))
        for(module in sort(unique(datExpr2$dynamicLabels))){
                moduleGenes=datExpr2$X[datExpr2$dynamicLabels==module]
                print(moduleExpressionPlot(WGCNAobject, moduleGenes, mode=mode, title=module))
        }
        dev.off()
}

#' constructNetworks: Construct all the weighted gene correlation networks
#'
#' A high level function that returns all networks
#' possible for a given experimental design
#'
#' @param datExpr either a SummarizedExperiment object or data.frame with genes are rows and samples as columns
#' @param sampleTable data.frame with sample names in first column and sample traits in the second and third column. First column should be called "Sample"
#' @param conditions1 first design conditions, ie healthy/disease
#' @param conditions2 second design conditions, ie frontal lobe/temporal lobe
#' @param write write results out to files? 
#' @param alphaLevel significance value passed to findBestTrait function, default is 0.05 
#' @param plot plot modules? Default is false
#' @param detectNumbers passed to makeTraitTable2; if you have any numeric traits, make sure this is TRUE so that these get detected. 
#' @param ... Arguments to pass to blockwiseModules function 
#'
#' @return A list of WGCNA objects, ie level one, two, and three networks. 
#' 
#' @author Dario Tommasini
#'
#' @import stringr
#' @import readr
#' @import WGCNA
#' @import flashClust
#' @import SummarizedExperiment
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' autism_se = eh_query[["EH8219"]]
#' set.seed(1)
#' autism_se = autism_se[sample(rownames(autism_se), 500),]
#' sampleTable = colData(autism_se)
#' conditions1 = unique(sampleTable[,2])
#' conditions2 = unique(sampleTable[,3])
#' autism_networks = constructNetworks(autism_se, sampleTable, conditions1[[1]], conditions2[[1]], 
#'   networkType = "signed", TOMType = "unsigned", 
#'   power = 10, minModuleSize = 100, maxBlockSize = 25000,
#'   reassignThreshold = 0, minKMEtoStay = 0, mergeCutHeight = 0,
#'   numericLabels = TRUE, pamRespectsDendro = FALSE, 
#'   deepSplit = 4, verbose = 3)
#' autism_networks[["combined"]]
#' 
constructNetworks <- function(datExpr, sampleTable, conditions1, conditions2, write=FALSE, alphaLevel=0.05, plot=FALSE, detectNumbers = TRUE, ...){

  # Check input data format
  stopifnot(inherits(datExpr, "SummarizedExperiment") | inherits(datExpr, "data.frame"))
  stopifnot("Sample" %in% colnames(sampleTable))
  
  # Put data in expected format, with genes in first column names "X"
  if(inherits(datExpr, "SummarizedExperiment")){
    datExpr = data.frame(X = rownames(assays(datExpr)[[1]]), assays(datExpr)[[1]])
  } else if(inherits(datExpr, "data.frame")){
    datExpr = data.frame(X = rownames(datExpr), datExpr)
  }
  
  # Make trait tables
	conditions1TraitTable=makeTraitTable2(sampleTable, 3, detectNumbers = detectNumbers) #subset by conditions1, resolve conditions2
	conditions2TraitTable=makeTraitTable2(sampleTable, 2, detectNumbers = detectNumbers) #subset by conditions2, resolve conditions1
	combinedTraitTable=cbind(conditions1TraitTable, conditions2TraitTable[,-1])

	myNetworks=list()
	if(write) {
	  dir.create("combined")
	  setwd("combined")
	}
	
	message("### Working on combined network ###")
	myNetworks = append(myNetworks, performWGCNA(datExpr, combinedTraitTable, "combined", alphaLevel = alphaLevel, plot = plot, ...))
	if(write) setwd("..")
	
	# first dimension
	for(trait in unique(conditions1)){
	  if(write) {
	    dir.create(trait)
		  setwd(trait)
	  }
	  message("### Working on ", trait, " network ###")
		myNetworks=append(myNetworks, performWGCNA(datExpr[,c("X", sampleTable$Sample[sampleTable[,2]==trait])], conditions1TraitTable[sampleTable[,2]==trait,], trait, plot=plot, ...))
		if(write) setwd("..")
	}

	# second dimension
	for(trait in unique(conditions2)){
	  if(write) {
	    dir.create(trait)
		  setwd(trait)
	  }
	  message("### Working on ", trait, " network ###")
		myNetworks=append(myNetworks, performWGCNA(datExpr[,c("X", sampleTable$Sample[sampleTable[,3]==trait])], conditions2TraitTable[sampleTable[,3]==trait,], trait, plot=plot, ...))
		if(write) setwd("..")
	}

	names(myNetworks)=c("combined", conditions1, conditions2)
	return(myNetworks)
}

library(multiWGCNA)
library(ExperimentHub)
eh = ExperimentHub()

# Note: this requires the SummarizedExperiment package to be installed
eh_query = query(eh, c("multiWGCNAdata"))
autism_se = eh_query[["EH8219"]]

# Collect the metadata in the sampleTable
sampleTable = colData(autism_se)

# Randomly sample 2000 genes from the expression matrix
set.seed(1)
autism_se = autism_se[sample(rownames(autism_se), 1500),]

# Check the data
assays(autism_se)[[1]][1:5, 1:5]
sampleTable

# Define our conditions for trait 1 (disease) and 2 (brain region)
conditions1 = unique(sampleTable[,2])
conditions2 = unique(sampleTable[,3])

# Construct the combined networks and all the sub-networks (autism only, controls only, FC only, and TC only)
# The input data can be either a SummarizedExperiment object or a data.frame with genes as rows and samples as columns

test_that("Can run on WGCNA on SummarizedExperiment", {
  autism_networks = constructNetworks(autism_se, sampleTable, conditions1[[1]], conditions2[[1]],
                                      networkType = "unsigned", power = 10,
                                      minModuleSize = 40, maxBlockSize = 25000,
                                      reassignThreshold = 0, minKMEtoStay = 0.7,
                                      mergeCutHeight = 0.10, numericLabels = TRUE,
                                      pamRespectsDendro = FALSE, verbose=3)
  expect_s4_class(autism_networks[[1]], "WGCNA")
})

test_that("Can run on WGCNA on data.frame", {
  autism_networks = constructNetworks(assays(autism_se)[[1]], sampleTable, conditions1[[1]], conditions2[[1]],
                                      networkType = "unsigned", power = 10,
                                      minModuleSize = 40, maxBlockSize = 25000,
                                      reassignThreshold = 0, minKMEtoStay = 0.7,
                                      mergeCutHeight = 0.10, numericLabels = TRUE,
                                      pamRespectsDendro = FALSE, verbose=3)
  expect_s4_class(autism_networks[[1]], "WGCNA")
})

#' The WGCNA Class
#'
#' The WGCNA class is the main class used in multiWGCNA to store results from 
#' a weighted gene co-expression nework analysis. These include the original 
#' unaltered expression data used as input, connectivity metrics, module 
#' assignment, input sample conditions, trait 
#'
#' @slot datExpr The expression data, connectivity data, and module assignment
#' @slot conditions A data.frame with integer conditions for WGCNA
#' @slot trait A data.frame showing pearson correlation values to traits
#' @slot moduleEigengenes A data.frame of module eigengenes for each module across samples
#' @slot outlierModules A vector of modules classified by our algorithm as being driven by sample outliers
#'
#' @return NA
#'
#' @import methods
setClass("WGCNA", slots=list(datExpr="data.frame", 
                             conditions="data.frame", 
                             trait="data.frame", 
                             moduleEigengenes="data.frame", 
                             outlierModules="vector"))

setMethod("show", "WGCNA", function(object) {
		cat("##### datExpr #####\n")
		print(head(object@datExpr[,1:5]))
		if(length(object@conditions)>0){
			cat("\n##### conditions #####\n")
			print(head(object@conditions))
		}
		if(length(object@conditions)>0){
			cat("\n##### module-trait correlation #####\n")
			print(head(object@trait))
		}
		if(length(object@moduleEigengenes)>0){
			cat("\n##### module eigengenes #####\n")
			print(head(object@moduleEigengenes[,1:5]))
		}
		if(length(object@outlierModules)>0){
			cat("\n##### outlier modules #####\n")
			print(object@outlierModules)
		}
	}
)

#' Get expression data
#'
#' Returns the expression data frame a WGCNA object as a data.frame
#'
#' @param object An object of class WGCNA
#' @param genes a list of genes to subset to; default is NULL
#'
#' @return a data.frame
#' 
#' @author Dario Tommasini
#'
#' @export
#' 
#' @examples
#' library(ExperimentHub)
#' eh = ExperimentHub()
#' eh_query = query(eh, c("multiWGCNAdata"))
#' astrocyte_networks = eh_query[["EH8222"]]
#' datExpr = GetDatExpr(astrocyte_networks[[1]], 
#'   genes = topNGenes(astrocyte_networks$EAE, "EAE_015", 20))
#' coexpressionLineGraph(datExpr)	+ 
#'   geom_vline(xintercept = 20.5, linetype='dashed')
#' 
GetDatExpr = function(object, genes = NULL){
  datExpr = t(cleanDatExpr(object@datExpr))
  if(!is.null(genes)) datExpr = datExpr[match(genes, rownames(datExpr)),]
  return(datExpr)
}

---
title: "multiWGCNA: visualizing condition-specific networks"
author: "Dario Tommasini"
date: "`r Sys.Date()`"
output: 
  BiocStyle::html_document:
    toc: true
vignette: >
  %\VignetteIndexEntry{Astrocyte multiWGCNA network}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

In this vignette, we will be showing various ways users can analyze their condition-specific network across the conditions of their experiment. We will be using the astrocyte Ribotag data from Itoh et al. PNAS. 2018 (https://doi.org/10.1073/pnas.1716032115). This data comes with the multiWGCNAdata package hosted on ExperimentHub and can be accessed as shown below. 

# Load multiWGCNA library

```{r setup}
library(multiWGCNA)
```

# Load astrocyte Ribotag RNA-seq data 

```{r}
# Download data from the ExperimentHub
library(ExperimentHub)
eh = ExperimentHub()

# Note: this requires the SummarizedExperiment package to be installed
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_se = eh_query[["EH8223"]]

# Collect the metadata in the sampleTable; the first column must be named "Sample"
sampleTable = colData(astrocyte_se)

# Check the data
assays(astrocyte_se)[[1]][1:5, 1:5]
sampleTable

# Define our conditions for trait 1 (disease) and 2 (brain region)
conditions1 = unique(sampleTable[,2])
conditions2 = unique(sampleTable[,3])
```

# Network construction

We now perform network construction, module eigengene calculation, module-trait correlation. Let's use power = 12 since we used this in our paper (Tommasini and Fogel. BMC Bioinformatics. 2023.) for all the networks. 

```{r, eval = FALSE}
# Construct the combined networks and all the sub-networks (EAE, WT, and each region)
# Same parameters as Tommasini and Fogel. BMC Bioinformatics
astrocyte_networks = constructNetworks(astrocyte_se, sampleTable, conditions1, conditions2, 
                                  networkType = "signed", TOMType = "unsigned", 
                                  power = 12, minModuleSize = 100, maxBlockSize = 25000,
                                  reassignThreshold = 0, minKMEtoStay = 0, mergeCutHeight = 0,
                                  numericLabels = TRUE, pamRespectsDendro = FALSE, 
                                  deepSplit = 4, verbose = 3)

```

This step takes a while since it performs seven network constructions, so we also provide the WGCNA object list (astrocyte_networks) in a loadable format. These were generated from the function above. 

```{r}
# Load pre-computed astrocyte networks
astrocyte_networks = eh_query[["EH8222"]] 

# Check one of the WGCNA objects
astrocyte_networks[["combined"]]
```

# Compare modules by overlap 

Next, we compare modules (by hypergeometric overlap) across conditions. We'll save the results in a list. 

```{r, fig.height = 5, fig.width = 8}
# Save results to a list
results = list()
results$overlaps = iterate(astrocyte_networks, overlapComparisons, plot=FALSE)

# Check the overlaps, ie between the EAE and wildtype networks
head(results$overlaps$EAE_vs_WT$overlap)
```

# Identify a module of interest

Then, we perform differential module expression analysis to detect modules with disease-associated expression patterns. This incorporates the linear model described in the paper and tests for significance using ANOVA. 

```{r, fig.height = 6, fig.width = 7}
# Run differential module expression analysis (DME) on combined networks
results$diffModExp = runDME(astrocyte_networks[["combined"]], 
                            sampleTable,
                            p.adjust = "fdr", 
                            refCondition = "Region", 
                            testCondition = "Disease") 
                            # plot=TRUE, 
                            # out="ANOVA_DME.pdf")

# Check results sorted by disease association FDR
results$diffModExp[order(results$diffModExp$Disease),]

# You can check the expression of module M13 from Tommasini and Fogel. BMC Bioinformatics. 2023 like this. Note that the values reported in the bottom panel title are p-values and not adjusted for multiple comparisons like in results$diffModExp
diffModuleExpression(astrocyte_networks[["combined"]], 
                     geneList = topNGenes(astrocyte_networks[[1]], "combined_013"), 
                     design = sampleTable,
                     test = "ANOVA",
                     plotTitle = "combined_013",
                     plot = TRUE)

```

# Draw the multiWGCNA network

We can now check to see if M13 is present in any of the sub-networks. An easy way to do this is using the network-network correspondences from hypergeometric overlap. These are stored in results$overlaps. We can plot these in a convenient visualization scheme that also organizes the three levels of the multiWGCNA analysis: 1) combined network, 2) EAE and wildtype networks, and 3) the four regional networks. 

```{r, fig.height = 6, fig.width = 7}
drawMultiWGCNAnetwork(astrocyte_networks, 
                      results$overlaps, 
                      "combined_013", 
                      design = sampleTable, 
                      overlapCutoff = 0, 
                      padjCutoff = 1, 
                      removeOutliers = TRUE, 
                      alpha = 1e-50, 
                      layout = NULL, 
                      hjust = 0.4, 
                      vjust = 0.3, 
                      width = 0.5)
```

This corresponds to Figure 2C from Tommasini and Fogel. BMC Bioinformatics. 2023. We see that M13 is really only present in the EAE network, but not any of the other sub-networks. Most importantly, it cannot be resolved in the wildtype network. This makes M13 a biologically interesting network, both in terms of differential expression and differential co-expression. 

We can identify the EAE module that corresponds to M13 using the overlap analysis: 

```{r, fig.height = 8, fig.width = 10}
bidirectionalBestMatches(results$overlaps$combined_vs_EAE)
```

The colors correspond to -log10(FDR) derived from the hypergeometric test, while the numbers in the cells correspond to the number of genes overlapping. From this plot, we see that the module from the EAE network that corresponds to combined_013 is called EAE_015.

# Observe differential co-expression of top module genes

We can visually check that combined_013/EAE_015 genes are co-expressed in EAE and not co-expressed in WT samples. 

```{r, fig.height=5, fig.width=7}
# Get expression data for top 20 genes in EAE_015 module
datExpr = GetDatExpr(astrocyte_networks[[1]], 
                     genes = topNGenes(astrocyte_networks$EAE, "EAE_015", 20))

# Plot
coexpressionLineGraph(datExpr, splitBy = 1.5, fontSize = 2.5) + 
  geom_vline(xintercept = 20.5, linetype='dashed')
```

This corresponds to Figure 2D from Tommasini and Fogel. BMC Bioinformatics. 2023. Indeed, we see that these representative module members, which include many immune-related genes, co-vary in EAE samples but less so in WT samples. 

# Follow up with a preservation analysis

Typically, you would want to follow this up with a preservation analysis between EAE and WT (described in general_workflow.Rmd). This is slow so we don't actually run this in the vignette. 

```{r, eval = FALSE, fig.height = 3, fig.width = 7}
# To enable multi-threading
# library(doParallel)
# library(WGCNA)
# nCores = 2
# registerDoParallel(cores = nCores)
# enableWGCNAThreads(nThreads = nCores)

# Turn off multi-threading
# registerDoSEQ()
# disableWGCNAThreads()

# Calculate preservation statistics
results$preservation=iterate(astrocyte_networks[c("EAE", "WT")], 
                             preservationComparisons, 
                             write=FALSE, 
                             plot=TRUE, 
                             nPermutations=2)
```

# Determining if preservation value is significant

Then, one can perform a permutation procedure that estimates the probability of observing a disease (or wildtype) module with this preservation score in the wildtype (or disease) setting (PreservationPermutationTest). The test is designed to control for the other condition in the sampleTable. In this case, it will equally distribute the samples belonging to each anatomical region when testing preservation of this disease module in the wildtype samples. This is the slowest step! We recommend to let this run on a computing cluster overnight. 

```{r, eval = FALSE}
options(paged.print = FALSE)
results$permutation.test = PreservationPermutationTest(astrocyte_networks$combined@datExpr[sample(17000,3000),], 
                                                       sampleTable, 
                                                       constructNetworksIn = "EAE", # Construct networks using EAE samples
                                                       testPreservationIn = "WT", # Test preservation of disease samples in WT samples
                                                       nPermutations = 10, # Number of permutations for permutation test
                                                       nPresPermutations = 10, # Number of permutations for modulePreservation function
                                                       
                                                       # WGCNA parameters for re-sampled networks (should be the same as used for network construction)
                                                       networkType = "signed", TOMType = "unsigned", 
                                                       power = 12, minModuleSize = 100, maxBlockSize = 25000,
                                                       reassignThreshold = 0, minKMEtoStay = 0, mergeCutHeight = 0,
                                                       numericLabels = TRUE, pamRespectsDendro = FALSE, 
                                                       deepSplit = 4, verbose = 3
                                                       )
```

Because this step is slow, we provide pre-computed results within the package, stored as `permutationTestResults`. Since module preservation scores are dependent on the module size (larger modules have larger preservation scores), we generate the null distribution of preservation scores based on similarly sized modules from each permutation. Note that we have to filter out "outlier modules" first, as these are modules where the correlations are driven by a single sample. 

```{r, eval = TRUE}
# Load pre-computed results
data(permutationTestResults) 

# Remove outlier modules
permutationTestResultsFiltered = lapply(permutationTestResults, function(x) x[!x$is.outlier.module,])

# Extract the preservation score distribution
results$scores.summary = PreservationScoreDistribution(permutationTestResultsFiltered, 
                                                       moduleOfInterestSize = 303 # The size of the module of interest (dM15)
                                                       )

# Observed preservation score of dM15
observed.score = 9.16261490617938

# How many times did we observe a score lower than or equal to this observed score?
z.summary.dist = results$scores.summary$z.summary
below=length(z.summary.dist[z.summary.dist <= observed.score])
probability= below/100
message("Probability of observing a score of ", round(observed.score, 2), " is ", probability)
```

You can plot the observed score and the null distribution on a histogram like in Figure 2B of the manuscript. 

```{r, eval = TRUE}
# Plot on a histogram
ggplot(results$scores.summary, aes(x=z.summary)) + 
      geom_histogram(color="black", fill="white", bins = 15)+
      xlab("Preservation score")+
      ylab("Frequency")+
      geom_vline(xintercept=observed.score, color="red3", linetype="solid")+
      scale_y_continuous(expand = c(0,0))+
      theme_classic()+
      theme(plot.title = element_text(hjust = 0.5))
```

# Conclusion

This analysis provides a great deal of evidence suggesting that dM15 is a transcriptional network that is gained in this disease status and is not present in healthy astrocytes. Further investigation will reveal that this network corresponds to the reactive astrocyte transcriptional network, which indeed is active in the EAE model and is absent in healthy astrocytes. Please see Tommasini and Fogel. BMC Bioinformatics. 2023. for more details. 

```{r}
sessionInfo()
```

---
title: "multiWGCNA: the full workflow"
author: "Dario Tommasini"
date: "`r Sys.Date()`"
output: 
  BiocStyle::html_document:
    toc: true
vignette: >
  %\VignetteIndexEntry{General Workflow}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

The multiWGCNA R package is a WGCNA-based procedure designed for RNA-seq datasets with two biologically meaningful variables. We developed this package because we've found that the experimental design for network analysis can be ambiguous. For example, should I analyze my treatment and wildtype samples separately or together? We find that it is informative to perform all the possible design combinations, and subsequently integrate these results. For more information, please see Tommasini and Fogel. BMC Bioinformatics. 2023.

In this vignette, we will be showing how users can perform a full start-to-finish multiWGCNA analysis. We will be using the regional autism data from Voineagu et al. 2011 (https://www.nature.com/articles/nature10110). This dataset has two sample traits: 1) autism versus control, and 2) temporal cortex versus frontal cortex. 

# Install the multiWGCNA R package

```{r eval=FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("multiWGCNA")
```

The development version of multiWGCNA can be installed from GitHub like this:

```{r eval=FALSE}
if (!require("devtools", quietly = TRUE))
    install.packages("devtools")
    
devtools::install_github("fogellab/multiWGCNA")
```

Load multiWGCNA library

```{r}
library(multiWGCNA)
```

# Load microarray data from human post-mortem brains

Since this is purely an example of how to run the entire analysis from start to finish, we are going to limit our analysis to 1500 randomly selected probes. 

Input data: 

* The expression data can be either a SummarizedExperiment object or a data.frame with genes as rows and samples as columns
* The sampleTable should be a data.frame with the first column named "Sample". The second and third column names can be arbitrary, i.e. "disease" or "Genotype"


```{r}
# Download data from the ExperimentHub
library(ExperimentHub)
eh = ExperimentHub()

# Note: this requires the SummarizedExperiment package to be installed
eh_query = query(eh, c("multiWGCNAdata"))
autism_se = eh_query[["EH8219"]]

# Collect the metadata in the sampleTable
sampleTable = colData(autism_se)

# Randomly sample 1500 genes from the expression matrix
set.seed(1)
autism_se = autism_se[sample(rownames(autism_se), 1500),]

# Check the data
assays(autism_se)[[1]][1:5, 1:5]
# sampleTable$Status = paste0("test_", sampleTable$Status, "_test")
# sampleTable$Sample = paste0(sampleTable$Sample, "_test")
sampleTable

# Define our conditions for trait 1 (disease) and 2 (brain region)
conditions1 = unique(sampleTable[,2])
conditions2 = unique(sampleTable[,3])
```

# Perform network construction, module eigengene calculation, module-trait correlation 

Also, let's find best trait for each module and identify outlier modules (ie modules driven by a single sample). 

Let's use power = 10 since Voineagu et al. 2011 used that for all their networks. They also constructed "unsigned" networks and a mergeCutHeight of 0.10 so that modules with similar expression are merged. This step may take a minute.  

```{r}
# Construct the combined networks and all the sub-networks (autism only, controls only, FC only, and TC only)
autism_networks = constructNetworks(autism_se, sampleTable, conditions1, conditions2,
                                  networkType = "unsigned", power = 10,
                                  minModuleSize = 40, maxBlockSize = 25000,
                                  reassignThreshold = 0, minKMEtoStay = 0.7,
                                  mergeCutHeight = 0.10, numericLabels = TRUE,
                                  pamRespectsDendro = FALSE, verbose=3,
                                  saveTOMs = TRUE)
```

# Compare modules by overlap across conditions

For calculating significance of overlap, we use the hypergeometric test (also known as the one-sided Fisher exact test). We'll save the results in a list. Then, let's take a look at the mutual best matches between autism modules and control modules. 

```{r, fig.height = 4, fig.width = 7}
# Save results to a list
results=list()
results$overlaps=iterate(autism_networks, overlapComparisons, plot=TRUE)

# Check the reciprocal best matches between the autism and control networks
head(results$overlaps$autism_vs_controls$bestMatches)
```

You can use the TOMFlowPlot to plot the recruitment of genes from one network analysis to another. See Figure 6 from Tommasini and Fogel, 2023 for an example. Note that you will need to set saveTOMs = TRUE in the constructNetworks function above. 

```{r, fig.height=6, fig.width=4}
networks = c("autism", "controls")
toms = lapply(networks, function(x) {
  load(paste0(x, '-block.1.RData'))
  get("TOM")
})

# Check module autism_005
TOMFlowPlot(autism_networks, 
            networks, 
            toms, 
            genes_to_label = topNGenes(autism_networks$autism, "autism_005"), 
            color = 'black', 
            alpha = 0.1, 
            width = 0.05)
```

# Perform differential module expression analysis 

This test for an association between the module eigengene (ME) and the two sample traits or their interaction. Therefore, the model is ME = trait1 + trait2 + trait1*trait2 and tests for a significant association to the traits using factorial ANOVA. These results can be stored in the diffModExp component of the results list. It can also perform PERMANOVA, which uses multivariate distances and can thus be applied to the module expression rather than just the first principal component (module eigengene). 

Module combined_004 seems to be the module most significantly associated with disease status using standard ANOVA (module combined_000 represents genes that were unassigned so disregard this module). Let's use the diffModuleExpression function to visually check this module for an association to autism status. 

Note: By setting plot = TRUE, the runDME function generates a PDF file called "combined_DME.pdf" in the current directory with these plots for all the modules. 

```{r, fig.height = 6, fig.width = 7}
# Run differential module expression analysis (DME) on combined networks
results$diffModExp = runDME(autism_networks[["combined"]], 
                            sampleTable, 
                            p.adjust="fdr", 
                            refCondition="Tissue", 
                            testCondition="Status") 
                            # plot=TRUE, 
                            # out="combined_DME.pdf")

# to run PERMANNOVA
# library(vegan)
# results$diffModExp = runDME(autism_networks[["combined"]], p.adjust="fdr", refCondition="Tissue", 
#                          testCondition="Status", plot=TRUE, test="PERMANOVA", out="PERMANOVA_DME.pdf")

# Check adjusted p-values for the two sample traits
results$diffModExp

# You can check the expression of a specific module like this. Note that the values reported in the bottom panel title are p-values and not FDR-adjusted like in results$diffModExp
diffModuleExpression(autism_networks[["combined"]], 
                     geneList = topNGenes(autism_networks[["combined"]], "combined_004"), 
                     design = sampleTable,
                     test = "ANOVA",
                     plotTitle = "combined_004",
                     plot = TRUE)
```

# Perform the module preservation analysis

Determine if any modules in the autism data are not preserved in the healthy data (or vice versa). This is the slowest step as the calculation of permuted statistics takes a while. Typically, this can be parallelized using the enableWGCNAThreads function, but for this vignette we'll just do 10 permutations so one core should suffice.   

```{r, fig.height = 3, fig.width = 7}
# To enable multi-threading
# library(doParallel)
# library(WGCNA)
# nCores = 8
# registerDoParallel(cores = nCores)
# enableWGCNAThreads(nThreads = nCores)

# Calculate preservation statistics
results$preservation=iterate(autism_networks[conditions1], # this does autism vs control; change to "conditions2" to perform comparison between FC and TC
                             preservationComparisons, 
                             write=FALSE, 
                             plot=TRUE, 
                             nPermutations=10)
```

# Summarize interesting results from the analyses

These include differentially preserved trait-associated modules and differentially expressed modules. 

```{r}
# Print a summary of the results
summarizeResults(autism_networks, results)
```

# Print the session info

```{r}
sessionInfo()
```

---
title: "multiWGCNA: visualizing condition-specific networks"
author: "Dario Tommasini"
date: "`r Sys.Date()`"
output: 
  BiocStyle::html_document:
    toc: true
vignette: >
  %\VignetteIndexEntry{Astrocyte multiWGCNA network}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

In this vignette, we will be showing various ways users can analyze their condition-specific network across the conditions of their experiment. We will be using the astrocyte Ribotag data from Itoh et al. PNAS. 2018 (https://doi.org/10.1073/pnas.1716032115). This data comes with the multiWGCNAdata package hosted on ExperimentHub and can be accessed as shown below. 

# Load multiWGCNA library

```{r setup}
library(multiWGCNA)
```

# Load astrocyte Ribotag RNA-seq data 

```{r}
# Download data from the ExperimentHub
library(ExperimentHub)
eh = ExperimentHub()

# Note: this requires the SummarizedExperiment package to be installed
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_se = eh_query[["EH8223"]]

# Collect the metadata in the sampleTable; the first column must be named "Sample"
sampleTable = colData(astrocyte_se)

# Check the data
assays(astrocyte_se)[[1]][1:5, 1:5]
sampleTable

# Define our conditions for trait 1 (disease) and 2 (brain region)
conditions1 = unique(sampleTable[,2])
conditions2 = unique(sampleTable[,3])
```

# Network construction

We now perform network construction, module eigengene calculation, module-trait correlation. Let's use power = 12 since we used this in our paper (Tommasini and Fogel. BMC Bioinformatics. 2023.) for all the networks. 

```{r, eval = FALSE}
# Construct the combined networks and all the sub-networks (EAE, WT, and each region)
# Same parameters as Tommasini and Fogel. BMC Bioinformatics
astrocyte_networks = constructNetworks(astrocyte_se, sampleTable, conditions1, conditions2, 
                                  networkType = "signed", TOMType = "unsigned", 
                                  power = 12, minModuleSize = 100, maxBlockSize = 25000,
                                  reassignThreshold = 0, minKMEtoStay = 0, mergeCutHeight = 0,
                                  numericLabels = TRUE, pamRespectsDendro = FALSE, 
                                  deepSplit = 4, verbose = 3)

```

This step takes a while since it performs seven network constructions, so we also provide the WGCNA object list (astrocyte_networks) in a loadable format. These were generated from the function above. 

```{r}
# Load pre-computed astrocyte networks
astrocyte_networks = eh_query[["EH8222"]] 

# Check one of the WGCNA objects
astrocyte_networks[["combined"]]
```

# Compare modules by overlap 

Next, we compare modules (by hypergeometric overlap) across conditions. We'll save the results in a list. 

```{r, fig.height = 5, fig.width = 8}
# Save results to a list
results = list()
results$overlaps = iterate(astrocyte_networks, overlapComparisons, plot=FALSE)

# Check the overlaps, ie between the EAE and wildtype networks
head(results$overlaps$EAE_vs_WT$overlap)
```

To compare module overlap across networks, one could use the ModuleFlowPlot function: 

```{r, fig.height=10, fig.width=7}
ModuleFlowPlot(WGCNAlist = astrocyte_networks, 
              comparisonList = results$overlaps, 
              networks = c('combined',  'EAE', 'WT'), 
              spacer = 100, # size of spacer
              label.y = 500, # vertical adjustment for the network labels
              x.scale = 10, # how much to spread out the nodes along x-axis
              color.by = 'trait', # color nodes by their best trait correlation
              col = c(Cbl = 'lightpink', Ctx = 'chartreuse', Hippo = 'skyblue', Sc = 'orange', EAE = 'red', None = 'grey'), 
              p.adj.threshold = 1e-50, 
              label.size = 4)
```

# Identify a module of interest

Then, we perform differential module expression analysis to detect modules with disease-associated expression patterns. This incorporates the linear model described in the paper and tests for significance using ANOVA. 

```{r, fig.height = 6, fig.width = 7}
# Run differential module expression analysis (DME) on combined networks
results$diffModExp = runDME(astrocyte_networks[["combined"]], 
                            sampleTable,
                            p.adjust = "fdr", 
                            refCondition = "Region", 
                            testCondition = "Disease") 
                            # plot=TRUE, 
                            # out="ANOVA_DME.pdf")

# Check results sorted by disease association FDR
results$diffModExp[order(results$diffModExp$Disease),]

# You can check the expression of module M13 from Tommasini and Fogel. BMC Bioinformatics. 2023 like this. Note that the values reported in the bottom panel title are p-values and not adjusted for multiple comparisons like in results$diffModExp
diffModuleExpression(astrocyte_networks[["combined"]], 
                     geneList = topNGenes(astrocyte_networks[[1]], "combined_013"), 
                     design = sampleTable,
                     test = "ANOVA",
                     plotTitle = "combined_013",
                     plot = TRUE)

```

# Draw the multiWGCNA network

We can now check to see if M13 is present in any of the sub-networks. An easy way to do this is using the network-network correspondences from hypergeometric overlap. These are stored in results$overlaps. We can plot these in a convenient visualization scheme that also organizes the three levels of the multiWGCNA analysis: 1) combined network, 2) EAE and wildtype networks, and 3) the four regional networks. 

```{r, fig.height = 6, fig.width = 7}
drawMultiWGCNAnetwork(astrocyte_networks, 
                      results$overlaps, 
                      "combined_013", 
                      design = sampleTable, 
                      overlapCutoff = 0, 
                      padjCutoff = 1, 
                      removeOutliers = TRUE, 
                      alpha = 1e-50, 
                      layout = NULL, 
                      hjust = 0.4, 
                      vjust = 0.3, 
                      width = 0.5)
```

This corresponds to Figure 2C from Tommasini and Fogel. BMC Bioinformatics. 2023. We see that M13 is really only present in the EAE network, but not any of the other sub-networks. Most importantly, it cannot be resolved in the wildtype network. This makes M13 a biologically interesting network, both in terms of differential expression and differential co-expression. 

We can identify the EAE module that corresponds to M13 using the overlap analysis: 

```{r, fig.height = 8, fig.width = 10}
bidirectionalBestMatches(results$overlaps$combined_vs_EAE)
```

The colors correspond to -log10(FDR) derived from the hypergeometric test, while the numbers in the cells correspond to the number of genes overlapping. From this plot, we see that the module from the EAE network that corresponds to combined_013 is called EAE_015.

# Observe differential co-expression of top module genes

We can visually check that combined_013/EAE_015 genes are co-expressed in EAE and not co-expressed in WT samples. 

```{r, fig.height=5, fig.width=7}
# Get expression data for top 20 genes in EAE_015 module
datExpr = GetDatExpr(astrocyte_networks[[1]], 
                     genes = topNGenes(astrocyte_networks$EAE, "EAE_015", 20))

# Plot
coexpressionLineGraph(datExpr, splitBy = 1.5, fontSize = 2.5) + 
  geom_vline(xintercept = 20.5, linetype='dashed')
```

This corresponds to Figure 2D from Tommasini and Fogel. BMC Bioinformatics. 2023. Indeed, we see that these representative module members, which include many immune-related genes, co-vary in EAE samples but less so in WT samples. 

# Follow up with a preservation analysis

Typically, you would want to follow this up with a preservation analysis between EAE and WT (described in general_workflow.Rmd). This is slow so we don't actually run this in the vignette. 

```{r, eval = FALSE, fig.height = 3, fig.width = 7}
# To enable multi-threading
# library(doParallel)
# library(WGCNA)
# nCores = 2
# registerDoParallel(cores = nCores)
# enableWGCNAThreads(nThreads = nCores)

# Turn off multi-threading
# registerDoSEQ()
# disableWGCNAThreads()

# Calculate preservation statistics
results$preservation=iterate(astrocyte_networks[c("EAE", "WT")], 
                             preservationComparisons, 
                             write=FALSE, 
                             plot=TRUE, 
                             nPermutations=2)
```

# Determining if preservation value is significant

Then, one can perform a permutation procedure that estimates the probability of observing a disease (or wildtype) module with this preservation score in the wildtype (or disease) setting (PreservationPermutationTest). The test is designed to control for the other condition in the sampleTable. In this case, it will equally distribute the samples belonging to each anatomical region when testing preservation of this disease module in the wildtype samples. This is the slowest step! We recommend to let this run on a computing cluster overnight. 

```{r, eval = FALSE}
options(paged.print = FALSE)
results$permutation.test = PreservationPermutationTest(astrocyte_networks$combined@datExpr[sample(17000,3000),], 
                                                       sampleTable, 
                                                       constructNetworksIn = "EAE", # Construct networks using EAE samples
                                                       testPreservationIn = "WT", # Test preservation of disease samples in WT samples
                                                       nPermutations = 10, # Number of permutations for permutation test
                                                       nPresPermutations = 10, # Number of permutations for modulePreservation function
                                                       
                                                       # WGCNA parameters for re-sampled networks (should be the same as used for network construction)
                                                       networkType = "signed", TOMType = "unsigned", 
                                                       power = 12, minModuleSize = 100, maxBlockSize = 25000,
                                                       reassignThreshold = 0, minKMEtoStay = 0, mergeCutHeight = 0,
                                                       numericLabels = TRUE, pamRespectsDendro = FALSE, 
                                                       deepSplit = 4, verbose = 3
                                                       )
```

Because this step is slow, we provide pre-computed results within the package, stored as `permutationTestResults`. Since module preservation scores are dependent on the module size (larger modules have larger preservation scores), we generate the null distribution of preservation scores based on similarly sized modules from each permutation. Note that we have to filter out "outlier modules" first, as these are modules where the correlations are driven by a single sample. 

```{r, eval = TRUE}
# Load pre-computed results
data(permutationTestResults) 

# Remove outlier modules
permutationTestResultsFiltered = lapply(permutationTestResults, function(x) x[!x$is.outlier.module,])

# Extract the preservation score distribution
results$scores.summary = PreservationScoreDistribution(permutationTestResultsFiltered, 
                                                       moduleOfInterestSize = 303 # The size of the module of interest (dM15)
                                                       )

# Observed preservation score of dM15
observed.score = 9.16261490617938

# How many times did we observe a score lower than or equal to this observed score?
z.summary.dist = results$scores.summary$z.summary
below=length(z.summary.dist[z.summary.dist <= observed.score])
probability= below/100
message("Probability of observing a score of ", round(observed.score, 2), " is ", probability)
```

You can plot the observed score and the null distribution on a histogram like in Figure 2B of the manuscript. 

```{r, eval = TRUE}
# Plot on a histogram
ggplot(results$scores.summary, aes(x=z.summary)) + 
      geom_histogram(color="black", fill="white", bins = 15)+
      xlab("Preservation score")+
      ylab("Frequency")+
      geom_vline(xintercept=observed.score, color="red3", linetype="solid")+
      scale_y_continuous(expand = c(0,0))+
      theme_classic()+
      theme(plot.title = element_text(hjust = 0.5))
```

# Conclusion

This analysis provides a great deal of evidence suggesting that dM15 is a transcriptional network that is gained in this disease status and is not present in healthy astrocytes. Further investigation will reveal that this network corresponds to the reactive astrocyte transcriptional network, which indeed is active in the EAE model and is absent in healthy astrocytes. Please see Tommasini and Fogel. BMC Bioinformatics. 2023. for more details. 

```{r}
sessionInfo()
```

---
title: "multiWGCNA: the full workflow"
author: "Dario Tommasini"
date: "`r Sys.Date()`"
output: 
  BiocStyle::html_document:
    toc: true
vignette: >
  %\VignetteIndexEntry{General Workflow}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

The multiWGCNA R package is a WGCNA-based procedure designed for RNA-seq datasets with two biologically meaningful variables. We developed this package because we've found that the experimental design for network analysis can be ambiguous. For example, should I analyze my treatment and wildtype samples separately or together? We find that it is informative to perform all the possible design combinations, and subsequently integrate these results. For more information, please see Tommasini and Fogel. BMC Bioinformatics. 2023.

In this vignette, we will be showing how users can perform a full start-to-finish multiWGCNA analysis. We will be using the regional autism data from Voineagu et al. 2011 (https://www.nature.com/articles/nature10110). This dataset has two sample traits: 1) autism versus control, and 2) temporal cortex versus frontal cortex. 

# Install the multiWGCNA R package

```{r eval=FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("multiWGCNA")
```

The development version of multiWGCNA can be installed from GitHub like this:

```{r eval=FALSE}
if (!require("devtools", quietly = TRUE))
    install.packages("devtools")
    
devtools::install_github("fogellab/multiWGCNA")
```

Load multiWGCNA library

```{r}
library(multiWGCNA)
```

# Load microarray data from human post-mortem brains

Since this is purely an example of how to run the entire analysis from start to finish, we are going to limit our analysis to 1500 randomly selected probes. 

Input data: 

* The expression data can be either a SummarizedExperiment object or a data.frame with genes as rows and samples as columns
* The sampleTable should be a data.frame with the first column named 'Sample' (note the capitalized 'S'). The second and third column names can be arbitrary, i.e. "disease" or "Genotype"


```{r}
# Download data from the ExperimentHub
library(ExperimentHub)
eh = ExperimentHub()

# Note: this requires the SummarizedExperiment package to be installed
eh_query = query(eh, c("multiWGCNAdata"))
autism_se = eh_query[["EH8219"]]

# Collect the metadata in the sampleTable
sampleTable = colData(autism_se)

# Randomly sample 1500 genes from the expression matrix
set.seed(1)
autism_se = autism_se[sample(rownames(autism_se), 1500),]

# Check the data
assays(autism_se)[[1]][1:5, 1:5]
sampleTable

# Define our conditions for trait 1 (disease) and 2 (brain region)
conditions1 = unique(sampleTable[,2])
conditions2 = unique(sampleTable[,3])
```

# Perform network construction, module eigengene calculation, module-trait correlation 

Network construction is usually the slowest step. This wrapper also finds the best trait association for each module and identifies outlier modules (i.e. modules driven by a single sample). 

Let's use power = 10 since Voineagu et al. 2011 used that for all their networks. They also constructed "unsigned" networks and a mergeCutHeight of 0.10 so that modules with similar expression are merged. Note that these parameter settings are meant to match Voineagu et al. 2011. For an example of more typical WGCNA settings, see the astrocyte vignette. 

```{r}
# Construct the combined networks and all the sub-networks (autism only, controls only, FC only, and TC only)
autism_networks = constructNetworks(autism_se, sampleTable, conditions1, conditions2,
                                  networkType = "unsigned", power = 10,
                                  minModuleSize = 40, maxBlockSize = 25000,
                                  reassignThreshold = 0, minKMEtoStay = 0.7,
                                  mergeCutHeight = 0.10, numericLabels = TRUE,
                                  pamRespectsDendro = FALSE, verbose=3,
                                  saveTOMs = TRUE)
```

# Compare modules by overlap across conditions

For calculating significance of overlap, we use the hypergeometric test (also known as the one-sided Fisher exact test). We'll save the results in a list. Then, let's take a look at the mutual best matches between autism modules and control modules. 

```{r, fig.height = 4, fig.width = 7}
# Save results to a list
results=list()
results$overlaps=iterate(autism_networks, overlapComparisons, plot=TRUE)

# Check the reciprocal best matches between the autism and control networks
head(results$overlaps$autism_vs_controls$bestMatches)
```

You can use the ModuleFlotPlot to visualize the module overlap across the various networks: 

```{r, fig.height=6, fig.width=8}
ModuleFlowPlot(autism_networks, # WGCNA object list
               comparisonList = results$overlaps, # overlap comparisons
               networks = c('combined', 'autism', 'controls', 'FC', 'TC'), 
               width = 0.3, # width of nodes
               height = 0.2, # height of nodes
               only.signif = TRUE, # only significant overlaps
               only.contiguous = TRUE, # only contiguous comparisons (controlled by order)
               scale.by.size = TRUE, # scale nodes by module size
               spacer = 10, # space between nodes
               label.y = 50, # adjust placement of the network labels
               col = c(autism = 'pink', FC = 'lightgreen', None = 'grey') # color palette
               ) 
```

Of course, this doesn't give you gene-level resolution. To do that, you can use the TOMFlowPlot to plot the recruitment of genes from one network analysis to another. See Figure 6 from Tommasini and Fogel, 2023 for an example. Note that you will need to set saveTOMs = TRUE in the constructNetworks function above. In this example, we will observe the movement of module 5 from the autism network across all five computed networks. Thank you to Xinye Li for generalizing the function to more than two networks!

```{r, fig.height=5, fig.width=8}
networks = c('combined', 'autism', 'controls', 'FC', 'TC')
toms = lapply(networks, function(x) {
  load(paste0(x, '-block.1.RData'))
  get("TOM")
})

tom.df = BuildTOMFlowDF(autism_networks, 
                        networks, 
                        toms, 
                        genes_to_label = topNGenes(autism_networks$autism, "autism_005"))

PlotMultiNodesTOMflow(tom.df, labels = networks)
```

# Perform differential module expression analysis 

This test for an association between the module eigengene (ME) and the two sample traits or their interaction. Therefore, the model is ME = trait1 + trait2 + trait1*trait2 and tests for a significant association to the traits using factorial ANOVA. These results can be stored in the diffModExp component of the results list. It can also perform PERMANOVA, which uses multivariate distances and can thus be applied to the module expression rather than just the first principal component (module eigengene). 

Module combined_004 seems to be the module most significantly associated with disease status using standard ANOVA (module combined_000 represents genes that were unassigned so disregard this module). Let's use the diffModuleExpression function to visually check this module for an association to autism status. 

Note: By setting plot = TRUE, the runDME function generates a PDF file called "combined_DME.pdf" in the current directory with these plots for all the modules. 

```{r, fig.height = 6, fig.width = 7}
# Run differential module expression analysis (DME) on combined networks
results$diffModExp = runDME(autism_networks[["combined"]], 
                            sampleTable, 
                            p.adjust="fdr", 
                            refCondition="Tissue", 
                            testCondition="Status") 
                            # plot=TRUE, 
                            # out="combined_DME.pdf")

# to run PERMANOVA
# library(vegan)
# results$diffModExp = runDME(autism_networks[["combined"]], p.adjust="fdr", refCondition="Tissue", 
#                          testCondition="Status", plot=TRUE, test="PERMANOVA", out="PERMANOVA_DME.pdf")

# Check adjusted p-values for the two sample traits
results$diffModExp

# You can check the expression of a specific module like this. Note that the values reported in the bottom panel title are p-values and not FDR-adjusted like in results$diffModExp
diffModuleExpression(autism_networks[["combined"]], 
                     geneList = topNGenes(autism_networks[["combined"]], "combined_004"), 
                     design = sampleTable,
                     test = "ANOVA",
                     plotTitle = "combined_004",
                     plot = TRUE)
```

# Perform the module preservation analysis

Determine if any modules in the autism data are not preserved in the healthy data (or vice versa). This is the slowest step as the calculation of permuted statistics takes a while. Typically, this can be parallelized using the enableWGCNAThreads function, but for this vignette we'll just do 10 permutations so one core should suffice.   

```{r, fig.height = 3, fig.width = 7}
# To enable multi-threading
# library(doParallel)
# library(WGCNA)
# nCores = 8
# registerDoParallel(cores = nCores)
# enableWGCNAThreads(nThreads = nCores)

# Calculate preservation statistics
results$preservation=iterate(autism_networks[conditions1], # this does autism vs control; change to "conditions2" to perform comparison between FC and TC
                             preservationComparisons, 
                             write=FALSE, 
                             plot=TRUE, 
                             nPermutations=10)
```

# Summarize interesting results from the analyses

These include differentially preserved trait-associated modules and differentially expressed modules. 

```{r}
# Print a summary of the results
summarizeResults(autism_networks, results)
```

# Print the session info

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preservation.R
\name{PreservationPermutationTest}
\alias{PreservationPermutationTest}
\title{PreservationPermutationTest}
\usage{
PreservationPermutationTest(
  referenceDatExpr,
  design,
  constructNetworksIn,
  testPreservationIn,
  nPermutations = 100,
  nPresPermutations = 100,
  ...
)
}
\arguments{
\item{referenceDatExpr}{the combined datExpr}

\item{design}{the sampleTable}

\item{constructNetworksIn}{the condition to use for network construction, e.g. for the astrocyte data, this is "EAE"}

\item{testPreservationIn}{the condition to use for testing preservation, e.g. for the astrocyte data, this was "WT"}

\item{nPermutations}{the number of permutations to perform for permutation test}

\item{nPresPermutations}{the number of permutations to perform in modulePreservation function}

\item{...}{arguments to pass to blockwiseModules function for network construction (should be the same as used for constructing the original network)}
}
\value{
A list of data.frames with preservation results for each permutation
}
\description{
Performs a permutation test to determine if a null distribution of expected
preservation scores for modules in this dataset if the labels were 
randomly assigned. Please look at the astrocyte vignette for more info.
}
\examples{
\dontrun{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]] 
astrocyte_se = eh_query[["EH8223"]] 
sampleTable = colData(astrocyte_se)
results = list()
results$permutation.test = PreservationPermutationTest(
astrocyte_networks$combined@datExpr[sample(17000,3000),], 
sampleTable, 
constructNetworksIn = "EAE", # Construct networks using EAE samples
testPreservationIn = "WT", # Test preservation of disease samples in WT 
nPermutations = 10, # Number of permutations for permutation test
nPresPermutations = 10, # Number of permutations for modulePreservation
networkType = "signed", TOMType = "unsigned", 
power = 12, minModuleSize = 100, maxBlockSize = 25000,
reassignThreshold = 0, minKMEtoStay = 0, mergeCutHeight = 0,
numericLabels = TRUE, pamRespectsDendro = FALSE, 
deepSplit = 4, verbose = 3
)
}
}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{cleanDatExpr}
\alias{cleanDatExpr}
\title{cleanDatExpr}
\usage{
cleanDatExpr(datExpr, checkGenesSamples = FALSE)
}
\arguments{
\item{datExpr}{a data.frame were columns are samples and rows are samples and the gene symbols are in the first row}

\item{checkGenesSamples}{call the WGCNA function checkGenesSamples?}
}
\value{
Returns a datExpr with rows as samples and columns as genes
}
\description{
A function that converts a data.frame where row 1 is gene symbols to a 
numeric matrix where columns are genes and rows are samples for compatibility 
with most WGCNA functions.
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_se = eh_query[["EH8223"]]
datExpr = data.frame(X = rownames(assays(astrocyte_se)[[1]]), assays(astrocyte_se)[[1]])
cleanDatExpr(datExpr)

}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffModuleExpAnalysis.R
\name{moduleExpressionPlot}
\alias{moduleExpressionPlot}
\title{Plots an expression profile for a module}
\usage{
moduleExpressionPlot(
  WGCNAobject,
  geneList,
  mode = c("PC1", "averageZscore"),
  legend = FALSE,
  title = NULL,
  clusterGenes = FALSE
)
}
\arguments{
\item{WGCNAobject}{an object of class WGCNAobject}

\item{geneList}{a vector of gene names to be extracted from WGCNAobject}

\item{mode}{use first principal component or averageZscore?}

\item{legend}{plot legend?}

\item{title}{title of the plot}

\item{clusterGenes}{cluster heatmap genes by hierarchical clustering?}
}
\value{
a patchworked ggplot object
}
\description{
A plotting function that returns a heatmap and barplot for a module
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]]
moduleExpressionPlot(astrocyte_networks[["combined"]], 
  geneList = topNGenes(astrocyte_networks$combined, "combined_013"))

}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlaps.R
\name{BuildTOMFlowDF}
\alias{BuildTOMFlowDF}
\title{BuildTOMFlowDF}
\usage{
BuildTOMFlowDF(WGCNAlist, networks, toms, genes_to_label, method = "average")
}
\arguments{
\item{WGCNAlist}{list of WGCNA objects}

\item{networks}{list of network names of length 2}

\item{toms}{a list of TOM distance objects of length 2}

\item{genes_to_label}{genes to label across two networks}

\item{method}{linkage method to pass to flashClust for clustering, default is average linkage}
}
\value{
a data.frame
}
\description{
Preprocess for plotting a sankey flow diagram showing the movement of genes from one WGCNA
to another WGCNA. Uses the flashClust framework.
}
\author{
Dario Tommasini, Xinye Li
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{iterate}
\alias{iterate}
\title{iterate: Iterate function across networks}
\usage{
iterate(WGCNAlist, FUN, ...)
}
\arguments{
\item{WGCNAlist}{a vector of objects of type WGCNAobject}

\item{FUN}{function to iterate, either overlapComparisons or preservationComparisons}

\item{...}{argmuents to be passed on to overlapComparisons or preservationComparisons}
}
\value{
a comparison list from overlapComparisons or preservationComparisons
}
\description{
A high level function that iterates functions across
a list of WGCNA objects
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]]
results = list()
iterate(astrocyte_networks, overlapComparisons, plot=FALSE)

}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{name}
\alias{name}
\title{name: Name of WGCNAobject}
\usage{
name(WGCNAobject)
}
\arguments{
\item{WGCNAobject}{an object of class WGCNA}
}
\value{
Returns the name of the WGCNA object, ie "EAE" for 
astrocyte_networks$EAE.
}
\description{
Returns the name of a WGCNAobject.
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]]
name(astrocyte_networks$EAE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffModuleExpAnalysis.R
\name{performANOVA}
\alias{performANOVA}
\title{Perform ANOVA}
\usage{
performANOVA(datExpr, design, testCondition, refCondition, alphaLevel = 0.05)
}
\arguments{
\item{datExpr}{expression data.frame}

\item{design}{the sampleTable}

\item{testCondition}{test column in sampleTable}

\item{refCondition}{reference column in sampleTable}

\item{alphaLevel}{the significance level}
}
\value{
a data.frame with p-values for each association
}
\description{
Test association between module expression to traits using ANOVA
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multiWGCNA-package.R
\docType{package}
\name{multiWGCNA-package}
\alias{multiWGCNA}
\alias{multiWGCNA-package}
\title{multiWGCNA: multiWGCNA}
\description{
An R package for deeping mining gene co-expression networks in multi-trait expression data. Provides functions for analyzing, comparing, and visualizing WGCNA networks across conditions. multiWGCNA was designed to handle the common case where there are multiple biologically meaningful sample traits, such as disease vs wildtype across development or anatomical region.
}
\author{
\strong{Maintainer}: Dario Tommasini \email{dtommasini0@gmail.com} (\href{https://orcid.org/0000-0002-1214-6547}{ORCID})

Authors:
\itemize{
  \item Brent Fogel [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/networks.R
\name{drawMultiWGCNAnetwork}
\alias{drawMultiWGCNAnetwork}
\title{Draw multiWGCNA network}
\usage{
drawMultiWGCNAnetwork(
  WGCNAlist,
  comparisonList,
  moduleOfInterest,
  design,
  overlapCutoff = 0,
  padjCutoff = 1,
  removeOutliers = TRUE,
  alpha = 1e-50,
  layout = NULL,
  hjust = 0.4,
  vjust = 0.3,
  width = 0.5,
  colors = NULL
)
}
\arguments{
\item{WGCNAlist}{list of WGCNA objects}

\item{comparisonList}{the list of overlap comparisons ie from iterate(myNetworks, overlapComparisons, ...)}

\item{moduleOfInterest}{module of interest, ie "combined_001"}

\item{design}{the sampleTable design matrix}

\item{overlapCutoff}{cutoff to remove module correspondences with less than this number of genes}

\item{padjCutoff}{cutoff to remove module correspondences above this significance value}

\item{removeOutliers}{remove outlier modules?}

\item{alpha}{alpha level of significance}

\item{layout}{layout of network to be passed to plot function of igraph object, defaults to multiWGCNA custom layout}

\item{hjust}{horizontal justification of labels}

\item{vjust}{vertical justification of labels}

\item{width}{width of labels}

\item{colors}{colors to use for modules, should be the same length as the number of WGCNA objects in the WGCNAlist. Defaults to random colors for each condition.}
}
\value{
an igraph plot
}
\description{
Draw a network where nodes are modules and edges represent significant gene overlap. 
Modules are sorted by levels 1, 2, and 3.
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_se = eh_query[["EH8223"]]
sampleTable = colData(astrocyte_se)
astrocyte_networks = eh_query[["EH8222"]]
results = list()
results$overlaps = iterate(astrocyte_networks, overlapComparisons, plot=FALSE)
drawMultiWGCNAnetwork(astrocyte_networks, 
  results$overlaps, 
  "combined_013", 
  sampleTable)
  
}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlaps.R
\name{bidirectionalBestMatches}
\alias{bidirectionalBestMatches}
\title{Best matching modules}
\usage{
bidirectionalBestMatches(comparisonList, plot = TRUE)
}
\arguments{
\item{comparisonList}{a list with an elemnt "overlap", which is a data.frame resulting from a call to computeOverlapsFromWGCNA}

\item{plot}{whether to generate a heatmap; default is TRUE}
}
\value{
A ggplot object
}
\description{
Find all the modules from dataset1 that have a best match to a module in dataset2
if that module in dataset2 is also a best match to the module in dataset1
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]]
comparisonList = list()
comparisonList$overlaps = computeOverlapsFromWGCNA(astrocyte_networks$EAE, astrocyte_networks$WT)
bidirectionalBestMatches(comparisonList)

}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runWGCNA.R
\name{constructNetworks}
\alias{constructNetworks}
\title{constructNetworks: Construct all the weighted gene correlation networks}
\usage{
constructNetworks(
  datExpr,
  sampleTable,
  conditions1,
  conditions2,
  write = FALSE,
  alphaLevel = 0.05,
  plot = FALSE,
  detectNumbers = TRUE,
  ...
)
}
\arguments{
\item{datExpr}{either a SummarizedExperiment object or data.frame with genes are rows and samples as columns}

\item{sampleTable}{data.frame with sample names in first column and sample traits in the second and third column. First column should be called "Sample"}

\item{conditions1}{first design conditions, ie healthy/disease}

\item{conditions2}{second design conditions, ie frontal lobe/temporal lobe}

\item{write}{write results out to files?}

\item{alphaLevel}{significance value passed to findBestTrait function, default is 0.05}

\item{plot}{plot modules? Default is false}

\item{detectNumbers}{passed to makeTraitTable2; if you have any numeric traits, make sure this is TRUE so that these get detected.}

\item{...}{Arguments to pass to blockwiseModules function}
}
\value{
A list of WGCNA objects, ie level one, two, and three networks.
}
\description{
A high level function that returns all networks
possible for a given experimental design
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
autism_se = eh_query[["EH8219"]]
set.seed(1)
autism_se = autism_se[sample(rownames(autism_se), 500),]
sampleTable = colData(autism_se)
conditions1 = unique(sampleTable[,2])
conditions2 = unique(sampleTable[,3])
autism_networks = constructNetworks(autism_se, sampleTable, conditions1[[1]], conditions2[[1]], 
  networkType = "signed", TOMType = "unsigned", 
  power = 10, minModuleSize = 100, maxBlockSize = 25000,
  reassignThreshold = 0, minKMEtoStay = 0, mergeCutHeight = 0,
  numericLabels = TRUE, pamRespectsDendro = FALSE, 
  deepSplit = 4, verbose = 3)
autism_networks[["combined"]]

}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffModuleExpAnalysis.R
\name{runDME}
\alias{runDME}
\title{Run differential module expression}
\usage{
runDME(
  WGCNAobject,
  design,
  alphaLevel = 0.05,
  testCondition = NULL,
  refCondition = NULL,
  p.adjust = "fdr",
  plot = FALSE,
  test = c("ANOVA", "PERMANOVA"),
  write = FALSE,
  out = NULL
)
}
\arguments{
\item{WGCNAobject}{object of class WGCNA with the modules to run DME on}

\item{design}{the sampleTable}

\item{alphaLevel}{level of significance}

\item{testCondition}{the column of the sampleTable to be resolved}

\item{refCondition}{the column of the sampleTable to be used as biological variation}

\item{p.adjust}{adjust for multiple comparisons, argument to pass to p.adjust function}

\item{plot}{generate a plot?}

\item{test}{statistical test to perform, either "ANOVA" or "PERMANOVA"}

\item{write}{write results to a file?}

\item{out}{file name for DME plots, only used if write is TRUE}
}
\value{
a data.frame summarizing the results of the analysis
}
\description{
A wrapper to run diffModuleExpression on all the modules in a network
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_se = eh_query[["EH8223"]]
sampleTable = colData(astrocyte_se)
astrocyte_networks = eh_query[["EH8222"]]
runDME(astrocyte_networks[["combined"]], 
  design = sampleTable,
  p.adjust = "fdr", 
  refCondition = "Region", 
  testCondition = "Disease") 

}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlaps.R
\name{ModuleFlowPlot}
\alias{ModuleFlowPlot}
\title{Module sankey diagram}
\usage{
ModuleFlowPlot(
  WGCNAlist,
  comparisonList,
  networks,
  labels = NULL,
  alpha = 1,
  x.scale = 2,
  y.scale = 1,
  width = 0.3,
  height = 0.2,
  color.by = c("trait", "network", "none"),
  color.low = "cyan",
  color.high = "magenta",
  col = NULL,
  my_layout = NULL,
  use.padj = FALSE,
  only.contiguous = TRUE,
  only.signif = TRUE,
  show.legend = TRUE,
  scale.by.size = TRUE,
  spacer = 10,
  label.y = 50,
  label.size = 4,
  base_family = "Helvetica",
  p.adj.threshold = 0.05,
  overlap.threshold = 10
)
}
\arguments{
\item{WGCNAlist}{a data.frame resulting from a call to computeOverlapsFromWGCNA}

\item{comparisonList}{an object of class WGCNA to compare with dataset2}

\item{networks}{an object of class WGCNA to compare with dataset1}

\item{labels}{labels to show under each networks}

\item{alpha}{alpha value for the node tiles, default is 1}

\item{x.scale}{x spacing parameter}

\item{y.scale}{y spacing parameter}

\item{width}{width of the block}

\item{height}{height of the block}

\item{color.by}{color fill for nodes, either 'network', 'trait', or NULL}

\item{color.low}{default is cyan}

\item{color.high}{default is magenta}

\item{col}{color palette for node tiles, default is internal colors function}

\item{my_layout}{pass a custom layout for nodes? Default is null}

\item{use.padj}{use p.adj for edge width, default is false (use gene overlap value)}

\item{only.contiguous}{only show continuous overlaps? Default is TRUE}

\item{only.signif}{plot only significant overlap?}

\item{show.legend}{show the legend?}

\item{scale.by.size}{scale the node height by the size of the module? Default is TRUE.}

\item{spacer}{space between the nodes}

\item{label.y}{adjust the y coordinate for the network labels, default is 1}

\item{label.size}{size of the node labels, default is 4}

\item{base_family}{font family for ggraph}

\item{p.adj.threshold}{adjusted p-value significance threshold for overlap}

\item{overlap.threshold}{threshold for number of genes overlapping}
}
\value{
Returns a ggalluvial diagram comparing two networks
}
\description{
A plotting function that returns a plot
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]]
results = list()
results$overlaps = iterate(astrocyte_networks, overlapComparisons, plot=FALSE)
ModuleFlowPlot(astrocyte_networks, results$overlaps, c('combined',  'EAE', 'WT'))
}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{getModule}
\alias{getModule}
\title{name: Name of WGCNAobject}
\usage{
getModule(WGCNAobject, gene, use = "dynamicLabels")
}
\arguments{
\item{WGCNAobject}{an object of class WGCNA}

\item{gene}{a gene symbol, should be present in WGCNA object}

\item{use}{which module label to use e.g. dynamicLabels or dynamicColors}
}
\value{
A module name
}
\description{
Returns the module name for a given gene
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]]
getModule(astrocyte_networks[[1]], 'Srgn')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{summarizeResults}
\alias{summarizeResults}
\title{summarizeResults: Summarize results from a results list object}
\usage{
summarizeResults(
  myNetworks,
  results,
  alphaLevel = 0.05,
  write = FALSE,
  outputFile = "results.txt"
)
}
\arguments{
\item{myNetworks}{a list of WGCNAobjects}

\item{results}{results list}

\item{alphaLevel}{alpha level of significance}

\item{write}{write to file?}

\item{outputFile}{name of output file, defaults to results.txt}
}
\value{
prints a summary of results from the multiWGCNA analysis
}
\description{
Prints (or writes) a summary of the results from a results list object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{WGCNA-class}
\alias{WGCNA-class}
\title{The WGCNA Class}
\value{
NA
}
\description{
The WGCNA class is the main class used in multiWGCNA to store results from 
a weighted gene co-expression nework analysis. These include the original 
unaltered expression data used as input, connectivity metrics, module 
assignment, input sample conditions, trait
}
\section{Slots}{

\describe{
\item{\code{datExpr}}{The expression data, connectivity data, and module assignment}

\item{\code{conditions}}{A data.frame with integer conditions for WGCNA}

\item{\code{trait}}{A data.frame showing pearson correlation values to traits}

\item{\code{moduleEigengenes}}{A data.frame of module eigengenes for each module across samples}

\item{\code{outlierModules}}{A vector of modules classified by our algorithm as being driven by sample outliers}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{makeTraitTable}
\alias{makeTraitTable}
\title{Generate a trait table from a sample table}
\usage{
makeTraitTable(inputTable, column, detectNumbers = FALSE)
}
\arguments{
\item{inputTable}{the sampleTable data.frame}

\item{column}{the column from the sampleTable to use as traits}

\item{detectNumbers}{whether to consider traits with numbers as numerical rather than categorical variables}
}
\value{
a data.frame with integer values denoting the categorical sample traits
}
\description{
Generates a WGCNA-compatible trait table from a sampleTable dataframe. This 
function is deprecated. Use makeTraitTable2 instead.
}
\examples{
sampleTable = data.frame(Sample = c(paste0("EAE", 1:10), paste0("WT", 1:10)), 
                         Disease = c(rep("EAE", 10), rep("WT", 10)), 
                         Region = c(rep(c("Cbl", "Sc"), 5))) 
makeTraitTable(sampleTable, 2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preservation.R
\name{coexpressionLineGraph}
\alias{coexpressionLineGraph}
\title{Coexpression Line Graph}
\usage{
coexpressionLineGraph(datExpr, splitBy = 1, fontSize = 2.15, colors = NULL)
}
\arguments{
\item{datExpr}{a data.frame with genes as rows and samples as columns}

\item{splitBy}{how much to split genes by on line graph}

\item{fontSize}{the font size of the gene labels}

\item{colors}{a vector of colors; default is random colors generated by colors function}
}
\value{
a ggplot object
}
\description{
Plots a line graph showing the co-expression of selected genes across samples
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]]
datExpr = GetDatExpr(astrocyte_networks[[1]], 
  genes = topNGenes(astrocyte_networks$EAE, "EAE_015", 20))
coexpressionLineGraph(datExpr)	+ 
  geom_vline(xintercept = 20.5, linetype='dashed')

}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preservation.R
\name{diffCoexpression}
\alias{diffCoexpression}
\title{Differential co-expresison analysis}
\usage{
diffCoexpression(
  datExpr,
  conditions,
  geneList = NULL,
  plot = FALSE,
  method = c("pearson", "spearman"),
  removeFreeNodes = TRUE,
  labelSize = 0.5,
  labelDist = 0,
  shape = "circle",
  degreeForSize = FALSE,
  label = FALSE,
  onlyPositive = FALSE,
  z.threshold = NULL,
  FDR.threshold = 0.05,
  nodeSize = 3
)
}
\arguments{
\item{datExpr}{a data.frame containing expression values}

\item{conditions}{a vector containing conditions for the samples}

\item{geneList}{vector of genes, will use all genes if NULL (default)}

\item{plot}{plot a network?}

\item{method}{either "pearson" or "spearman"}

\item{removeFreeNodes}{remove free nodes from network?}

\item{labelSize}{label size}

\item{labelDist}{distance from labels to nodes}

\item{shape}{shape of nodes}

\item{degreeForSize}{should node size correspond to degree?}

\item{label}{label nodes?}

\item{onlyPositive}{only draw positive correlations?}

\item{z.threshold}{z-score threshold}

\item{FDR.threshold}{FDR threshold}

\item{nodeSize}{size of node}
}
\value{
A list including a matrix of z-scores, a matrix of raw p-values, a 
matrix of adjusted p-values, and a summary data.frame
}
\description{
Performs a differential co-expression ananlysis given an expression data.frame 
and a conditions vector
}
\examples{
\donttest{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_se = eh_query[["EH8223"]]
datExpr = assays(astrocyte_se)[[1]]
diffCoexpression(datExpr, c(rep(1,20), rep(2,16)), 
  geneList = c("Gfap", "Vim", "Aspg", "Serpina3n", "Cp", "Osmr", "Cd44", 
               "Cxcl10", "Hspb1", "Timp1", "S1pr3", "Steap4", "Lcn2"))
}
    

}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{GetDatExpr}
\alias{GetDatExpr}
\title{Get expression data}
\usage{
GetDatExpr(object, genes = NULL)
}
\arguments{
\item{object}{An object of class WGCNA}

\item{genes}{a list of genes to subset to; default is NULL}
}
\value{
a data.frame
}
\description{
Returns the expression data frame a WGCNA object as a data.frame
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]]
datExpr = GetDatExpr(astrocyte_networks[[1]], 
  genes = topNGenes(astrocyte_networks$EAE, "EAE_015", 20))
coexpressionLineGraph(datExpr)	+ 
  geom_vline(xintercept = 20.5, linetype='dashed')

}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlaps.R
\name{moduleToModuleHeatmap}
\alias{moduleToModuleHeatmap}
\title{Module to module heatmap}
\usage{
moduleToModuleHeatmap(
  comparisonDf,
  dataset1 = NULL,
  dataset2 = NULL,
  trait1 = NULL,
  trait2 = NULL,
  list1 = NULL,
  list2 = NULL,
  filterByTrait = FALSE,
  alphaLevel = 0.05
)
}
\arguments{
\item{comparisonDf}{the data.frame output of computeOverlapFromWGCNA}

\item{dataset1}{optional; WGCNA object for dataset 1}

\item{dataset2}{optional; WGCNA object for dataset 2}

\item{trait1}{optional; subset to modules correlated to this trait for dataset 1}

\item{trait2}{optional; subset to modules correlated to this trait for dataset 2}

\item{list1}{subset to this list of modules for dataset 1}

\item{list2}{subset to this list of modules for dataset 2}

\item{filterByTrait}{only plot for modules that correlate with some trait?}

\item{alphaLevel}{the alpha level of significance for module-trait correlation, defaults to 0.05}
}
\value{
A ggplot object
}
\description{
Returns a heatmap where color corresponds to FDR-adjusted overlap (hypergeometric test) and the label corresponds to the number of overlapping genes
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]]
overlapDf = computeOverlapsFromWGCNA(astrocyte_networks$EAE, astrocyte_networks$WT)
moduleToModuleHeatmap(overlapDf)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{permutationTestResults}
\alias{permutationTestResults}
\title{Permutation test results}
\format{
A list of data.frames containing preservation results for each 
permutation
}
\usage{
data(permutationTestResults)
}
\description{
The results of running the PreservationPermutationTest in the astrocyte
vignette. This is provided since this function is quite slow. Please see
the astrocyte vignette for more details.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlaps.R
\name{ModuleFlowPlot2Way}
\alias{ModuleFlowPlot2Way}
\title{Module sankey diagram}
\usage{
ModuleFlowPlot2Way(
  overlapDf,
  dataset1,
  dataset2,
  only.signif = TRUE,
  show.legend = TRUE,
  col = NULL,
  ...
)
}
\arguments{
\item{overlapDf}{a data.frame resulting from a call to computeOverlapsFromWGCNA}

\item{dataset1}{an object of class WGCNA to compare with dataset2}

\item{dataset2}{an object of class WGCNA to compare with dataset1}

\item{only.signif}{plot only significant overlap?}

\item{show.legend}{show the legend?}

\item{col}{color palette, default is internal colors function}

\item{...}{params to GetSignificantOverlap function for thresholds}
}
\value{
Returns a ggalluvial diagram comparing two networks
}
\description{
A plotting function that returns a sankey plot comparing two networks. 
This is a deprecated function, please use ModuleFlowPlot instead.
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]]
overlapDf = computeOverlapsFromWGCNA(astrocyte_networks$EAE, astrocyte_networks$WT)
ModuleFlowPlot2Way(overlapDf, astrocyte_networks$EAE, astrocyte_networks$WT)
}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlaps.R
\name{GetSignificantOverlap}
\alias{GetSignificantOverlap}
\title{Get significant overlap}
\usage{
GetSignificantOverlap(df, p.adj.threshold = 0.05, overlap.threshold = 10)
}
\arguments{
\item{df}{a data.frame resulting from a call to computeOverlapsFromWGCNA}

\item{p.adj.threshold}{an object of class WGCNA to compare with dataset2}

\item{overlap.threshold}{an object of class WGCNA to compare with dataset1}
}
\value{
Returns a data.frame with significant overlaps
}
\description{
A function that subsets an overlap comaprisons data.frame.
}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preservation.R
\name{getPreservation}
\alias{getPreservation}
\title{getPreservation}
\usage{
getPreservation(reference, test, nPermutations = 100, write = FALSE)
}
\arguments{
\item{reference}{reference network of class WGCNA}

\item{test}{test network of class WGCNA}

\item{nPermutations}{number of permutations to perform; at least 50 permutations}

\item{write}{write to file?}
}
\value{
a data.frame summarizing results of preservation analysis
}
\description{
Performs a network preservation analysis
}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preservation.R
\name{preservationComparisons}
\alias{preservationComparisons}
\title{Preservation comparisons}
\usage{
preservationComparisons(
  comparisonList,
  WGCNAlist,
  first,
  second,
  element,
  plot = FALSE,
  write = FALSE,
  alphaLevel = 0.05,
  nPermutations = 100
)
}
\arguments{
\item{comparisonList}{a list passed by the iterate function}

\item{WGCNAlist}{list of objects of type WGCNAobject}

\item{first}{index of first WGCNAobject}

\item{second}{index of second WGCNAobject}

\item{element}{element position in the comparison list (passed by iterate function)}

\item{plot}{generate plots?}

\item{write}{write results to file?}

\item{alphaLevel}{alpha level of significance for module-trait correlation}

\item{nPermutations}{number of permutations, defaults to 100}
}
\value{
a list of preservation comparisons results across levels 1, 2, 3
}
\description{
A high level function that performs a perservation comparison between two
WGCNAobjects in a WGCNAlist, usually supplied by iterate function
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]]
results = list()
iterate(astrocyte_networks[c("EAE", "WT")], 
  preservationComparisons, 
  write=FALSE, 
  plot=FALSE, 
  nPermutations=2)

}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffModuleExpAnalysis.R
\name{diffModuleExpression}
\alias{diffModuleExpression}
\title{Differential module expression}
\usage{
diffModuleExpression(
  WGCNAobject,
  geneList,
  design,
  plotTitle = NULL,
  mode = c("PC1", "Zscore"),
  testColumn = 2,
  refColumn = 3,
  test = c("ANOVA", "PERMANOVA"),
  plot = TRUE
)
}
\arguments{
\item{WGCNAobject}{WGCNA object}

\item{geneList}{vector of genes in WGCNAobject}

\item{design}{the sampleTable}

\item{plotTitle}{title for the plot}

\item{mode}{either PC1 or Zscore, default is PC1}

\item{testColumn}{the column of the sampleTable to be resolved}

\item{refColumn}{the column of the sampleTable to be used as biological variation}

\item{test}{statistical test to perform, either "ANOVA" or "PERMANOVA"}

\item{plot}{generate a plot?}
}
\value{
a data.frame with the resulting p-values
}
\description{
Runs (and plots) the differential module expression analysis
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_se = eh_query[["EH8223"]]
sampleTable = colData(astrocyte_se)
astrocyte_networks = eh_query[["EH8222"]]
diffModuleExpression(astrocyte_networks[["combined"]], 
  topNGenes(astrocyte_networks$combined, "combined_013"), 
  sampleTable,
  test = "ANOVA",
  plotTitle = "combined_013",
  plot = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlaps.R
\name{PlotMultiNodesTOMflow}
\alias{PlotMultiNodesTOMflow}
\title{PlotMultiNodesTOMflow}
\usage{
PlotMultiNodesTOMflow(
  TOMDF,
  labels = NULL,
  alpha = 0.1,
  width = 0.05,
  color = "black"
)
}
\arguments{
\item{TOMDF}{created by BuildTOMFlowDF}

\item{labels}{labels to use for the networks, default is Network1, Network2, etc.}

\item{alpha}{alpha of flows}

\item{width}{width of the strata}

\item{color}{color of flows}
}
\value{
a ggplot object
}
\description{
Plots a sankey flow diagram showing the movement of genes from one WGCNA
to multi-WGCNA networks. Uses the ggalluvial framework.
}
\author{
Dario Tommasini, Xinye Li
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlaps.R
\name{moduleComparisonPlot}
\alias{moduleComparisonPlot}
\title{Module comparison plot}
\usage{
moduleComparisonPlot(overlapDf, dataset1, dataset2)
}
\arguments{
\item{overlapDf}{a data.frame resulting from a call to computeOverlapsFromWGCNA}

\item{dataset1}{an object of class WGCNA to compare with dataset2}

\item{dataset2}{an object of class WGCNA to compare with dataset1}
}
\value{
Returns a ggplot object (flowplot and heatmap) showing the module correspondence 
between two objects of class WGCNA
}
\description{
A plotting function that returns a heatmap and barplot for a module
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]]
overlapDf = computeOverlapsFromWGCNA(astrocyte_networks$EAE, astrocyte_networks$WT)
moduleComparisonPlot(overlapDf, astrocyte_networks$EAE, astrocyte_networks$WT)

}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preservation.R
\name{PreservationScoreDistribution}
\alias{PreservationScoreDistribution}
\title{PreservationScoreDistribution}
\usage{
PreservationScoreDistribution(preservationData, moduleOfInterestSize)
}
\arguments{
\item{preservationData}{the results from PreservationPermutationTest}

\item{moduleOfInterestSize}{the number of genes in your module of interest}
}
\value{
A data.frame with Z-summary preservation scores of the module from
each permutation and the corresponding module size
}
\description{
Extracts the preservation score distribution from the results of 
PreservationPermutationTest.
}
\examples{
# Remove outlier modules
permutationTestResultsFiltered = lapply(permutationTestResults, function(x) 
x[!x$is.outlier.module,])

# Find preservation score distribution for a given module size
scores.summary = PreservationScoreDistribution(
permutationTestResultsFiltered, 
moduleOfInterestSize = 303)
                                                        
}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlaps.R
\name{computeOverlapsFromWGCNA}
\alias{computeOverlapsFromWGCNA}
\title{computeOverlapsFromWGCNA}
\usage{
computeOverlapsFromWGCNA(dataset1, dataset2)
}
\arguments{
\item{dataset1}{an object of class WGCNA to compare with dataset2}

\item{dataset2}{an object of class WGCNA to compare with dataset1}
}
\value{
Returns a data.frame showing the overlap results for modules from 
dataset1 with dataset2
}
\description{
Computes overlap between the modules of two objects of class WGCNA
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]]
computeOverlapsFromWGCNA(astrocyte_networks$EAE, astrocyte_networks$WT)

}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlaps.R
\name{overlapComparisons}
\alias{overlapComparisons}
\title{Overlap comparisons}
\usage{
overlapComparisons(
  comparisonList,
  WGCNAlist,
  first,
  second,
  element,
  plot = TRUE,
  write = FALSE
)
}
\arguments{
\item{comparisonList}{a list passed by the iterate function}

\item{WGCNAlist}{list of objects of class WGCNA}

\item{first}{index of first WGCNA object}

\item{second}{index of second WGCNA object}

\item{element}{element position in the comparison list (passed by iterate function)}

\item{plot}{generate plots?}

\item{write}{write results to file?}
}
\value{
A list, in which the first element is a data.frame showing the 
overlap results and the second element is a data.frame showing the best 
matching modules between the two WGCNA objects.
}
\description{
Compares modules between two objects of type WGCNAobjects
within a WGCNAobject list given the indices. Recommended to be used in 
conjunction with the iterate function.
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]]
results = list()
results$overlaps = iterate(astrocyte_networks, overlapComparisons, plot=FALSE)

}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preservation.R
\name{preservationComparisonPlot}
\alias{preservationComparisonPlot}
\title{Preservation Comparison Scatterplot}
\usage{
preservationComparisonPlot(
  preservationList,
  dataset1,
  dataset2,
  alphaLevel = 0.05,
  outliers = FALSE
)
}
\arguments{
\item{preservationList}{a list resulting from a call to preservationComparisons}

\item{dataset1}{an object of class WGCNAobject to compare with dataset2}

\item{dataset2}{an object of class WGCNAobject to compare with dataset1}

\item{alphaLevel}{alpha level of significance, default is 0.05}

\item{outliers}{leave outlier modules? By default these are removed}
}
\value{
a ggplot object
}
\description{
A plotting  function that draws a scatterplot of preservation scores between
two WGCNA objects
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]]
results = list()
results$preservation=iterate(astrocyte_networks[c("EAE", "WT")], 
  preservationComparisons, 
  write=FALSE, 
  plot=FALSE, 
  nPermutations=2)
preservationComparisonPlot(results$preservation$EAE_vs_WT, 
  astrocyte_networks$EAE, 
  astrocyte_networks$WT)
  
}
\author{
Dario Tommasini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generalFunctions.R
\name{topNGenes}
\alias{topNGenes}
\title{topNGenes: Top N genes of a module}
\usage{
topNGenes(WGCNAobject, module, nGenes = NULL)
}
\arguments{
\item{WGCNAobject}{an object of class WGCNA}

\item{module}{the name of the module in WGCNAobject}

\item{nGenes}{an integer from 1 to module size; returns all genes if left NULL}
}
\value{
a character vector of the genes/probes in the module
}
\description{
Returns the top N number of genes of a module.
All genes returned if no number is specified.
Genes are in order of intramodular connectivity.
}
\examples{
library(ExperimentHub)
eh = ExperimentHub()
eh_query = query(eh, c("multiWGCNAdata"))
astrocyte_networks = eh_query[["EH8222"]]
topNGenes(astrocyte_networks$EAE, "EAE_015", nGenes = 10)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlaps.R
\name{TOMFlowPlot}
\alias{TOMFlowPlot}
\title{TOMFlowPlot}
\usage{
TOMFlowPlot(
  WGCNAlist,
  networks,
  toms,
  genes_to_label,
  alpha = 0.1,
  color = "black",
  width = 0.05
)
}
\arguments{
\item{WGCNAlist}{list of WGCNA objects}

\item{networks}{list of network names of length 2}

\item{toms}{a list of TOM distance objects of length 2}

\item{genes_to_label}{genes to label across two networks}

\item{alpha}{alpha of flows}

\item{color}{color of flows}

\item{width}{width of the strata}
}
\value{
a ggplot object
}
\description{
Plots a sankey flow diagram showing the movement of genes from one WGCNA
to another WGCNA. Uses the ggalluvial framework.
}
\author{
Dario Tommasini
}
